#include "ThreasholdDlg.h"

Threashold::Threashold( GLIMSProject *proj ) :
	QDialog( NULL, "", true )
{
    mProj = proj;
    buildDialog();
    initGUI();
}

void Threashold::buildDialog( )
{
	//GLIMSDataset* data = mProj->getDataset();
	GLIMSLineData linedata;// = data->getLineData();

    QHBox *hbox;
    QGroupBox *gbox;
    QPushButton *pbut;
    QVBoxLayout *main = new QVBoxLayout( this );
    main->setAutoAdd( true );
    main->setMargin( 10 );
	
    mPrev = new Preview( this, mProj->getImage(), 0 );
	
    hbox = new QHBox( this );
    new QLabel( "Band To Process: ", hbox );
    mBandBox = new QComboBox( hbox );
    hbox->setMaximumHeight( 45 );
    connect( mBandBox,
             SIGNAL( activated( int ) ),
             this,
             SLOT( bandSwitched( int ) ) );

    gbox = new QGroupBox( 2, Qt::Horizontal, "Upper Threshold", this );
    gbox->setMaximumHeight( 45 );
    mUpper = new QSlider( Qt::Horizontal, gbox );
    mLblUVal = new QLabel( gbox );
    mLblUVal->setMinimumWidth( 50 );
    mLblUVal->setMaximumWidth( 50 );
    connect( mUpper,
             SIGNAL( valueChanged( int ) ),
             this,
             SLOT( upperChanged( int ) ) );

    gbox = new QGroupBox( 2, Qt::Horizontal, "Lower Threshold", this );
    gbox->setMaximumHeight( 45 );
    mLower = new QSlider( Qt::Horizontal, gbox );
    mLblLVal = new QLabel( gbox );
    mLblLVal->setMinimumWidth( 50 );
    mLblLVal->setMaximumWidth( 50 );
    connect( mLower,
             SIGNAL( valueChanged( int ) ),
             this,
             SLOT( lowerChanged( int ) ) );

    hbox = new QHBox( this );
    hbox->setMaximumHeight( 50 );
    pbut = new QPushButton( "Proceed", hbox );
    connect( pbut,
             SIGNAL( clicked() ),
             this,
             SLOT( proceedClicked() ) );

    pbut = new QPushButton( "Cancel", hbox );
    connect( pbut,
             SIGNAL( clicked() ),
             this,
             SLOT( close() ) );
}

void Threashold::proceedClicked( )
{
    mPrev->getSel( mSelX, mSelY, mSelW, mSelH );
    std::string fname = createClassMap();
    smoothClassMap( fname, mSelW, mSelH );
	delete mStatDlg;
    rasterToVector( fname );

    mProj->getViewSet()->repaintViews();

	hide();
    show();
}

void Threashold::initGUI( )
{
    Image *img = mProj->getImage();
    std::vector<ImageFormat::BandInf> binf = img->getBandInfSet();

    for( int i=0; i < (int)binf.size(); i++ )
        mBandBox->insertItem( binf[i].name.c_str() );

    bandSwitched( 0 );
}

void Threashold::bandSwitched( int isel )
{
    Image *img = mProj->getImage();
    
    std::vector<ImageFormat::BandInf> &binf = img->getBandInfSet();

    if( isel > (int)binf.size() )
        return;

    mLower->setMinValue( (int)binf[isel].min );
    mLower->setMaxValue( (int)binf[isel].max );
    mUpper->setMinValue( (int)binf[isel].min );
    mUpper->setMaxValue( (int)binf[isel].max );

    mUpper->setValue( (int)binf[isel].max );
    mLower->setValue( (int)binf[isel].min );

    mPrev->setBand( isel );
}

void Threashold::upperChanged( int val )
{
    char buf[10];
    sprintf( buf, "%d", val );
    mLblUVal->setText( buf );
    mIUpper = val;
    mPrev->setHigh( val );
}

void Threashold::lowerChanged( int val )
{
    char buf[10];
    sprintf( buf, "%d", val );
    mLblLVal->setText( buf );
    mILower = val;
    mPrev->setLow( val );
}


/** 
* \todo 2006-01-06 appears this was designed to export resulting thresholded image to a file, but not implemented
*/
std::string Threashold::createClassMap( )
{
    const char SNOW  = (char)255;
    const char OTHER = (char)0;

    int     width;
    int     height;
    int     iband;
    void    *line;
    double  pcnt;
    void    *data;
    int     dtsz;
    
    Image       *img = mProj->getImage();
    ImageFormat *frmt = img->getFormat();

    std::vector<ImageFormat::BandInf> &binf = img->getBandInfSet();

    iband  = mBandBox->currentItem();
    if( iband >= (int)binf.size() )
        return "";

    if( ImageFormat::isFloat( binf[iband].type ) )
    {
        QMessageBox::information( NULL, 
                                  "Class Map Datatype Error", 
                                  "This band is of float datatype which is unusable with this tool." );
        return "";
    }

    if( mIUpper <= mILower )
    {
        QMessageBox::information( NULL, 
                                  "Class Map Settings Error", 
                                  "Upper must be greater than lower" );
        return "";
    }

    QString fname = "";
    
    /*
    QFileDialog::getSaveFileName();
    if( fname.isNull() )
        return "";

    ofstream out( (const char*)fname );
    if( !out )
    {
        QMessageBox::information( NULL, 
                                  "Class Map File Error", 
                                  "Could not save to choosen file" );
        return "";
    }
    */

    height = binf[iband].height;
    width  = binf[iband].width;

	mStatDlg = new StatusDlg( "Creating Class Map" );
	mStatDlg->show();
    mStatDlg->setVal( 0 );
    pcnt = 0.0;
    // data = (void*)frmt->getBand( iband, mSelX, mSelY, mSelW, mSelH, mSelW, mSelH );
    dtsz = ImageFormat::dtypeSize( binf[iband].type );

    mCMap = new unsigned char[ mSelW * mSelH ];
    for( int y=0; y < mSelH; y++ )
    {
        line = frmt->getBand( iband, mSelX, y+mSelY, mSelW, 1, mSelW, 1 );
        // line = &(((char*)data)[ (mSelY+y)*mSelW*dtsz + mSelX*dtsz ]);
        for( int x=0; x < mSelW; x++ )
        {
            bool isIn = false;
            int dx = x * dtsz;
            switch( binf[iband].type )
            {
            case ImageFormat::INT8:
                if( mIUpper >= ((char*)line)[dx] &&
                    mILower <= ((char*)line)[dx] )
                    isIn = true;
                break;
        
            case ImageFormat::UINT8:
                if( mIUpper >= ((unsigned char*)line)[dx] &&
                    mILower <= ((unsigned char*)line)[dx] )
                    isIn = true;
                break;
            
            case ImageFormat::INT16:
                if( mIUpper >= ((short*)line)[dx] &&
                    mILower <= ((short*)line)[dx] )
                    isIn = true;
                break;
        
            case ImageFormat::UINT16:
                if( mIUpper >= ((unsigned short*)line)[dx] &&
                    mILower <= ((unsigned short*)line)[dx] )
                    isIn = true;
                break;
        
            case ImageFormat::INT32:
                if( mIUpper >= ((int*)line)[dx] &&
                    mILower <= ((int*)line)[dx] )
                    isIn = true;
                break;
        
            case ImageFormat::UINT32:
                if( mIUpper >= ((unsigned int*)line)[dx] &&
                    mILower <= ((unsigned int*)line)[dx] )
                    isIn = true;
                break;
        
            case ImageFormat::INT64:
                if( mIUpper >= ((long*)line)[dx] &&
                    mILower <= ((long*)line)[dx] )
                    isIn = true;
                break;
        
            case ImageFormat::UINT64:
                if( mIUpper >= ((unsigned long*)line)[dx] &&
                    mILower <= ((unsigned long*)line)[dx] )
                    isIn = true;
                break;
        
            default:
                break;
            }

            if( isIn )
                mCMap[ y*mSelW + x ] = SNOW;
            else
                mCMap[ y*mSelW + x ] = OTHER;
        }

        switch( dtsz )
        {
        case 1:
            delete [] (char*)line;
            break;
    
        case 2:
            delete [] (short*)line;
            break;
    
        case 4:
            delete [] (float*)line;
            break;
    
        case 8:
            delete [] (double*)line;
            break;
        }

        pcnt = (double)y / (double)mSelH;
        mStatDlg->setVal( (int)(pcnt * 100.0) );
    }
   
    return (const char*)fname;
}

void Threashold::rasterToVector( std::string fname )
{
    const char SNOW  = (char)255;
    const char OTHER = (char)0;

    int     width;
    int     height;
    int     iband;
	int		iline;
    void    *line;
    
    Image       *img = mProj->getImage();
    ImageFormat *frmt = img->getFormat();

    std::vector<ImageFormat::BandInf> &binf = img->getBandInfSet();

    iband  = mBandBox->currentItem();
    if( iband >= (int)binf.size() )
        return;

    height = binf[iband].height;
    width  = binf[iband].width;

    char *map = genEdgePoints( mSelW, mSelH, (unsigned char)255 );
    
    std::vector< std::vector<R2VPoint> > lines =
        buildLines( map, mSelW, mSelH, (char)255 );

    delete [] map;

    cleanLines( lines );

    int pntcnt = 0;

    std::vector<GlacierLine> glineset;
    for( iline=0; iline < (int)lines.size(); iline++ )
    {
        std::vector<R2VPoint> &line = lines[iline];
        //if( (int)line.size() <= 1 ) continue;
        GlacierLine gline;
        std::vector<Node> &ns = gline.nodeset();
        pntcnt += line.size();
        Node n;
        n.x = line[0].x + mSelX;
        n.y = line[0].y + mSelY;
        frmt->getLL( iband, n.x, n.y );
        ns.push_back( n );
        int ipnt = 1;
        for( ; ipnt < (int)line.size()-1; ipnt+=4 )
        {
            n.x = line[ipnt].x + mSelX;
            n.y = line[ipnt].y + mSelY;
            frmt->getLL( iband, n.x, n.y );
            ns.push_back( n );
        }

        n.x = line[line.size()-1].x + mSelX;
        n.y = line[line.size()-1].y + mSelY;
        frmt->getLL( iband, n.x, n.y );
        ns.push_back( n );
        
        glineset.push_back( gline );
    }
     
    GLIMSLineData &ldata = mProj->getDataset()->getLineData();
    ldata.addLineSetLL( glineset );
}

char *
Threashold::genEdgePoints( int w, 
                           int h, 
                           unsigned char DN )
{
	int y;
    char *map = new char[ w * h ];
    unsigned char *p1line, *p2line, *p3line;


    for( y=1; y < h-2; y++ )
    {
        // SHIFT LINES UP 1
        /*
        memcpy( p3line, &p3line[ w+2 ], w+2 );
        memcpy( &p3line[ w+2 ], &p3line[ (w+2) * 2 ], w+2 );
        in.read( &p3line[ (w+1) * 2 ], w );
        */

        p1line = &mCMap[ (y-1) * w ];
        p2line = &mCMap[ y     * w ];
        p3line = &mCMap[ (y+1) * w ];

        for( int x=1; x < w-1; x++ )
        {
            if( p2line[ x ] == DN )
            {
                int sum = p1line[ x-1 ] + // UL
                          p1line[ x   ] + // UM
                          p1line[ x+1 ] + // UR
                          p2line[ x-1 ] + // L
                          p2line[ x+1 ] + // R
                          p3line[ x-1 ] + // LL
                          p3line[ x   ] + // LM
                          p3line[ x+1 ];  // LR
                if( sum != ( DN*8 ) )
                {
                    // FOUND AN EDGE POINT
                    map[ y*w + x ] = DN;
                }
            }
        }
    }

    for( y=1; y < h-1; y++ )
    {
        for( int x=0; x < w-3; x++ )
        {
            int ypos   = y*w;

            if( map[ ypos + x ] == (char)0 &&
                map[ ypos + x+1 ] == (char)255 &&
                map[ ypos + x+2 ] == (char)255 &&
                map[ ypos + x+3 ] == (char)0 )
            {
                map[ ypos + x+1 ] = (char)0;
            }
        }
    }

    return map;
}

std::vector< std::vector<R2VPoint> >  
Threashold::buildLines( char *map, int w, int h, char DN )
{
    std::vector< std::vector<R2VPoint> > lines;

    for( int y=0; y < h; y++ )
    {
        for( int x=0; x < w; x++ )
        {
            if( map[ y*w + x ] == DN )
            {
                std::vector<R2VPoint> line;
                int i = x;
                int j = y;
                bool hasAdj = true;
                
                while( hasAdj )
                {
                    R2VPoint pnt = { i, j };
                    line.push_back( pnt );
                    
                    map[ j*w + i ] = (char)0;
                    hasAdj = false;

                    // CHECK ADJ POINTS
                    if( j > 0 && i > 0 && map[ (j-1)*w + i-1 ] == DN )
                    {
                        i = i-1;
                        j = j-1;
                        hasAdj = true;
                        continue;
                    }

                    if( j > 0 && map[ (j-1)*w + i ] == DN )
                    {
                        j = j-1;
                        hasAdj = true;
                        continue;
                    }

                    if( j > 0 && i < w-1 && map[ (j-1)*w + i+1 ] == DN )
                    {
                        i = i+1;
                        j = j-1;
                        hasAdj = true;
                        continue;
                    }

                    if( i > 0 && map[ j*w + i-1 ] == DN )
                    {
                        i = i-1;
                        hasAdj = true;
                        continue;
                    }

                    if( i < w-1 && map[ j*w + i+1 ] == DN )
                    {
                        i = i+1;
                        hasAdj = true;
                        continue;
                    }

                    if( j < h-1 && i > 0 && map[ (j+1)*w + i-1 ] == DN )
                    {
                        i = i-1;
                        j = j+1;
                        hasAdj = true;
                        continue;
                    }

                    if( j < h-1 && map[ (j+1)*w + i ] == DN )
                    {
                        j = j+1;
                        hasAdj = true;
                        continue;
                    }

                    if( j < h-1 && i > 0 && map[ (j+1)*w + i+1 ] == DN )
                    {
                        i = i+1;
                        j = j+1;
                        hasAdj = true;
                        continue;
                    }                    
                }

                lines.push_back( line );
            }
        }
    }

    return lines;
}

void Threashold::cleanLines( 
    std::vector< std::vector<R2VPoint> > &lines )
{
    enum { UL, UM, UR, ML, MR, LL, LM, LR };
    int prevdir;
    int curdir;
    bool    clean = false;
    int start, cnt;
    R2VPoint prev, cur;

    for( int iline=0; iline < (int)lines.size(); iline++ )
    {
        std::vector<R2VPoint> &line = lines[iline];
        prev    = line[0];
        prevdir = -1;
        start   = 0;
        cnt     = 0;
        for( int ipnt=1; ipnt < (int)line.size(); ipnt++ )
        {
            cur = line[ipnt];
            if( cur.x < prev.x && cur.y < prev.y )
                curdir = UL;
            else if( cur.x == prev.x && cur.y < prev.y )
                curdir = UM;
            else if( cur.x > prev.x && cur.y < prev.y )
                curdir = UR;
            else if( cur.x < prev.x && cur.y == prev.y )
                curdir = ML;
            else if( cur.x > prev.x && cur.y == prev.y )
                curdir = MR;
            else if( cur.x < prev.x && cur.y > prev.y )
                curdir = LL;
            else if( cur.x == prev.x && cur.y > prev.y )
                curdir = LM;
            else if( cur.x > prev.x && cur.y > prev.y )
                curdir = LR;
            else 
                clean = true;

            if( curdir == prevdir )
            {
                cnt++;
                clean = false;
            } else {
                clean = true;
            }

            if( clean )
            {
                if( cnt > 1 )
                {
                    line.erase( line.begin() + start,
                                line.begin() + start + cnt );
                    ipnt -= (cnt-2);
                }
                start = ipnt;
                cnt = 0;
            }
            prevdir = curdir;
            prev = cur;
        }
    }
}

void Threashold::smoothClassMap( std::string fname, int w, int h )
{
    const int NITERATIONS = 50;
    double pcnt;
    unsigned char *map = mCMap;

    mStatDlg->setMsg( "Smoothing Class Map" );
    mStatDlg->setVal( 0 );
    pcnt = 0.0;
    for( int it=0; it < NITERATIONS; it++ )
    {
        for( int y=1; y < h-1; y++ )
        {
            for( int x=1; x < w-1; x++ )
            {
                int yposm1 = (y-1)*w;
                int ypos   = y*w;
                int yposp1 = (y+1)*w;
    
                int val = map[ yposm1 + x-1 ] +
                          map[ yposm1 + x ]   +
                          map[ yposm1 + x+1 ] +
                          map[ ypos + x-1 ]   +
                          map[ ypos + x+1 ]   +
                          map[ yposp1 + x-1 ] +
                          map[ yposp1 + x ]   +
                          map[ yposp1 + x+1 ];
    
                if( map[ ypos + x ] == 255 && val <= 255*3 )
                    map[ ypos + x ] = 0;
                else if( val >= (255*5) )
                    map[ ypos + x ] = 255;
            }
            pcnt = (double)(h*it + y) / (double)(NITERATIONS*h);
            mStatDlg->setVal( (int)( pcnt*100.0 ) );
        }
    }
}



