#ifndef __THREASHOLDDLG_H
#define __THREASHOLDDLG_H

#include <qdialog.h>
#include <qslider.h>
#include <qpushbutton.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qlayout.h>
#include <qhbox.h>

#include <glimsproject.h>

#include <iostream>
#include <vector>
#include <fstream>

#include "preview.h"
#include "statusdlg.h"

typedef struct {
    unsigned short  x;
    unsigned short  y;
} R2VPoint;


/**
 * The %Threshhold class implements the primary dialog for setting
 * the threshhold value and starting the automatic polygon generation.
 *
 * \ingroup threshholdPlugin
 */
class Threashold : public QDialog
{
Q_OBJECT
protected:
    QSlider     *mUpper;
    QSlider     *mLower;
    int         mIUpper;
    int         mILower;

    int         mSelX;
    int         mSelY;
    int         mSelW;
    int         mSelH;

    QLabel      *mLblLVal;
    QLabel      *mLblUVal;
    QComboBox   *mBandBox;

    StatusDlg   *mStatDlg;

    unsigned char *mCMap;

    GLIMSProject *mProj;

	GLIMSLineData linedata;

    Preview      *mPrev;

    void buildDialog( );
    void initGUI( );
    std::string createClassMap( );
    void rasterToVector( std::string fname );

    char * genEdgePoints( int w, int h, unsigned char DN );

     std::vector< std::vector<R2VPoint> >
         buildLines( char *map, int w, int h, char DN );

     void cleanLines( std::vector< std::vector<R2VPoint> > &lines );
     void smoothClassMap( std::string fname, int w, int h );

public:
    Threashold( GLIMSProject *proj );

public slots:
    void proceedClicked( );
    void bandSwitched( int );

    void upperChanged( int );
    void lowerChanged( int );
};

#endif

