#include "preview.h"
#include <qmessagebox.h>

Preview::Preview( QWidget *parent, Image *img, int band ) :
    QWidget( parent ),
	mImg( NULL ),
    mBand( band )
{
	mImg = new Image( img->getFormat() );
    
    setMouseTracking( true );
    setMinimumSize( 200, 200 );
    setBackgroundColor( QColor( 0,0,0 ) );

    mLow  = 0;
    mHigh = 255;

    mBoxMov = false;
    mBoxRes = false;

    setBand( band );

    setBox( 0, 0, 100, 100 );
}

void Preview::getRaw( )
{
    const int WIDTH  = width();
    const int HEIGHT = height();

    int sw, sh;
    int iw, ih;

	if( mImg == NULL )
		return;

    iw = mImg->width();
    ih = mImg->height();

	if( ( iw + ih + WIDTH + HEIGHT ) < 4 )
		return;

    if( ((double)HEIGHT / (double)ih) < 
        ((double)WIDTH/(double)iw) )
    {
        sh = HEIGHT;
        sw = (int)((double)(sh * iw) / (double)ih);
        mImgX = (WIDTH-sw) / 2;
        mImgY = 0;
    } else {
        sw = WIDTH;
        sh = (int)((double)(sw * ih) / (double)iw);
        mImgX = 0;
        mImgY = (HEIGHT-sh) / 2;
    }

	if( ( sw + sh ) < 2 )
		return;

    QImage *tmp = mImg->getRect( 0, 0, iw, ih, sw, sh );
	if( tmp == NULL )
		return;
    mRaw = tmp->copy();

    // delete tmp;
}

void Preview::createCur( )
{
    unsigned char *bits;
    int iw, ih;

	if( mRaw.isNull() )
		return;

    QImage tmp = mRaw.copy();
    bits = tmp.bits();

    iw = mRaw.width();
    ih = mRaw.height();

    for( int y=0; y < ih; y++ )
    {
        for( int x=0; x < iw; x++ )
        {
            unsigned char *pixel = &bits[ y*iw*4 + x*4 ];
            pixel[3] = 255;
            pixel[2] = mLut[ pixel[2] ];
            pixel[1] = mLut[ pixel[1] ];
            pixel[0] = mLut[ pixel[0] ];
        }
    }
    mCur.convertFromImage( tmp );
    repaint( true );
}

void Preview::createLut( )
{
    // double scale = (255 / double(mHigh-mLow));

    for( int i=0; i < 256; i++ )
    {
        if( i < mLow )
            mLut[i] = (unsigned char)0;
        else if( i > mHigh )
            mLut[i] = (unsigned char)0;
        else
            mLut[i] = 255; // (unsigned char)((i - mLow) * scale);
    }
}

void Preview::resizeEvent( QResizeEvent * )
{
    getRaw();
    createCur();
}

void Preview::paintEvent( QPaintEvent *pe )
{
	QPainter p( this );
    p.drawPixmap( mImgX, mImgY, mCur );

    p.setPen( QColor( 255, 0, 0 ) );
    p.drawRect( mBoxX, mBoxY, mBoxW, mBoxH );
}

void Preview::mouseMoveEvent(  QMouseEvent *me )
{   
    mMseX = me->x();
    mMseY = me->y();
    
    if( mBoxMov )
    {
        setBox( mBoxSX + (mMseX-mMseSX),
                mBoxSY + (mMseY-mMseSY),
                mBoxW, mBoxH );
        repaint( true );
    } else if( mBoxRes ) {
        if( !( mMseX < mImgX || mMseX > mImgX + mCur.width() ||
               mMseY < mImgY || mMseY > mImgY + mCur.height() ) )
        {
            int mxoff = mMseX - mMseSX;
            int myoff = mMseY - mMseSY;
            int boxx, boxw, boxy, boxh;
            bool isRight = false;
            bool isBot   = false;
    
            boxx = mBoxX;
            boxy = mBoxY;
            boxw = mBoxW;
            boxh = mBoxH;
    
            switch( mBoxPos )
            {
            case UL_CORNER:
                boxx = mBoxSX + mxoff;
                boxy = mBoxSY + myoff;
                boxw = mBoxSW - mxoff;
                boxh = mBoxSH - myoff;
                break;
    
            case UR_CORNER:
                boxy = mBoxSY + myoff;
                boxw = mBoxSW + mxoff;
                boxh = mBoxSH - myoff;
                isRight = true;
                break;
    
            case LL_CORNER:
                boxx = mBoxSX + mxoff;
                boxw = mBoxSW - mxoff;
                boxh = mBoxSH + myoff;
                isBot = true;
                break;
    
            case LR_CORNER:
                boxw = mBoxSW + mxoff;
                boxh = mBoxSH + myoff;
                isRight = true;
                isBot = true;
                break;
    
            case UPPER_EDGE:
                boxh = mBoxSH - myoff;
                boxy = mBoxSY + myoff;
                break;
    
            case BOT_EDGE:
                boxh = mBoxSH + myoff;
                isBot = true;
                break;
    
            case RIGHT_EDGE:
                boxw = mBoxSW + mxoff;
                isRight = true;
                break;
    
            case LEFT_EDGE:
                boxx = mBoxSX + mxoff;
                boxw = mBoxSW - mxoff;
                break;
    
            default:
                break;
            }
    
            if( isRight )
            {
                if( boxw < 10 )
                    boxw = 10;                    
            } else {
                if( boxw < 10 )
                {
                    boxx = boxx + boxw - 10;
                    boxw = 10;
                }
            }
    
            if( isBot )
            {
                if( boxh < 10 )
                    boxh = 10;
            } else {
                if( boxh < 10 )
                {
                    boxy = boxy + boxh - 10;
                    boxh = 10;
                }
            }
    
            setBox( boxx, boxy, boxw, boxh );
    
            repaint( true );    
        }

        
    } else {
        mBoxPos = NOT_ON_BOX;

        if( mMseY == mBoxY )
            mBoxPos = UPPER_EDGE;
        
        if( mMseY == mBoxY + mBoxH )
            mBoxPos = BOT_EDGE;
    
        if( mMseX == mBoxX )
        {
            if( mBoxPos == UPPER_EDGE )
            {
                mBoxPos = UL_CORNER;
                setCursor( sizeFDiagCursor );
            } else if( mBoxPos == BOT_EDGE ) {
                mBoxPos = LL_CORNER;
                setCursor( sizeBDiagCursor );
            } else {
                setCursor( sizeHorCursor );
                mBoxPos = LEFT_EDGE;
            }
        }
    
        if( mMseX == mBoxX + mBoxW )
        {
            if( mBoxPos == UPPER_EDGE )
            {
                mBoxPos = UR_CORNER;
                setCursor( sizeBDiagCursor );
            } else if( mBoxPos == BOT_EDGE ) {
                mBoxPos = LR_CORNER;
                setCursor( sizeFDiagCursor );
            } else {
                mBoxPos = RIGHT_EDGE;
                setCursor( sizeHorCursor );
            }
        }
    
        if( mBoxPos == UPPER_EDGE || mBoxPos == BOT_EDGE )
            setCursor( sizeVerCursor );

        if( mBoxPos == NOT_ON_BOX )
            unsetCursor();
    }


}

void Preview::mousePressEvent( QMouseEvent * )
{
    if( mMseX > mBoxX && mMseX < mBoxX + mBoxW &&
        mMseY > mBoxY && mMseY < mBoxY + mBoxH )
    {
        mMseSX = mMseX;
        mMseSY = mMseY;
        mBoxSX = mBoxX;
        mBoxSY = mBoxY;
        mBoxMov = true;
    }

    if( mBoxPos != NOT_ON_BOX )
    {
        mMseSX = mMseX;
        mMseSY = mMseY;
        mBoxSX = mBoxX;
        mBoxSY = mBoxY;
        mBoxSW = mBoxW;
        mBoxSH = mBoxH;
        mBoxRes = true;
    }
}

void Preview::mouseReleaseEvent( QMouseEvent * )
{
    mBoxMov = false;
    mBoxRes = false;
}

void Preview::setLow( int low )
{
    mLow = low;
    createLut();
    createCur();
}

void Preview::setHigh( int high )
{
    mHigh = high;
    createLut();
    createCur();
}

void Preview::setBox( int x, int y, int w, int h )
{
    if( x < mImgX )
        x = mImgX;
    if( y < mImgY )
        y = mImgY;
    if( x+w > mImgX + mCur.width() )
        x = mImgX + mCur.width() - w;
    if( y+h > mImgY + mCur.height() )
        y = mImgY + mCur.height() - h;
    mBoxX = x;
    mBoxY = y;
    mBoxW = w;
    mBoxH = h;
}

void Preview::getSel( int &ox, int &oy, int &ow, int &oh )
{
    double scale = (double)mImg->width() / (double)mCur.width();
    ox = (int)((mBoxX - mImgX) * scale);
    oy = (int)((mBoxY - mImgY) * scale);
    ow = (int)(mBoxW * scale);
    oh = (int)(mBoxH * scale);
}
