#ifndef __PREVIEW_H
#define __PREVIEW_H

#include <qwidget.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qevent.h>
#include <qpainter.h>

#include <image.h>

/**
 * The %Preview class implements the dialog for providing the
 * user with preview of the dataset based on the threshhold
 * currently set on the dialog.
 *
 * \ingroup threshholdPlugin
 */
class Preview : public QWidget
{
protected:
    typedef enum {
        NOT_ON_BOX,
        UL_CORNER,
        LL_CORNER,
        UR_CORNER,
        LR_CORNER,
        LEFT_EDGE,
        BOT_EDGE,
        UPPER_EDGE,
        RIGHT_EDGE
    } BoxPos;

    Image   *mImg;
    QImage   mRaw;
    QPixmap  mCur;
    int      mBand;
    int      mLow;
    int      mHigh;

    int      mBoxX;
    int      mBoxY;
    int      mBoxW;
    int      mBoxH;

    int      mBoxSX;
    int      mBoxSY;
    int      mBoxSW;
    int      mBoxSH;
    int      mMseSX;
    int      mMseSY;

    int      mMseX;
    int      mMseY;

    bool     mBoxMov;
    bool     mBoxRes;

    BoxPos   mBoxPos;

    int      mImgX;
    int      mImgY;
    unsigned char     mLut[256];

    bool     mInited;

    void createCur( );
    void getRaw( );
    void createLut( );
    void setBox( int, int, int, int );

public:
    Preview( QWidget *parent, Image *img, int band );

    void setBand( int band )
    {
        ImageState is = mImg->getImageState();
        is.mRed = is.mGreen = is.mBlue = band;
        mImg->setBandSel( is );

        mBand = band;
        createLut();
        getRaw();
        createCur();
    }

    void setLow( int low );
    void setHigh( int high );
    void getSel( int &ox, int &oy, int &ow, int &oh );

    void resizeEvent( QResizeEvent * );
    void paintEvent( QPaintEvent *pe );
    void mouseMoveEvent(  QMouseEvent * );
    void mousePressEvent( QMouseEvent * );
    void mouseReleaseEvent( QMouseEvent * );
};

#endif

