
#include "bandchooserdlg.h"

BandChooserDlg::BandChooserDlg( QWidget *parent,
                                Image *img )
: QWidget( parent ), mImg( img ) {
  setCaption( "Band Chooser" ); // LABEL THE WINDOW

  // CREATE THE MAIN LAYOUT
  QVBoxLayout *vblMainLayout  = new QVBoxLayout( this );
  vblMainLayout->setAutoAdd( true );

  createIconSets();

  // CLEAR THE CURRENT COLOR
  mCurColor = -1;

  // CLEAR THE SELECTED BAND SET
  mBandSel[0] = 0;
  mBandSel[1] = 0;
  mBandSel[2] = 0;

  // CREATE AND ADD HEADER TO BAND LIST
  mList = new QListView( this );
  mList->addColumn( "Color" );
  mList->addColumn( "Name" );
  mList->addColumn( "Dimension" );
  mList->addColumn( "Datatype" );
  mList->setSorting( -1 );

  // ADD EVENT HANDLER FOR BAND SELECTION CHANGE
  /*
connect( mList,
           SIGNAL( currentChanged( QListViewItem* ) ),
           this,
           SLOT( setColor( QListViewItem*) ) );
  */
  connect( mList,
           SIGNAL( clicked( QListViewItem* ) ),
           this,
           SLOT( setColor( QListViewItem*) ) );
  // CREATE COLOR SELECT SECTION
  QHGroupBox *grpbox = new QHGroupBox( this );
  QButtonGroup *cgGroup = new QButtonGroup( NULL );
  mRgbGroup = new QButtonGroup( NULL );

  mRgbGroup->setExclusive( true );
  cgGroup->setExclusive( true );

  QHBox *hbox = new QHBox( grpbox );
  hbox->setMaximumWidth( 150 );
  mRadColor = new QRadioButton( "Color", hbox );
  cgGroup->insert( mRadColor, 0 );

  mButRed = new QPushButton( "", hbox );
  mButRed->setToggleButton( true );
  mButRed->setMaximumSize( 25, 25 );
  mButRed->setPixmap( mColorIcon[0] );
  mRgbGroup->insert( mButRed, 0 );

  mButGreen = new QPushButton( "", hbox );
  mButGreen->setToggleButton( true );
  mButGreen->setPixmap( mColorIcon[1] );
  mButGreen->setMaximumSize( 25, 25 );
  mRgbGroup->insert( mButGreen, 1 );

  mButBlue = new QPushButton( "", hbox );
  mButBlue->setToggleButton( true );
  mButBlue->setPixmap( mColorIcon[2] );
  mButBlue->setMaximumSize( 25, 25 );
  mRgbGroup->insert( mButBlue, 2 );

  mRadGrey = new QRadioButton( "Greyscale", grpbox );
  mRadGrey->setChecked( true );
  cgGroup->insert( mRadGrey, 1 );

  // ADD EVENT HANDLER FOR SHADE CHANGE
  connect( cgGroup, SIGNAL( clicked( int ) ), this, SLOT( toggleShade( int ) ) );

  // ADD EVENT HANDLER FOR COLOR CHANGE
  connect( mRgbGroup, SIGNAL( clicked( int ) ), this, SLOT( colorChanged( int ) ) );

  // CREATE BUTTONS TO DISPLAY IMAGE OR CLOSE BAND CHOOSER
  QPushButton *butDisplay = new QPushButton( "Load Bands", this );
  connect( butDisplay, SIGNAL( clicked() ), this, SLOT( loadBands() ) ) ;

  // INITIALIZE COLOR SELECTION TO GREYSCALE
  toggleShade( 1 );
}


void BandChooserDlg::createIconSets( ) {
  int w = 16;
  int h = 16;

  QPixmap *pix = new QPixmap( w, h );
  QPainter p1( pix );
  p1.fillRect( 0, 0, w, h, QBrush( red ) );
  mColorIcon[0] = *pix;

  pix = new QPixmap( w, h );
  QPainter p2( pix );
  p2.fillRect( 0, 0, w, h, QBrush( green ) );
  mColorIcon[1] = *pix;

  pix = new QPixmap( w, h );
  QPainter p3( pix );
  p3.fillRect( 0, 0, w, h, QBrush( blue ) );
  mColorIcon[2] = *pix;

  pix = new QPixmap( w, h );
  QPainter p4( pix );
  p4.fillRect( 0, 0, w, h, QBrush( QColor( 100, 100, 100 ) ) );
  mColorIcon[3] = *pix;

}

void BandChooserDlg::loadBands( ) {
  bool validSet = true;
  char *errmsg = "Error:  Please choose a valid set of bands to be displayed";

  // TEST FOR COLOR
  if ( mColor ) {
    errmsg = "Unable to display image.  Please choose bands for RGB display.";

    // TEST FOR VALID RGB SELECTION
    if ( mBandSel[0] == -1 || mBandSel[1] == -1 || mBandSel[2] == -1 )
      validSet = false;
  } else {
    errmsg = "Can not display image.  Please choose a band for greyscale display.";

    // TEST FOR VALID GREYSCALE SELECTION
    if ( mBandSel[0] == -1 )
      validSet = false;
  }

  // DISPLAY ERROR MSG IF INVALID
  if ( !validSet ) {
    QMessageBox::information( NULL, "Display Error", errmsg );
    return;
  }

  ImageState is;
  is.mRed = mBandSel[0];
  is.mGreen = mBandSel[1];
  is.mBlue = mBandSel[2];
  is.mColor = mColor;
  mImg->setBandSel( is );

  // SEND BAND SELECTION CHANGED SIGNAL
  emit selBandsChanged();
  // mViewSet->redrawViews();

}


void BandChooserDlg::reloadList( ) {
  char buf[100];                          // STRING BUFFER
  std::vector<ImageFormat::BandInf> &bandinf =
  mImg->getBandInfSet();
  // CLEAR THE LIST
  mList->clear();

  // ----------------------------------------------------------
  // LOAD ALL AVAILABLE BANDS
  for ( int iband=(int)bandinf.size()-1; iband > -1; iband-- ) {
    int             w, h;
    std::string     name;
    std::string     dtype;
    std::string     sdims;

    // ----------------------------------------------------------
    // GET DATA FROM BAND AND CREATE STRING
    w       = bandinf[iband].width;
    h       = bandinf[iband].height;
    name    = bandinf[iband].name;
    dtype   = ImageFormat::dtypeToString( bandinf[iband].type );

    sprintf( buf, "%d x %d", w, h );
    sdims = buf;

    // ----------------------------------------------------------

    // ----------------------------------------------------------
    // CREATE NEW ITEM & INSERT INTO LIST
    QListViewItem *item = new QListViewItem( mList,
                                             "",
                                             name.c_str(),
                                             sdims.c_str(),
                                             dtype.c_str() );

    mList->insertItem( item );
    // ----------------------------------------------------------
  }
  // ----------------------------------------------------------
}



void BandChooserDlg::toggleShade( int col ) {
  QListViewItem* ili = NULL;      // LIST ITEM INDEX

  // ASSIGN TO LOCAL MEMBER
  mColor = !(bool)col;

  // CLEAR IMAGE SELECTIONS
  mBandSel[0] = -1;
  mBandSel[1] = -1;
  mBandSel[2] = -1;

  // SET THE COLOR COLUMN OF THE LIST TO "" FOR ALL BANDS
  ili = mList->firstChild();
  while ( ili ) {
    ili->setText( 0, "" );
    ili = ili->nextSibling();
  }

  // TOGGLE ENABLED STATE OF RADIO BUTTON GROUP
  mButRed->setEnabled( mColor );
  mButGreen->setEnabled( mColor );
  mButBlue->setEnabled( mColor );

  // SET RED RADIO BUTTON AS THE CHECKED BUTTON
  mRgbGroup->setButton( 0 );

  // SET THE CURRENT SELECTED COLOR TO RED (NOT SURE IF THIS IS NECESSARY)
  if ( !mColor )
    mCurColor = 0;

  reloadList();
}



void BandChooserDlg::colorChanged( int color ) {
  // ASSIGN THE COLOR TO MEMBER
  mCurColor = color;
}

void BandChooserDlg::setColor( QListViewItem* selItem ) {
  QListViewItem *ili = NULL;              // LOOP INDEX
  int count = 0;

  if ( mBlockSig ) {
    mBlockSig = false;
  } else {
    // GET THE CURRENT SELECTED ITEM
    if ( selItem == NULL )
      return;

    // ----------------------------------------------------------
    // CLEAR COLOR COLUMN FOR SELECTED BAND
    QPixmap pix;
    ili = mList->firstChild();
    while ( ili ) {
      if ( ili == selItem ) {
        if ( mColor ) {
          if ( mBandSel[0] == count )
            mBandSel[0] = -1;
          else if ( mBandSel[1] == count )
            mBandSel[1] = -1;
          else if ( mBandSel[2] == count )
            mBandSel[2] = -1;

          mBandSel[mCurColor] = count;
        } else
          mBandSel[0] = mBandSel[1] = mBandSel[2] = count;
      }

      ili->setPixmap( 0, pix );
      ili = ili->nextSibling();
      count++;
    }

    count = 0;
    ili = mList->firstChild();
    while ( ili ) {
      if ( count == mBandSel[0] ) {
        if ( mColor ) {
          ili->setPixmap( 0, mColorIcon[0] );
        } else {
          ili->setPixmap( 0, mColorIcon[3] );
          break;
        }
      } else if ( count == mBandSel[1] ) {
        ili->setPixmap( 0, mColorIcon[1] );
      } else if ( count == mBandSel[2] ) {
        ili->setPixmap( 0, mColorIcon[2] );
      }
      count++;
      ili = ili->nextSibling();
    }
    // ----------------------------------------------------------

    if ( mColor ) {

      mCurColor++;
      if ( mCurColor > 2 )
        mCurColor = 0;
      mRgbGroup->setButton( mCurColor );
    }
    //mBlockSig = true;
  }
  mList->setSelected( selItem, false );
}





