#ifndef __BANDCHOOSERDLG_H
#define __BANDCHOOSERDLG_H


// Qt
#include <qwidget.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qlistview.h>
#include <qcheckbox.h>
#include <qradiobutton.h>
#include <qhgroupbox.h>
#include <qbuttongroup.h>
#include <qhbox.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qmessagebox.h>
#include <qpixmap.h>
#include <qpainter.h>

#include <vector>
#include <string>
#include <cstdio>

// LOCAL
#include "image.h"

/**
 * A user interface dialog that presents the bands
 * of the open image within glimsviewer.
 *
 * There are radio buttons for RGB or Greyscale that
 * allows the user to select a multiple bands or a
 * single band for display.
 *
 */

class BandChooserDlg : public QWidget {
  Q_OBJECT

protected:
  Image               *mImg; //!< reference to the band info set


/**
 *   @name widgets in dialog
 */
  //!@{
  QListView      *mList;
  QRadioButton   *mRadColor;
  QRadioButton   *mRadGrey;
  QPushButton    *mButRed;
  QPushButton    *mButGreen;
  QPushButton    *mButBlue;
  QButtonGroup   *mRgbGroup;
  //!@}

  QPixmap         mColorIcon[4];

  bool            mColor;         //!< color or greyscale (true || false)

  // std::vector<int>   mBandIndex;
  int             mCurColor;
  int             mBandSel[3];
  bool            mBlockSig;


  void createIconSets( );

public:
/**
 * Initializes local member variables and creates the UI
 * for selecting bands.
 *
 */
  BandChooserDlg( QWidget *parent,
                  Image *img );

public slots:
/**
 * Display current band selection.
 * Event handler to display the current band selection in
 * the image views.  If the selection is invalid, this will
 * tell the user to make a valid selection.
 */
  void loadBands( );

/**
 * (Re)populate the band list.
 * Fill the band selection list with all available bands
 * grouped by image.
 */
  void reloadList( );

  /**
   * Change current color.
   * Event handler for the RGB radio button set.  Changes
   * the current color to the selected.
   *
   * @param color  color selection
   */
  void colorChanged( int color );

  /**
   * Set current color.
   * Event handler for the RGB radio buttons.  Sets the
   * current color to that which is selected.
   *
   * @param selItem selected color item in list view
   */
  void setColor( QListViewItem* selItem );

  /**
   * Toggle greyscale/color.
   * Event handler for color checkbox to change from color
   * to greyscale & vice versa.
   *
   * @param col    flag to indicate if toggling to color or greyscale mode
   */
  void toggleShade( int col );

  /**
   * Set the active image.
   * Sets the active Image to the new Image and reloads
   * the band chooser list.
   *
   * @param img    Image to be set as the active image.
   */
  void setImage( Image *img ) {
    mImg = img; reloadList();
  }

/**
 * Signal emitted when band selection is changed.
 */
signals:
  void selBandsChanged( );
};

#endif




