#include "configdlg.h"

ConfigDlg::ConfigDlg( QWidget *parent ) :
QTabWidget( parent ) {
  setCaption( "Configuration" );
  resize( 400, 250 );
  connectDlgs();
}


/*
 *
 *
 *     CONSTRUCTOR:
 *          ConfigDlg
 *
 *     PARAMS:
 *          lt          PTR to the parent calling LineTool
 *
 *     PURPOSE:
 *          Creates and loads the child dialogs that this widget
 *          holds.
 *
 *
 *
 */
ConfigDlg::ConfigDlg( QWidget *parent,
                      GLIMSDataset &gds ) :
QTabWidget( parent, NULL, false ) {
  // mTabDlg->show();

  // LABEL WINDOW
  setCaption( "Configuration" );
  loadTabs( gds );
  resize( 400, 500 );
  connectDlgs();
}

void ConfigDlg::connectDlgs( ) {
  connect( mLSD,
           SIGNAL( repaintViews() ),
           this,
           SLOT( emitRepaintViews() ) );

  connect( mGCD,
           SIGNAL( repaintViews() ),
           this,
           SLOT( emitRepaintViews() ) );

}

/*
 *
 *
 *     DESTRUCTOR:
 *          ~ConfigDlg
 *
 *     PARAMS:
 *
 *     PURPOSE:
 *          None
 *
 *
 *
 */
ConfigDlg::~ConfigDlg( ) {

}

void ConfigDlg::loadTabs( GLIMSDataset &gds ) {

// CREATE CHILD DIALOGS
  mLSD = new LineSelectorDlg( gds.getLineData() );
  mGCD = new GlacierConfigDlg( gds.getGIDData(),
                               gds.getLineData() );

  mSDG = new SessionDlg( gds.getSession() );
  mIID = new ImageInfDlg( gds.getImageInf() );


  // CREATE TABS FOR EACH CHILD DIALOG AND ADD THEM
  addTab( mLSD, "Line Config" );
  addTab( mGCD, "Glacier Config" );
  addTab( mSDG, "Session" );
  addTab( mIID, "Image Info" );

  /*
  connect( &gds,
           SIGNAL( datasetChanged() ),
           this,
           SLOT( resetAll() ) );

  connect( mGCD,
           SIGNAL( repaintViews( bool ) ),
           this,
           SLOT( emitRepaintViews( bool ) ) );
  */

}

void ConfigDlg::resetAll( ) {
  mLSD->loadList();
  mGCD->loadList();
  mSDG->loadData();
  mIID->loadData();

}

void ConfigDlg::refreshAll( ) {
  mGCD->updateListSelection();
}


void ConfigDlg::emitRepaintViews( ) {
  mGCD->updateListSelection() ;
  emit repaintViews( );
}



