/**
 * This class is a tabbed widget that holds the four config
 * dialogs: LineSelectorDlg, GlacierConfigDlg, SessionDlg, and
 * ImageInfDlg.
 *
 * @author Dan Mingus 2002-09-25 (or earlier)
 *
 * @history 2004-12-04 dls (undoc'd changes)
 *
 * @todo 2006-01-21 dls - All panels in the configuration dialog
 *                        need context sensitive help.
 */
#ifndef CONFIGDLG_H
#define CONFIGDLG_H

// QT
#include <qtabwidget.h>

// LOCAL
#include "glimsdataset.h"
#include "lineselectordlg.h"
#include "glacierconfigdlg.h"
#include "imageinfdlg.h"
#include "sessiondlg.h"
#include "selectionset.h"

class ConfigDlg : public QTabWidget {
  Q_OBJECT
  protected:
  LineSelectorDlg         *mLSD;         // PTR TO THE CHILD LineSelectorDlg
  GlacierConfigDlg        *mGCD;         // PTR TO THE CHILD GlacierConfigDlg
  SessionDlg              *mSDG;         // PTR TO THE CHILD SessionDlg
  ImageInfDlg             *mIID;         // PTR TO THE CHILD ImageInfoDlg

 /**
  * Set up signal-slot connections.
  */
  void connectDlgs( );


public:
  ConfigDlg( QWidget * );
  ConfigDlg( QWidget *,
             GLIMSDataset & );

  ~ConfigDlg( );

  void loadTabs( GLIMSDataset & );

  GlacierConfigDlg* getGCD( ) {
    return mGCD;
  }
  LineSelectorDlg*  getLSD( ) {
    return mLSD;
  }
  SessionDlg*       getSDG( ) {
    return mSDG;
  }
  ImageInfDlg*      getIID( ) {
    return mIID;
  }


public slots:

  /**
   * Reset reload all of the child dialogs.  SessionDlg does
   * not need to be reloaded.
   */
  void resetAll( );

  /**
   * Refresh dialog boxes.
   */
  void refreshAll( );

  /**
   * Ask for a repaint of the Views
   */
  void emitRepaintViews( );

signals:
  void repaintViews( );

};

#endif



