#include "ImageArithmaticCube.h"


ImageArithmaticCube::ImageArithmaticCube(int x1, int y1, int w1, int h1) :
  ImageCube( x1, y1, w1, h1){
  eqset = false;
  calculatedLayer = new std::vector<double>();
}


ImageArithmaticCube::~ImageArithmaticCube() {
//  for(int i=0; i<theCube.size(); i++){
//    delete theCube[i].values;
//  }
}


void ImageArithmaticCube::setEquation(std::string eq){
  parser.SetExpr(eq.c_str());
  eqset = true;
}


std::vector<double>* ImageArithmaticCube::calculateLayer(){
  extern TraceLog glimslog ;
  std::stringstream logstream ;
  logstream << "ImageArithmaticCube::calculateLayer START " << eqset << endl ;
  glimslog.log( logstream.str() ) ;
  logstream.str("") ;

  if(!eqset) return NULL;//if the equation is not set, we can't do anythin
  bool success = calculate();
  if (success) {
    logstream << "ImageArithmaticCube::calculateLayer calc successful " << endl ;
    glimslog.log( logstream.str() ) ;
    logstream.str("") ;
    return calculatedLayer;
  }
  else {
    logstream << "ImageArithmaticCube::calculateLayer calc bombed " << endl ;
    glimslog.log( logstream.str() ) ;
    logstream.str("") ;
    return NULL ;
  }
}


bool ImageArithmaticCube::calculate(){

  //holds the variables for the parser
  std::vector<double> vars (layers);

  //this sets up the variable that will represent the layers
  for(int i=0; i<layers; i++){
    std::stringstream ss;

    //the layer varible is a b followd by the layer num
    //these start at 0
    ss << "b" << (theCube[i]).layernum;

    //this defines the variable for the parser
    parser.DefineVar((ss.str()).c_str(), &(vars[i])) ;
  }


  int layersize = width * height;

  for(int j = 0 ; j < layersize ; j++) { //for each pixel

    //set up the variables with the data from that location
    for(unsigned int k = 0 ; k < vars.size() ; k++){
      vars[k] = (*(theCube[k].values))[j];
    }

    //caculates this location
    try {
      calculatedLayer->push_back( parser.Eval() );
    }
    catch (Parser::exception_type &e) {
      QMessageBox::information( 0, "Parsing Error", (e.GetMsg()).c_str()) ;
      return false ;
    }
  }
  return true ;
}


std::vector<double>* ImageArithmaticCube::getCalculatedLayer(){
  return calculatedLayer;
}

