#ifndef __IMAGEARITHMATICCUBE_H
#define __IMAGEARITHMATICCUBE_H

#include "ImageCube.h"
//#include <muParserDLL.h>
#include <muParser.h>

#include <iostream>
#include <sstream>

using namespace mu;

class ImageArithmaticCube : public ImageCube {

/**
 * ImageArithmaticCube is a specialized ImageCube that handles
 * arithmatic between layers.  It uses muParser to parse and deal
 * with the equation.  Info on muParser can be found in it's folder,
 * or at http://muparser.sourceforge.net/.
 *
 * @author Erica Liszewski 2006-04-20
 */
public:

  /**
   * ImageArithmaticCube(int x1, int y1, int w1, int h1)
   *  int x1 - the lowest x value this cube covers
   *  int y1 - the lowest y value this cube covers
   *  int w1 - the width (in pixels) of this cube
   *  int h1 - the height (in pixels) of this cube
   * Creates an ImageArithmaticCube with the specified dimesions.
   */
  ImageArithmaticCube(int x1, int y1, int w1, int h1);

  /**
   * ~ImageArithmaticCube()
   * Destroys the ImageArithmaticCube
   */
  ~ImageArithmaticCube();

  /**
   * void setEquation(std::string eq)
   *  std::string eq - the equation to be executed on this cube
   * Sets the equation that will be used for calculations.
   */
  void setEquation(std::string eq);

  /**
   * std::vector<double> calculateLayer()
   * Preforms the specified calculation on the cube, and returns
   * the resulting layer. This pointer is creates using new, so
   * it is your responsiblity to see that it's destroyed properly.
   */
  std::vector<double>* calculateLayer();

  /**
   * Performs the specified calculation on the cube, without returning
   * the resulting layer.
   *
   * @return boolean indicating success or failure of calculation
   *
   * @history 2006-07-29  dls added return to allow tool to shut down
   *                          if something went wrong with calculation
   */
  bool calculate();

  /**
   * std::vector<double> getCalculatedLayer()
   * Returns the calculated layer, but does no calculations.
   * You must call calculate() first or this will not return
   * anything useful.  This is ment for getting the calculated
   * layer if it is needed later in the program after calculate()
   * has been called.  If you want to do the calculations and get
   * the resulting layer, use calculateLayer().  The returned pointer
   * is created with new, so it is your responsiblity to see that
   * it is destroyed properly.
   */
  std::vector<double>* getCalculatedLayer();

protected:
  Parser parser ;
  std::vector<double>* calculatedLayer; //!< the results layer
  bool eqset;                           //!< true if the equation has been set for this Cube

};
#endif

