#ifndef __IMAGECALCULATOR_H
#define __IMAGECALCULATOR_H

#include "ImageInterface.h"
#include "ImageShapeFinder.h"
#include "ImageArithmaticCube.h"

/**
 * ImageCalculator is the middle-man between the GUI (ImageCalculatorDialog)
 * and the actual processing classes.  Input is passed in from the GUI,
 * and used to direct the processing classes to do their thing.  Some
 * error checking and messages may be done here, but no user input
 * checking is done below this level.
 *
 * @author Erica Liszewski 2006-04-20
 *
 * @history 2006-08-06 Deborah Lee Soltesz added algorithm choosing functionality.
 *                     This class serves simply to convey the algorithm choice from
 *                     the dialog to the ImageShapeFinder class where the algorithm
 *                     is implemented.
 *
 * @todo dls 2006-08-06 remove algorithm choice functionality
 */
class ImageCalculator{

public:

  /**
   *  ImageCalculator(GLIMSProject *proj)
   *  GLIMSProject *proj - the project to be associated with
   * Creates an ImageCalculator associated with the given project.
   */
  ImageCalculator(GLIMSProject *proj);

  /**
   * ~ImageCalculator()
   * destroys the Image Calculator
   */
  ~ImageCalculator();

  //! @todo get rid of the poly drawing algorithm chooser
  int algorithm ;
  void setAlgorithm  (int choice) {
    extern TraceLog glimslog ;
    std::stringstream logstream ;
    logstream << "ImageCalculator::setAlgorithm START " << choice << endl ;
    glimslog.log( logstream.str() ) ;
    logstream.str("") ;

    algorithm = choice ;
  }

  /**
   * void calculate(std::string eq)
   *  std::string eq - the equation to be calculated
   * Starts the calculation process on the givin string.
   */
  void calculate(std::string eq);

  /**
   * void calculate(std::string eq, int lineStyle)
   *  std::string eq - the equation to be calculated
   *  int lineStyle - the index for specifying a line style
   * Starts the calculation process on the givin string.
   *
   * @history 2006-07-29  dls added return if calculation failed.
   */
  void calculate(std::string eq, int lineStyle);

  /**
   * void testDriver()
   * A testing function that creates test data and runs a calculation
   */
  void testDriver();


protected:
  GLIMSProject *mProj;//the project
  ImageInterface *mImage;//an interface to the image associated


};

#endif

