#include "ImageCalculatorDialog.h"
#include "ImageArithmaticCube.h"
#include "maintoolbar.h"


ImageCalculatorDialog::ImageCalculatorDialog(GLIMSProject *proj){
  mProj = proj;
  calculator = new ImageCalculator(mProj);
  buildDialog();

  //this sets up the Dialog to die when closed
  this->setWFlags(WDestructiveClose);
}

ImageCalculatorDialog::~ImageCalculatorDialog(){
  //delete the calculator, because it's created with new
  if (calculator != NULL) {
    delete calculator;
  }
}

void ImageCalculatorDialog::buildDialog(){
  setCaption("Image Calculator");

  vbox = new QVBoxLayout(this);//overall layout

  //holds equation line and clear button
  QHBox *eqbox = new QHBox(this);
  vbox->addWidget(eqbox);

  //hold equation
  equation = new QLineEdit(eqbox);

  //button to clear equation
  QPushButton *clear = new QPushButton("Clear", eqbox);
  QObject::connect(clear, SIGNAL(clicked()), this, SLOT(clearEquation()));

  //holds all the keys and function boxes
  QHBox *keypad = new QHBox(this);
  keypad->setSpacing(10);

  vbox->addWidget(keypad);

  //TODO add in fn drop-down and band selector
  QVBox *selectables = new QVBox(keypad);

  //shows all layers and theire representing variable
  variables = new QListBox(selectables);
  QObject::connect(variables, SIGNAL(doubleClicked( QListBoxItem * )), this, SLOT(insertBand(QListBoxItem * )));
  variables->setMaximumHeight(100);

  // stuff for line color selector
  lineStyle = new QComboBox(selectables);
  //QObject::connect(variables, SIGNAL(activated( int)), this, SLOT(setColor(int)));


  GLIMSLineData &mLineData = mProj->getDataset()->getLineData();
  //get the linedefset - for attempt at changing line color

  LineDefSet ldfset = mLineData.getLDFSet();
  //LineDefSet &ldfset = mLineData.getLDFSet();


  for ( int i=0; i < (int)ldfset.size(); i++ ) {
    //char buf[20];
    LineDef &ldf = mLineData.getLDFSet()[i];
    //std::string lmat       = GLIMSGlobals::mGlbMDF[ldf.mLMat].id;
    //std::string rmat       = GLIMSGlobals::mGlbMDF[ldf.mRMat].id;
    std::string color      = GLIMSGlobals::AVAILCOLORSTR[ldf.mColor];
    // std::string glactype   = GLIMSGlobals::mGlbGlacTypes[ldf.mGlacType].id;
    std::string glactype   = GLIMSGlobals::mGlbGlacTypes[ldf.mGlacType].value;
    // std::string segmentlbl = GLIMSGlobals::mGlbLbls[ldf.mLabel].id;
    std::string style    = "";
    //std::string type     = GLIMSGlobals::mGlbTypes[ ldf.mType ].id;

    if ( ldf.mStyle == 1 )
      style = "Solid Line";
    else if ( ldf.mStyle == 2 )
      style = "Dash Line";
    else
      style = "Dot Line";

    stringstream lineName;
    lineName << glactype << ", " << color << " " << style;

    lineStyle->insertItem((lineName.str()).c_str(), i);
  }

  //end stuff for color selector


  //get the band names for displaying in the ListBox
  ImageInterface *im = ImageInterface::getInterface(mProj);
//  ImageInterface *im = new ImageInterface(mProj);

  std::vector<string> names = im->getBandNames();

  for(unsigned int j=0; j<names.size(); j++){
    std::stringstream bandname;
    bandname << "b" << j;

    bandname << " - " << names[j];
    variables->insertItem((bandname.str()).c_str());
  }

  //ImageInterface::destroyInterface();

  //holds various functions that are handled by the parser,
  //but not useful enough to get their own buttons
  functions = new QComboBox(selectables);
  QObject::connect(functions, SIGNAL(activated( const QString &)), this, SLOT(acceptFunction(const QString &)));

  functions->insertItem("sin");
  functions->insertItem("cos");
  functions->insertItem("tan");
  functions->insertItem("asin");
  functions->insertItem("acos");
  functions->insertItem("atan");
  functions->insertItem("sinh");
  functions->insertItem("cosh");
  functions->insertItem("tanh");
  functions->insertItem("asinh");
  functions->insertItem("acosh");
  functions->insertItem("atanh");
  functions->insertItem("log2");
  functions->insertItem("log10");
  functions->insertItem("log");
  functions->insertItem("ln");
  functions->insertItem("exp");
  functions->insertItem("sqrt");
  functions->insertItem("sign");
  functions->insertItem("rint");
  functions->insertItem("abs");
  functions->insertItem("min");
  functions->insertItem("max");
  functions->insertItem("sum");
  functions->insertItem("avg");


  //operator buttons
  QGrid *operators = new QGrid(2, keypad);

  ValueButton *but = new ValueButton("+", operators);
  but->setValue("+");
  QToolTip::add(but, "Addition");
  QObject::connect(but, SIGNAL(clicked(char* )), this, SLOT(acceptString(char* )));


  but = new ValueButton("^2", operators);
  but->setValue("^2");
  QToolTip::add(but, "Squared");
  QObject::connect(but, SIGNAL(clicked(char* )), this, SLOT(acceptString(char* )));


  but = new ValueButton("-", operators);
  but->setValue("-");
  QToolTip::add(but, "Subtraction");
  QObject::connect(but, SIGNAL(clicked(char* )), this, SLOT(acceptString(char* )));


  but = new ValueButton("sqrt", operators);
  but->setValue("sqrt");
  QToolTip::add(but, "Square Root");
  QObject::connect(but, SIGNAL(clicked(char* )), this, SLOT(acceptFunction(char* )));


  but = new ValueButton("/", operators);
  but->setValue("/");
  QToolTip::add(but, "Division");
  QObject::connect(but, SIGNAL(clicked(char* )), this, SLOT(acceptString(char* )));


  but = new ValueButton("^y", operators);
  but->setValue("^");
  QToolTip::add(but, "Power");
  QObject::connect(but, SIGNAL(clicked(char* )), this, SLOT(acceptString(char* )));


  but = new ValueButton("*", operators);
  but->setValue("*");
  QToolTip::add(but, "Multiplication");
  QObject::connect(but, SIGNAL(clicked(char* )), this, SLOT(acceptString(char* )));


  but = new ValueButton("%", operators);
  but->setValue("%");
  QToolTip::add(but, "Modulus");
  QObject::connect(but, SIGNAL(clicked(char* )), this, SLOT(acceptString(char* )));

  //the number section
  QGrid *numbers = new QGrid(3, keypad);

  but = new ValueButton("1", numbers );
  but->setValue("1");
  QObject::connect(but, SIGNAL(clicked(char* )), this, SLOT(acceptString(char* )));

  but = new ValueButton("2", numbers );
  but->setValue("2");
  QObject::connect(but, SIGNAL(clicked(char* )), this, SLOT(acceptString(char* )));

  but = new ValueButton("3", numbers );
  but->setValue("3");
  QObject::connect(but, SIGNAL(clicked(char* )), this, SLOT(acceptString(char* )));

  but = new ValueButton("4", numbers );
  but->setValue("4");
  QObject::connect(but, SIGNAL(clicked(char* )), this, SLOT(acceptString(char* )));

  but = new ValueButton("5", numbers );
  but->setValue("5");
  QObject::connect(but, SIGNAL(clicked(char* )), this, SLOT(acceptString(char* )));

  but = new ValueButton("6", numbers );
  but->setValue("6");
  QObject::connect(but, SIGNAL(clicked(char* )), this, SLOT(acceptString(char* )));

  but = new ValueButton("7", numbers );
  but->setValue("7");
  QObject::connect(but, SIGNAL(clicked(char* )), this, SLOT(acceptString(char* )));

  but = new ValueButton("8", numbers );
  but->setValue("8");
  QObject::connect(but, SIGNAL(clicked(char* )), this, SLOT(acceptString(char* )));

  but = new ValueButton("9", numbers );
  but->setValue("9");
  QObject::connect(but, SIGNAL(clicked(char* )), this, SLOT(acceptString(char* )));

  but = new ValueButton("( )", numbers );
  but->setValue("");
  QObject::connect(but, SIGNAL(clicked(char* )), this, SLOT(acceptFunction(char* )));

  but = new ValueButton("0", numbers );
  but->setValue("0");
  QObject::connect(but, SIGNAL(clicked(char* )), this, SLOT(acceptString(char* )));

  but = new ValueButton(".", numbers );
  but->setValue(".");
  QObject::connect(but, SIGNAL(clicked(char* )), this, SLOT(acceptString(char* )));

  //comparitors keypad (>, =, ect)
  QGrid *comparitors = new QGrid(2, keypad);

  but = new ValueButton("==", comparitors );
  but->setValue("==");
  QToolTip::add(but, "Equivalent");
  QObject::connect(but, SIGNAL(clicked(char* )), this, SLOT(acceptString(char* )));

  but = new ValueButton("!=", comparitors );
  but->setValue("!=");
  QToolTip::add(but, "Not Equal");
  QObject::connect(but, SIGNAL(clicked(char* )), this, SLOT(acceptString(char* )));

  but = new ValueButton("<", comparitors );
  but->setValue("<");
  QToolTip::add(but, "Less Than");
  QObject::connect(but, SIGNAL(clicked(char* )), this, SLOT(acceptString(char* )));

  but = new ValueButton(">", comparitors );
  but->setValue(">");
  QToolTip::add(but, "Greater Than");
  QObject::connect(but, SIGNAL(clicked(char* )), this, SLOT(acceptString(char* )));

  but = new ValueButton("<=", comparitors );
  but->setValue("<=");
  QToolTip::add(but, "Less Than or Equal To");
  QObject::connect(but, SIGNAL(clicked(char* )), this, SLOT(acceptString(char* )));

  but = new ValueButton(">=", comparitors );
  but->setValue(">=");
  QToolTip::add(but, "Greater Than or Equal To");
  QObject::connect(but, SIGNAL(clicked(char* )), this, SLOT(acceptString(char* )));

  but = new ValueButton("and", comparitors );
  but->setValue(" and ");
  QToolTip::add(but, "Boolean And");
  QObject::connect(but, SIGNAL(clicked(char* )), this, SLOT(acceptString(char* )));

  but = new ValueButton("or", comparitors );
  but->setValue(" or ");
  QToolTip::add(but, "Boolean Or");
  QObject::connect(but, SIGNAL(clicked(char* )), this, SLOT(acceptString(char* )));



  //the close and go buttons
  QHBox *buttonBox = new QHBox(this);
  vbox->addWidget(buttonBox);


  //! @todo dls - get rid of algorithm chooser
  //holds various functions that are handled by the parser,
  //but not useful enough to get their own buttons
  algorithms = new QComboBox(buttonBox);
  algorithms->insertItem("Basic Drawing Algorithm");
  algorithms->insertItem("Tweak 1");
  algorithms->insertItem("Tweak 2");

// end algo chooser


  go = new QPushButton("&Convert", buttonBox);//button to do processing
  QObject::connect(go, SIGNAL(clicked()), this, SLOT(doCalculation()));

  cancel = new QPushButton("Cancel", buttonBox);//cancel
  QObject::connect(cancel, SIGNAL(clicked()), this, SLOT(close()));

}

void ImageCalculatorDialog::doCalculation(){
  extern TraceLog glimslog ;
  std::stringstream logstream ;
  logstream << "ImageCalculatorDialog::doCalculation START" << endl ;
  glimslog.log( logstream.str() ) ;
  logstream.str("") ;

  const char* temp = (equation->text()).ascii();

  logstream << "ImageCalculatorDialog::doCalculation got equation" << endl ;
  glimslog.log( logstream.str() ) ;
  logstream.str("") ;

  //! @todo dls - get rid of algorithm chooser
  calculator->setAlgorithm(algorithms->currentItem()) ;
  
  logstream << "ImageCalculatorDialog::doCalculation got algorithm" << endl ;
  glimslog.log( logstream.str() ) ;
  logstream.str("") ;

  calculator->calculate(std::string(temp), lineStyle->currentItem());
  
  logstream << "ImageCalculatorDialog::doCalculation did calculation" << endl ;
  glimslog.log( logstream.str() ) ;
  logstream.str("") ;

  emit imgCalcComplete() ;
 
  logstream << "ImageCalculatorDialog::doCalculation END" << endl ;
  glimslog.log( logstream.str() ) ;
  logstream.str("") ;
}

void ImageCalculatorDialog::clearEquation(){
  equation->clear();
}


void ImageCalculatorDialog::acceptString(char* ch){
  equation->insert(ch);

}

void ImageCalculatorDialog::acceptFunction(char* ch){
  QString sel = "";

  if(equation->hasMarkedText())
    sel = equation->markedText();

  sel.insert(0, "(");
  sel.insert(0, ch);
  sel.append(")");

  equation->insert(sel);
  equation->setCursorPosition(equation->cursorPosition()-1);

}

void ImageCalculatorDialog::acceptFunction(const QString & ch){
  QString sel = "";

  if(equation->hasMarkedText())
    sel = equation->markedText();

  sel.insert(0, "(");
  sel.insert(0, ch);
  sel.append(")");

  equation->insert(sel);

  equation->setCursorPosition(equation->cursorPosition()-1);

}

void ImageCalculatorDialog::insertBand(QListBoxItem * item){
  QString band = item->text();
  band.truncate(band.find("-"));
  equation->insert(band);
}

void ImageCalculatorDialog::setColor(int index){ index++;} // just to get the compiler to stop complaining


