#ifndef __IMAGECALCULATORDIALOG_H
#define __IMAGECALCULATORDIALOG_H

#include <qdialog.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qlineedit.h>
#include <qtooltip.h>
#include <qlistbox.h>
#include <qpushbutton.h>
#include <qlineedit.h>
#include <qcombobox.h>
#include "glimsproject.h"
#include "ImageCalculator.h"
#include "ValueButton.h"


/**
 * ImageCalculatorDialog is the GUI interface for the image calculation
 * and arithmatic.  This class handles all user input, and calls other
 * classes for actual computation.
 *
 * @author Erica Liszewski 2006-04-20
 * @whatsnew 2006-07-28 New Image Calculator tool auto-generates polygons 
 *                      based on conditional statement that includes/excludes
 *                      pixels from the area
 *
 *
 * @history 2006-08-06 Deborah Lee Soltesz added algorithm choosing functionality.
 *                     The choice is passed along to the ImageShapeFinder class
 *                     that implements the algorithm through the ImageCalculator
 *                     class. This is a temporary capability and will be removed
 *                     when the final algorithm is developed.
 * 
 * @todo dls 2006-08-06 remove algorithm choice functionality
 */
class ImageCalculatorDialog : public QDialog {
  Q_OBJECT

public:
  /**
   * ImageCalculatorDialog(GLIMSProject *proj)
   * GLIMSProject *proj - the project this is associated with
   * Creates a new ImageCalculatorDialog associated with the
   * specified project;
   */
  ImageCalculatorDialog(GLIMSProject *proj);

  /**
   * ~ImageCalculatorDialog()
   * destructor
   */
  ~ImageCalculatorDialog();

protected:
  GLIMSProject    *mProj;        //!< the project
  ImageCalculator *calculator;   //!< the calculator that does processing
  QPushButton     *go;           //!< start calculation button
  QPushButton     *cancel;       //!< cancel button
  QLineEdit       *equation;     //!< the equation entered by user
  QComboBox       *functions;    //!< possible functions
  QComboBox       *lineStyle;    //!< possible line styles
  QListBox        *variables;    //!< possible variables (layers)
  QVBoxLayout     *vbox;         //!< box layout for layout

  //! @todo dls - get rid of algorithm chooser
  QComboBox       *algorithms;    //!< possible variables (layers)

  /**
   * void buildDialog()
   * Build up the GUI dialog box
   */
  void buildDialog();

protected slots:

  /**
   * void doCalculation()
   * Called when the go button is pushed.  Starts the calculation
   * with the equation currently in the equation QLineEdit.
   */
  void doCalculation();

  /**
   * void clearEquation()
   * Called when the Clear button is pushed.  Clears the equation in
   * the QLineEdit.
   */
  void clearEquation();

  /**
   * void acceptString(char* ch)
   *  char* ch - the string to be added
   * Called when various buttons are pressed that add a string value
   * to the equation (i.e. things like 'and', 'or', etc).
   */
  void acceptString(char* ch);

  /**
   * void acceptFunction(char* ch)
   *  char* ch - the function to be added
   * Called when various buttons are pushed, The function name is passed in as
   * a string, and the parenthesis are places at the end.  (i.e. for
   * a sine function, you would only pass in 'sin' and the parens would
   * be added to display 'sin()' in the equation.
   */
  void acceptFunction(char* ch);

  /**
   * void acceptFunction(const QString & ch)
   *  const QString & ch - the function to be added
   * Same as above, but takes a QString instead of a char*.  This is
   * used for the functions drop-down.
   */
  void acceptFunction(const QString & ch);

  /**
   * void insertBand(QListBoxItem * item)
   *  QListBoxItem * item - the layer to be added
   * Adds a variable representing a layer to the equation.  This is
   * called when an item in the layers listbox is double-clicked.
   */
  void insertBand(QListBoxItem * item);

  void setColor(int index);

signals:
  void imgCalcComplete() ;

};

#endif

