#include "ImageCube.h"

ImageCube::ImageCube(int x, int y, int w, int h, ImageInterface* theimage){
  image = theimage;

  width = w;
  height = h;
  layers = image->getBands();

  lowestPoint.x = x;
  lowestPoint.y = y;
  lowestPoint.z = -1;

  highestPoint.x = (x+w)-1;
  highestPoint.y = (y+h)-1;
  highestPoint.z = -1;

  addBands(image->getCubeAllBands((int)(lowestPoint.x + 0.5), 
                                  (int)(lowestPoint.y + 0.5), 
                                  (int)(highestPoint.x + 0.5), 
                                  (int)(highestPoint.y + 0.5)));
}

ImageCube::ImageCube(int x, int y, int w, int h){

  width = w;
  height = h;
  layers = 0;

  lowestPoint.x = x;
  lowestPoint.y = y;
  lowestPoint.z = -1;

  highestPoint.x = (x+w)-1;
  highestPoint.y = (y+h)-1;
  highestPoint.z = -1;

}


ImageCube::~ImageCube() {
  for(unsigned int i = 0 ; i < theCube.size() ; i++){
    delete theCube[i].values;
  }
}


std::vector<double> * ImageCube::getValuesOf(int z) {
  return theCube[z].values ;
}


double ImageCube::getValueAtLocal(int x, int y, int z){
  if (x < 0 || y < 0) {
    return -99.99 ;
  }

  int index = z;

  if(theCube[z].layernum != z){//if the layers aren't in order
    for(unsigned int i = 0 ; i < theCube.size() ; i++){
      if(theCube[i].layernum == z) {//find the correct layer
        index = i;
        break;
      }
    }
    if(index == z) return 0;//if the layer isn't there...
  }

  return (*(theCube[index].values))[((y*(width))+(x))] ;
}


double ImageCube::getValueAtOverall(int x, int y, int z){
  int nx = (int)(x - lowestPoint.x) ;
  int ny = (int)(y - lowestPoint.y) ;
  return getValueAtLocal(nx, ny, z);
}

ImageCube::ThreeDPoint ImageCube::getLowestPoint(){
  return lowestPoint;
}

ImageCube::ThreeDPoint ImageCube::getHighestPoint(){
  return highestPoint;
}

bool ImageCube::addBand(std::vector<double>* vals){
  Layer* templayer = new Layer();
  templayer->values = vals;
  templayer->layernum = layers;

  theCube.push_back(*templayer);
  layers++;

  return true;
}

bool ImageCube::addBand(std::vector<double>* vals, int layernum){

  Layer* templayer = new Layer();
  templayer->values = vals;
  templayer->layernum = layernum;

  std::vector<Layer>::iterator it = theCube.begin();
  bool inserted = false;
  while ( it != theCube.end() && !inserted ){
    if((*it).layernum > layernum){
      theCube.insert(it, *templayer);
      inserted = true;
    }
    it++;
  }


  if(!inserted){
    theCube.push_back(*templayer);
  }
  layers++;
  return true;

}


bool ImageCube::addBands(std::vector<double>* vals){
  int blocksize = width * height;

  if(((vals->size()) % blocksize) != 0) {
    QMessageBox::information( 0, "ImageCube Error", "Input vector size does not match expected." );
    return false;
  }

  int blocknum = (vals->size()) / blocksize;
  std::vector<double>* temp = new std::vector<double>(blocksize);

  for(int i = 0 ; i < blocknum ; i++){
    for(int j = 0 ; j < blocksize ; j++){
      (*temp)[j] = (*vals)[(i*blocksize)+j];
    }

    addBand(temp);
    temp = new std::vector<double>(blocksize);
  }

  return true;

}


