#include "ImageInterface.h"



bool ImageInterface::instanceFlag = false;
ImageInterface* ImageInterface::instance = NULL;

ImageInterface* ImageInterface::getInterface(GLIMSProject *proj){
  //return new ImageInterface(proj) ;

  if(! instanceFlag) {
    instance = new ImageInterface(proj);
    instanceFlag = true;
    return instance;
  }
  else
  {
    return instance;
  }

}

void ImageInterface::destroyInterface(){
  if(instanceFlag && instance != NULL) {
    delete instance;
    instanceFlag = false;
  }
}

ImageInterface::~ImageInterface(){
  //instanceFlag = false;
}

ImageInterface::ImageInterface( GLIMSProject *proj ){
  mProj = proj;
}

Image* ImageInterface::getImage(){
  return mProj->getImage();
}

ImageFormat* ImageInterface::getImageFormat(){
  return (mProj->getImage())->getFormat();
}


std::vector<double>* ImageInterface::getCubeLatLong(double lat1, double lon1, int z1, double lat2, double lon2, int z2){
  int highz, lowz;

  //find which z is smaller and which is larger
  if(z1>z2) {
    highz = z1;
    lowz = z2;
  }else{
    lowz = z1;
    highz = z2;
  }

  std::vector<double>* vals = NULL ;

  //create doubles for translating from Lat/Lon to XY
  double x11 = lat1;
  double x21 = lat2;
  double y11 = lon1;
  double y21 = lon2;

  getImageFormat()->getXY(lowz, x11, y11);
  getImageFormat()->getXY(lowz, x21, y21);

  int lowx;
  int lowy;
  int highy;
  int highx;

  //find which x is smaller and which is larger
  if(x11>x21) {
    highx = (int)(x11+0.5);
    lowx = (int)x21;
  }
  else{
    lowx = (int)x11;
    highx = (int)(x21+0.5);
  }

  //find which y is smaller and which is larger
  if(y11>y21) {
    highy = (int)(y11+0.5);
    lowy = (int)y21;
  }else{
    lowy = (int)y11;
    highy = (int)(y21+0.5);
  }


  for(int i = lowz; i<=highz; i++){//for each band

    //create doubles for translating from Lat/Lon to XY
    double x1 = lat1;
    double x2 = lat2;
    double y1 = lon1;
    double y2 = lon2;

    //translate corners to XY
    getImageFormat()->getXY(i, x1, y1);
    getImageFormat()->getXY(i, x2, y2);

    int nlowx;
    int nlowy;
    int nhighy;
    int nhighx;

    //find which x is smaller and which is larger
    if(x1>x2) {
      nhighx = (int)(x1+0.5);
      nlowx = (int)x2;
    }else{
      nlowx = (int)x1;
      nhighx = (int)(x2+0.5);
    }

    //find which y is smaller and which is larger
    if(y1>y2) {
      nhighy = (int)(y1+0.5);
      nlowy = (int)y2;
    }
    else{
      nlowy = (int)y1;
      nhighy = (int)(y2+0.5);
    }

    //get the values requested
    void * newline = getImageFormat()->getBand(i, nlowx, nlowy, (nhighx-nlowx), (nhighy-nlowy), (highx-lowx)+1, (highy-lowy)+1, ImageFormat::SCALE_NN) ;
    std::vector<double>* temp = NULL ;

    //get the values requested
    if (newline != NULL) {
      temp = (getValueAsDouble(newline, i, ((highx-lowx)+1)*((highy-lowy)+1))) ;
    }

    //get the values requested
    if(i==lowz && temp != NULL) vals = temp;

    else if (temp != NULL){
    //put the values into the vector...at the end
      vals->insert(vals->end(), temp->begin(), temp->end());
      delete temp;
    }
  }

  return vals;
}

std::vector<double>* ImageInterface::getCube(int x1, int y1, int z1, int x2, int y2, int z2){

  int lowx;
  int lowy;
  int lowz;
  int highx;
  int highy;
  int highz;

  //find which x is smaller and which is larger
  if(x1>x2) {
    highx = x1;
    lowx = x2;
  }
  else {
    lowx = x1;
    highx = x2;
  }

  //find which y is smaller and which is larger
  if(y1>y2) {
    highy = y1;
    lowy = y2;
  }
  else {
    lowy = y1;
    highy = y2;
  }

  //find which z is smaller and which is larger
  if(z1>z2) {
    highz = z1;
    lowz = z2;
  } 
  else {
    lowz = z1;
    highz = z2;
  }


  //create a vector, go ahead and assign it to the first (lowest) band
  std::vector<double>* vals = (getValueAsDouble(getImageFormat()->getBand(lowz, lowx, lowy, (highx-lowx)+1, (highy-lowy)+1, (highx-lowx)+1, (highy-lowy)+1, ImageFormat::SCALE_NN), lowz, ((highx-lowx)+1)*((highy-lowy)+1)));


  //create double values of corners, so they can be translated
  double templx = lowx;
  double temply = lowy;
  double temphx = highx;
  double temphy = highy;

  //translate corners to Lat/Lon
  getImageFormat()->getLL(lowz, templx, temply);
  getImageFormat()->getLL(lowz, temphx, temphy);

  //QMessageBox::information( 0, "Application name", "before loop" );


  for(int i = lowz+1; i<=highz; i++){//for each band

    //create doubles for translating from Lat/Lon to XY
    double nhighx = temphx;
    double nlowx = templx;
    double nhighy = temphy;
    double nlowy = temply;

    //translate corners to XY
    getImageFormat()->getXY(i, nlowx, nlowy);
    getImageFormat()->getXY(i, nhighx, nhighy);

    //get the values requested
    std::vector<double>* temp = (getValueAsDouble(getImageFormat()->getBand(i, (int)nlowx, (int)nlowy, (int)(nhighx-nlowx)+1, (int)(nhighy-nlowy)+1, (highx-lowx)+1, (highy-lowy)+1, ImageFormat::SCALE_NN), i, ((highx-lowx)+1)*((highy-lowy)+1)));

    //put the values into the vector...at the end
    vals->insert(vals->end(), temp->begin(), temp->end());
    delete temp;
  }

  return vals;//return vector
}

std::vector<double>* ImageInterface::getCubeAllBands(int x1, int y1, int x2, int y2){
    return getCube(x1, y1, 0, x2, y2, getImageFormat()->getNumBands()-1);
}

std::vector<double>* ImageInterface::getSlice(int x1, int y1, int z1, int x2, int y2, int z2){
    int lowx;
    int lowy;
    int highx;
    int highy;

    //figure out which x is larger and smaller
    if(x1>x2) {
      highx = x1;
      lowx = x2;
    }
    else{
      lowx = x1;
      highx = x2;
    }

    //figure out which y is larger and smaller
    if(y1>y2) {
      highy = y1;
      lowy = y2;
    }else{
      lowy = y1;
      highy = y2;
    }


  //if the slice is all in one layer
  if(z1 == z2){
    //return the vector from this layer
    return getValueAsDouble(getImageFormat()->getBand(z1, lowx, lowy, (highx-lowx)+1, (highy-lowy)+1, (highx-lowx)+1, (highy-lowy)+1, ImageFormat::SCALE_NN), z1, ((highx-lowx)+1)*((highy-lowy)+1));
  }
  else {//if the vector is accross layers
    if(x1 == x2 || y1 == y2){//one of these must be the same

      return getCube(highx, highy, z1, lowx, lowy, z2);
      /* @TODO so, yeah.  This works and does the stretching
       * all well in the x direction, but not the y direction
       * ideally, this problem would be fixed in the
       * ImageFormat::getBand(), but if not we can hack it here
       */

    }
    return NULL;//if something wasn't right, return NULL
  }

}


std::vector<double>* ImageInterface::getSliceScaled(int x1, int y1, int z1, int x2, int y2, int z2){
    int lowx;
    int lowy;
    int highx;
    int highy;


    //figure out which x is larger and smaller
    if(x1>x2) {
      highx = x1;
      lowx = x2;
    }else{
      lowx = x1;
      highx = x2;
    }

    //figure out which y is larger and smaller
    if(y1>y2) {
      highy = y1;
      lowy = y2;
    }else{
      lowy = y1;
      highy = y2;
    }

  //if the slice is all in one layer
  if(z1 == z2){
    //return the vector from this layer

    //get the size of the first layer, for scaling
    int scaledh = (highy - lowy)+1;
    int scaledw = (highx - lowx)+1;

    double nhx = highx+1;
    double nlx = lowx;
    double nhy = highy+1;
    double nly = lowy;

    if(z1 > 0){//if this isn't the only layer, we may need to scale

      //get the vector of BandInf
      std::vector<ImageFormat::BandInf> &binf = getImage()->getBandInfSet();

      //get the size of the type from the proper BandInf
      int baseh = binf[0].height;
      int basew = binf[0].width;
      int thish = binf[z1].height;
      int thisw = binf[z1].width;

      if( (baseh != thish) || (basew != thisw) ){
        //translate corners to Lat/Lon
        getImageFormat()->getLL(0, nlx, nly);
        getImageFormat()->getLL(0, nhx, nhy);

        //translate back to XY relative to z1
        getImageFormat()->getXY(z1, nlx, nly);
        getImageFormat()->getXY(z1, nhx, nhy);

        //was getting strange negatives sometimes, so we make this 0
        if(nlx < 0) nlx = 0;
        if(nly < 0) nly = 0;

        //translate back to ints
        lowx = (int)nlx;
        lowy = (int)nly;
        highx = (int)(nhx+0.5);
        highy = (int)(nhy+0.5);

      }
      return getValueAsDouble(getImageFormat()->getBand(z1, lowx, lowy, (highx-lowx), (highy-lowy), scaledw, scaledh, ImageFormat::SCALE_NN), z1, scaledw*scaledh);
    }
    else
      return getValueAsDouble(getImageFormat()->getBand(z1, lowx, lowy, (highx-lowx)+1, (highy-lowy)+1, (highx-lowx)+1, (highy-lowy)+1, ImageFormat::SCALE_NN), z1, ((highx-lowx)+1)*((highy-lowy)+1));

  } else {//if the vector is accross layers


    if(x1 == x2 || y1 == y2){//one of these must be the same
      /* @TODO so, yeah.  This works and does the stretching
       * all well in the x direction, but not the y direction
       * ideally, this problem would be fixed in the
       * ImageFormat::getBand(), but if not we can hack it here
       */
      return getCube(highx, highy, z1, lowx, lowy, z2);
    }
    return NULL;//if something wasn't right, return NULL
  }

}

std::vector<double>* ImageInterface::getPoint(int x1, int y1, int z1){
  return getValueAsDouble(getImageFormat()->getBand(z1, x1, y1, 1, 1, 0, 0), z1, 1);
}

int ImageInterface::getBands(){
  return getImageFormat()->getNumBands();
}
int ImageInterface::getHeight(){
  return getImage()->height();
}
int ImageInterface::getWidth(){
  return getImage()->width();
}
char* ImageInterface::getPath(){
  return NULL;//this needs to be fixed...but I don't know how
}

QImage* ImageInterface::getQImage(int neww, int newh){
  return getImage()->getRect(0, 0, getWidth(), getHeight(), neww, newh);
}

QImage* ImageInterface::getQImageArea(int x, int y, int w, int h, int neww, int newh){
  return getImage()->getRect(x, y, w, h, neww, newh);
}

std::vector<double>* ImageInterface::getValueAsDouble(void* line, int bandnum, int width){
  //create a vector to hold the values
  std::vector<double> *theValues = new std::vector<double>(width);

  //get the vector of BandInf
  std::vector<ImageFormat::BandInf> &binf = getImage()->getBandInfSet();

  //get the size of the type from the proper BandInf
  //int dtsz = ImageFormat::dtypeSize( binf[bandnum].type );

  for( int x=0; x < width; x++ ) {
    int dx = x ; //* dtsz;//dx is now the byte of the next value

    // based on type, do the proper cast to double
    switch( binf[bandnum].type ) {
      case ImageFormat::INT8:
         (*theValues)[x] = (double)(((char*)line)[dx]);
         break;

      case ImageFormat::UINT8:
         (*theValues)[x] = (double)(((unsigned char*)line)[dx]);
         break;

      case ImageFormat::INT16:
         (*theValues)[x] = (double)(((short*)line)[dx]);
         break;

      case ImageFormat::UINT16:
         (*theValues)[x] = (double)(((unsigned short*)line)[dx]);
         break;

      case ImageFormat::INT32:
         (*theValues)[x] = (double)(((int*)line)[dx]);
         break;

      case ImageFormat::UINT32:
         (*theValues)[x] = (double)(((unsigned int*)line)[dx]);
         break;

      case ImageFormat::INT64:
         (*theValues)[x] = (double)(((long*)line)[dx]);
         break;

      case ImageFormat::UINT64:
         (*theValues)[x] = (double)(((unsigned long*)line)[dx]);
         break;

      default:
          break;
    }
  }

  return theValues;
}

std::vector<std::string> ImageInterface::getBandNames(){
  //get the vector of BandInf
  std::vector<ImageFormat::BandInf> &binf = getImage()->getBandInfSet();

  std::vector<std::string> temp;

  for(int i=0; i<getBands(); i++){
    temp.push_back(binf[i].name);
  }

  return temp;
}

void ImageInterface::getLL( int band, double &x, double &y){
  getImageFormat()->getLL(band, x, y);
}

void ImageInterface::getXY( int band, double &lat, double &lon){
  getImageFormat()->getXY(band, lat, lon);
}

std::string ImageInterface::runDriverTest() {
  string test = "";
  stringstream ss;
  std::vector<double> temp = *(getSlice(1000, 1000, 2, 1000, 1010, 5));
  //std::vector<double> temp = getCube(1000, 1000, 2, 1010, 1010, 5);
  for(unsigned int i = 0; i < temp.size(); i++){
    if(i%11 == 0) ss << "\n";
    //int dec, sign;
    //translate double values to string values
    ss << temp[i] << "; ";
    //  std::string str2 = fcvt(temp[i], 0, &dec, &sign);
    //std::string str2 = "2.0";
    //test.append(str2+"; ");

  }

  test = ss.str();
  return test;

}

