#include "PixelArray.h"
#include <math.h>
#include <iostream>
#include <fstream>

PixelArray::PixelArray(){
  iter = pixels.begin();//set up the iterator
}

PixelArray::~PixelArray(){;}

void PixelArray::clearAll(){
  pixels.clear();//clear the vectore
}

int PixelArray::size(){
  return pixels.size();
}

void PixelArray::add(int x, int y){
  //create and add new PixelProfile to end
  pixels.push_back(*(new PixelProfile(x, y)));
  iter = pixels.begin();//update iterator
}

/* I'm assuming that most of the time pixels will be accessed in
 * the same order they were put into the array.  So, we keep an
 * iterator around between calls so we can easily grab the next
 * pixel.  If the next pixel is not the one we need, we run through
 * the list to find it.  Return NULL if it's not found.
 */
PixelProfile* PixelArray::getPixel(int x, int y){
  int thex = x;
  int they = y;

  iter++;//increment

  if(iter != pixels.end() && iter >= pixels.begin() && (*iter).getX() == thex && (*iter).getY() == they){
    return &(*iter) ;//return if it matches
  }

  int loopCount = 1 ;
  iter = pixels.begin();

  while (iter != pixels.end() && iter >= pixels.begin() && (int)((pixels.size()) / (pow(2.0, loopCount))) != 0 &&
        (((*iter).getX() != thex) || ((*iter).getY() != they))) {

    if ((*iter).getY() > they) {
      iter -= (int)((pixels.size()) / (pow(2.0, loopCount))) ;
      loopCount++ ;
    }
    else if ((*iter).getY() < they) {
      iter += (int)((pixels.size()) / (pow(2.0, loopCount))) ;
      loopCount++ ;
    }
    else if ((*iter).getY() == they) {
      if ((*iter).getX() < thex ) {
        iter++ ;
      }
      else if ((*iter).getX() > thex ) {
        iter-- ;
      }

    }
  }


  // if value's been found, return it
  if ((*iter).getX() == thex && (*iter).getY() == they) {
    return &(*iter);
  }

  // totally give up on quick and fancy and just search from beginning
  else{
    iter = pixels.begin();
    while (iter!= pixels.end() && iter >= pixels.begin()){
      if((*iter).getX()== thex && (*iter).getY()== they) {
        return &(*iter);
      }
      iter++;
    }
  }

  // the value is not in the array, return null
  return NULL;
}

void PixelArray::addToProfile(int x, int y, double val){
  PixelProfile* pix = getPixel(x, y);
  if(pix != NULL)//make sure we found it
    pix->add(val);
}

std::string PixelArray::getAsString(){
  std::string str = "";
  std::stringstream ss;


  for(std::vector<PixelProfile>::size_type i=0; i<pixels.size(); i++){
    ss << pixels[i].getAsString() << "\n";
  }
  str = ss.str();

  return str;
}


/**
 * @todo Improve performance if possible
 */
std::string PixelArray::getAsString(std::string delim){
  std::string str = "";
  std::stringstream ss;

  for(std::vector<PixelProfile>::size_type i=0; i<pixels.size(); i++){
    ss << pixels[i].getAsString(delim) << "\n";
  }
  str = ss.str();

  return str;
}


/**
 * @todo Improve performance if possible
 */
std::string PixelArray::getAsString(std::string delim, std::ofstream &f){
  for(std::vector<PixelProfile>::size_type i=0; i<pixels.size(); i++){
      f << pixels[i].getAsString(delim) << "\n";
  }
  return "";
}


std::string PixelArray::runDriverTest(){
  std::string str = "Starting Driver Test\n";

  //create some PixelProfiles
  this->add(1, 2);
  this->add(3, 4);
  this->add(5, 6);
  this->add(7, 8);
  this->add(9, 10);


  //create some data for each profile
  int i = 0 ;
  for(i=0; i<5; i++){
    (this->getPixel((2*i)+1, (2*i)+2))->add(i);
  }

  for(i=0; i<5; i++){
    (this->getPixel((2*i)+1, (2*i)+2))->add(i*4);
  }
  for(i=0; i<5; i++){
    (this->getPixel((2*i)+1, (2*i)+2))->add(i+9);
  }
  for(i=0; i<5; i++){
    (this->getPixel((2*i)+1, (2*i)+2))->add(3);
  }

  //test messager
  str.append("pixels created\n");
  //get the string of this PixelArray
  str.append(this->getAsString());

  return str;
}

