#ifndef __PIXELARRAY_H
#define __PIXELARRAY_H

#include "PixelProfile.h"

#include <vector>

/**
 * This class manages a group of PixelProfiles.  PixelProfiles
 * can be added, updated, orobtained through this class.  The
 * group of PixelProfiles can also be cleared.
 *
 * @author Erica Liszewski 2006-02-15
 */
class PixelArray {
protected:
  std::vector<PixelProfile> pixels;         //!< the PixelProfiles
  std::vector<PixelProfile>::iterator iter; //!< a way of navigating

public:
  PixelArray();
  ~PixelArray();

  /**
   * clears all the PixelProfiles leaving an empty PixelArray
   */
  void clearAll();

  /**
   * return the number of PixelProfiles in this PixelArray
   */
  int size();

  /**
   * creates and adds a new Pixel profile with x = x and y = y
   */
  void add(int x, int y);

  /**
   * returns a pointer to the PixelProfile with the input x and y,
   * returns NULL if no matching PixelProfile is found
   */
  PixelProfile* getPixel(int x, int y);

  /**
   * updates the PixelProfile with input x and y by adding the
   * value val
   * void addToProfile(int x, int y, double val)
   */
  void addToProfile(int x, int y, double val);

  /**
   * all values from all PixelProfiles are output with
   * each PixelProfile on it's own line.  Individual values
   * in the PixelProfile are separated by ;
   * std::string getAsString()
   * returns a string representation of this PixelArray
   */
  std::string getAsString();

  /**
   * same as above, but you can specify the Delimiter
   * std::string getAsString(std::string:delim)
   * std::string:delim - the desired Delimiter
   */
  std::string getAsString(std::string delim);
  std::string getAsString(std::string delim, std::ofstream &f) ;

  /**
   * runs a driver that creates test PixelProfiles and adds
   * values to them.  The returned string is the getAsString()
   * of this test set.
   * std::string runDriverTest()
   */
  std::string runDriverTest();

};

#endif

