#ifndef __PIXELPROFILE_H
#define __PIXELPROFILE_H

#include <vector>
#include <iostream>
#include <sstream>
#include <string>


/**
 * PixelProfile.h
 *  @author Erica Liszewski 2006-02-15
 * PixelProfile is designed to hold multiple values for a
 * single pixel.  The intent is to hold values for a given pixel
 * through muliple layers in an image cube.  The PixelProfile
 * holds the x and y of it's pixel, as well as any values
 * added to it.
 *
 * because data may be in different forms depending on
 * the format of the image, the add() function is overridden
 * to handle each case.  If more formats are needed, more
 * add() functions can be created.  All values in the PixelProfile
 * are stored as double.
 *
 */
class PixelProfile {
protected:
  int x; //!< x value
  int y; //!< y value
  std::vector<double> values;//values

public:
  PixelProfile(int newx, int newy);
  ~PixelProfile();



  /**
   *  void add(int val)
   * casts an int to a double and adds
   */
  void add(int val);

  /**
   * void add(double val)
   * adds a double value
   */
  void add(double val);

  /**
   * void add(float val)
   * casts a float to a double and adds
   */
  void add(float val);

  /**
   * void add(char* val)
   * translates a char* to a double, and adds
   */
  void add(char *val);

  /**
   * int getX()
   * returns the x value of the pixel
   */
  int getX();
  /**
   * int getY()
   * returns the y value of the pixel
   */
  int getY();

  /**
   * int size()
   * returns the number of values stored in this PixelArray
   */
  int size();

  /**
   *double valueAt(int loc)
   * returns the double value stored at location loc witing
   * the array of values
   */
  double valueAt(int loc);

  /**
   * std::string getAsString()
   * returns a string of this PixelProfile.  Each value is output
   * on a single line with ; as Delimiters
   */
  std::string getAsString();

  /**
   * std::string getAsString(std::string delim)
   *  std::string delim - the desired Delimiter
   * same as above, but you can spcify the Delimiter
   */
  std::string getAsString(std::string delim);

  /**
   * std::string runDriverTest()
   * creates several test values and adds them to this PixelProfile.
   * the returned string is the getAsString() of this test setup.
   */
  std::string runDriverTest();

  };

#endif

