#include "ImageInterface.h"
#include "SpreadsheetDialog.h"
#include "PixelProfile.h"
#include "PixelArray.h"

SpreadsheetDialog::SpreadsheetDialog( GLIMSProject *proj ) :
  QDialog( NULL, "", true ){
    mProj = proj;
    gtos = new GtoSConverter(mProj);
    buildDialog();
}

SpreadsheetDialog::~SpreadsheetDialog(){
    delete gtos;
}

void SpreadsheetDialog::buildDialog(){

  setCaption("GLIMS to Spreadsheet");

  //layout stuff
  QGrid   *grid = new QGrid( 1, this );



  QVBoxLayout *vbox = new QVBoxLayout( this );



  vbox->setAutoAdd( true );
  vbox->setMargin( 10 );

  QHBox   *hbox = new QHBox( grid);

  grid->setSpacing( 7 );
  grid->setMargin( 5 );

  new QLabel( "File: ", hbox );

  filePath = new QLineEdit( hbox );
  filePath->setMinimumWidth( 225 );

  hbox->setSpacing( 5 );

  browse = new QPushButton( "Browse", hbox );//browse button
  QObject::connect( browse, SIGNAL( clicked() ), this, SLOT( setSavePath() ) );


  QHBox * delimbox = new QHBox(grid);
  delimbox->setSpacing(5);

  new QLabel("Delimiter:  ", delimbox );

  delim = new QComboBox( delimbox, "Delimiter");
  delim->insertItem(";  (semi-colon)");
  delim->insertItem(",  (comma)");
  delim->insertItem("\t  (tab)");
  //QObject::connect(delim, signal(activated(const QString & str)), this, slot(setDelim(const QString & str)));

  QHBox   *buttonBox = new QHBox(grid );

  go = new QPushButton("Convert", buttonBox);//button to do processing
  QObject::connect(go, SIGNAL(clicked()), this, SLOT(doConvert()));

  cancel = new QPushButton("Cancel", buttonBox);//cancel
  QObject::connect(cancel, SIGNAL(clicked()), this, SLOT(close()));


  }

void SpreadsheetDialog::setSavePath() {

  QString img = QFileDialog::getSaveFileName(QString::null, QString::null, this);
  if ( img.isNull() )
    return;
  filePath->setText( (const char*)img );
}

void SpreadsheetDialog::doConvert(){
  std::string delimStr;
  const char* temp = (delim->currentText()).ascii();
  delimStr = temp[0];
  gtos->convert( std::string((filePath->text()).ascii()), delimStr);
  accept();
}


