#ifndef __SPREADSHEETDLG_H
#define __SPREADSHEETDLG_H

#include <qdialog.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qlineedit.h>
#include <qfiledialog.h>

#include "GtoSConverter.h"

#include "glimsproject.h"

class ImageInterface;

/**
 * SpreadSheetDialog is the GUI for the GLIMS to Spreadsheet converter.
 * This class handles user input and sends it off to the GtoSConverter
 * for processing.
 *
 * @author Erica Liszewski 2006-04-10
 * @whatsnew 2006-07-28 New Spreadsheet Exporter tool outputs image
 *                      data as delimited text for import to spreadsheet
 *                      or other data analysis application.
 */
class SpreadsheetDialog : public QDialog {
  Q_OBJECT

protected:

  QLabel *label;      //!< says File
  QPushButton *go;    //!< button to do conversion
  QPushButton *browse;//!< button for browsing where to save
  QPushButton *cancel;//!< button to cancel
  QComboBox *delim;   //!< selection of Delimiter
  GLIMSProject *mProj;//!< the project
  GtoSConverter *gtos;//!< the Converter

  QLineEdit *filePath;//!< holds the path

 /**
  *   builds the dialog box, connect all buttons to actions
  */
  void buildDialog();

public:

 /**
  *   GLIMSProject *proj - the project to associate with
  *   creates a new SpreadSheetDialog with the associated project.
  */
  SpreadsheetDialog( GLIMSProject *proj );

  /**
   * destructor
   */
  ~SpreadsheetDialog();

protected slots:

 /**
  *   opens a file browser, and sets the save path.
  *   connected to the browse button.
  */
  void setSavePath();

 /**
  *   starts the conversion, and closes the dialog when complete.
  *   connected to the go button
  */
  void doConvert();

};

#endif

