#include "glacierconfigdlg.h"


GlacierConfigDlg::GlacierConfigDlg( GLIMSGIDData &giddata,
                                    GLIMSLineData &linedata )
  : mLineData( linedata ), mGIDData( giddata ) {

  QPushButton *pb2;   // NEW GID BUTTON
  QPushButton *pb3;   // EDIT GID BUTTON
  QPushButton *pb5;   // SET LINES BUTTON
  QVBoxLayout *vbl;   // MAIN LAYOUT
  QHBoxLayout *hbl;   // BUTTON LAYOUT

  mSelectedGID = -1;

  // -------------------------------------------------------
  // CREATE THE LIST AND COLUMN HEADER
  mListView = new QListView( this );
  mListView->addColumn( "GID" );
  mListView->addColumn( "Name" );
  mListView->addColumn( "Local ID" );
  mListView->addColumn( "WGMS ID" );
  mListView->addColumn( "Parent ID" );
  // -------------------------------------------------------

  // REMOVE SORTING ABILITY
  mListView->setSorting( -1 );

  // -------------------------------------------------------
  // CREATE THE BUTTONS
  pb2 = new QPushButton( "New Glacier ID", this );
  pb3 = new QPushButton( "Edit Glacier ID", this );
  pb5 = new QPushButton( "Set Lines", this );
  // -------------------------------------------------------

  // -------------------------------------------------------
  // CONNECT THE BUTTONS TO EVENT HANDLERS
  connect( pb2, SIGNAL( clicked() ), this, SLOT( newGlacier() ) );
  connect( pb3, SIGNAL( clicked() ), this, SLOT( editGlacier() ) );
  connect( pb5, SIGNAL( clicked() ), this, SLOT( glacierIDChooser() ) );
  connect( mListView, SIGNAL( selectionChanged() ), this, SLOT( setSelectedGID() ) );
  // -------------------------------------------------------

  // CREATE THE MAIN LAYOUT
  vbl = new QVBoxLayout( this );

  // ADD THE LIST VIEW
  vbl->addWidget( mListView );

  // CREATE THE BUTTON LAYOUT
  hbl = new QHBoxLayout( vbl );

  // ADD THE BUTTONS
  hbl->addWidget( pb2 );
  hbl->addWidget( pb3 );
  hbl->addWidget( pb5 );

  connect( &giddata,
           SIGNAL( datasetChanged() ),
           this,
           SLOT( loadList() ) );

  // FILL THE LIST
  loadList();

  mChooserDlg = new GlacierIDChooserDlg (mGIDData, mLineData) ;
  mChooserDlg->hide() ;

  connect( mChooserDlg, SIGNAL (repaintViews()), this, SLOT(updateListSelection()) ) ;

}


GlacierConfigDlg::~GlacierConfigDlg( ) {

}


void GlacierConfigDlg::keyPressEvent (QKeyEvent * e) {

   int keyPressed = e->key() ;
   switch (keyPressed){
     case Key_Delete :
      mGIDData.push();
      delSel(e);
      break;

     case Key_Enter :
      editGlacier() ;
      break;

     default: e->ignore() ;
   }
   loadList() ;

}

void GlacierConfigDlg::delSel(QKeyEvent * e) {

  if ( !mGIDData.size() )
    return;

  bool kCtrlDown = false ;
  if (e->state() & Qt::ControlButton) {
    kCtrlDown = true ;
  }

  SelectionSet &refSelSet = mGIDData.getSelSet() ;

  if ( mGIDData.getType() == VectorData::POINT ) {
    for ( int isel=0; isel < (int)refSelSet.size(); isel++ ) {
      for ( int i=0; i < (int)refSelSet.size(); i++ )
        if ( refSelSet[isel].obj < refSelSet[i].obj )
          refSelSet[i].obj--;
      mGIDData.delNode( refSelSet[isel].obj );
      refSelSet.clear();
    }
  }
  else {
    int szLine = 0;
    for ( int isel=0; isel < (int)refSelSet.size(); isel++ ) {
      Shape &s = *mGIDData.getXYSet()[ refSelSet[isel].obj ];
      Line &l = dynamic_cast<Line&>( s );
      szLine = l.size();
      if ( kCtrlDown || szLine <= (int)refSelSet[isel].nodes.size() ) {
        for ( int i=0; i < (int)refSelSet.size(); i++ )
          if ( refSelSet[isel].obj < refSelSet[i].obj )
            refSelSet[i].obj--;
        mGIDData.delLine( refSelSet[isel].obj );
      }
      else {
        std::sort( refSelSet[isel].nodes.begin(), refSelSet[isel].nodes.end() );
        for ( int inode=refSelSet[isel].nodes.size()-1; inode > -1; inode-- )
          mGIDData.delLineNode( refSelSet[isel].obj, refSelSet[isel].nodes[inode] );
      }
    }

    if ( !kCtrlDown && refSelSet.size() == 1 && refSelSet[0].nodes.size() == 1 ) {
      if ( refSelSet[0].nodes[0] >= szLine-1 )
        refSelSet[0].nodes[0]--;
      if ( refSelSet[0].nodes[0] < 0 ) {
        refSelSet.clear();
      }
    }
    else {
      refSelSet.clear();
    }
  }

  emit repaintViews() ;

}


void GlacierConfigDlg::newGlacier( ) {
  // CREATE THE GID FORM & DISPLAY IT
  GlacierIDDlg *giddlg = new GlacierIDDlg( mGIDData );
  giddlg->exec();

  // RELOAD THE LIST
  loadList();
}


void GlacierConfigDlg::editGlacier( ) {
  // -------------------------------------------------------
  // TEST IF THERE'S A GID SELECTED
  if ( mSelectedGID == -1 ) {
    QMessageBox::information( NULL, "Error", "There are no Glacier IDs available" );
    return;
  }
  // -------------------------------------------------------

  // CREATE THE GID FORM & DISPLAY IT
  GlacierIDDlg *giddlg = new GlacierIDDlg( mGIDData, mSelectedGID );
  giddlg->exec();

  // RELOAD THE LIST
  loadList();
}


// -------------------------------------------------------
// SET CUR LINES
void GlacierConfigDlg::setCurLines( ) {
/*  SelectionSet &selset = mLineData.getSelSet();

  // for each selected line, assign the selected GID to it
  for ( int isel = 0 ; isel < selset.size() ; isel++ ) {
    mLineData.setGID( selset[isel].obj, mSelectedGID );
  }
  updateListSelection() ;
  */

}

// -------------------------------------------------------
// LOAD LIST
void GlacierConfigDlg::loadList( ) {
  // CLEAR THE LIST
  mListView->clear();
  std::vector<GlacierIDDef> &gidset = mGIDData.getDefSet();
  unsigned int i = 0 ; // iterator

  // ADD LIST ITEMS FOR EVERY GID
  for (i = mGIDData.getDefSet().size(); i > 0; i--) {
    GlacierIDDef gid = gidset[i - 1];
    QListViewItem *item = new QListViewItem( mListView,
                                             gid.toString().c_str(),
                                             gid.mName.c_str(),
                                             gid.mLocalID.c_str(),
                                             gid.mWGMSID.c_str(),
                                             gid.mParentID.c_str() );
    mListView->insertItem( item );
  }
}


// -------------------------------------------------------
// UPDATE LIST SELECTION

void GlacierConfigDlg::updateListSelection() {
  // added by dls 12/4/2004
  // show selected items as selected in the glacier id config dialog
  QListViewItem *item ; // temp list item
  unsigned int i = 0 ;  // iterator

  item = mListView->firstChild() ;
  if (item != NULL) {
    for (i = 0 ; i < (unsigned int)mListView->childCount() ; i++) {
      item->setSelected(false) ;
      item = item->nextSibling();
    }
  }
  else return ;

  // show selected glacier IDs as selected in listview
  SelectionSet &gidselset = mGIDData.getSelSet(); // set of selected glacier ID points from the GID vector data
  for (i = 0 ; &gidselset != NULL && i < (unsigned int)gidselset.size() ; i++) {
    item = mListView->firstChild() ;
    int selGID = gidselset[i].obj ;
    for (int j = 0 ; j != selGID ; j++) {
      item = item->nextSibling();
    }
    item->setSelected(true) ;
  }

  // show selected lines as selected in listview
  SelectionSet &lineselset = mLineData.getSelSet();                   // set of selected lines from the line vector data
  for (i = 0 ; &lineselset != NULL && i < (unsigned int)lineselset.size() ; i++) {  // step through selected lines
    int selLine = lineselset[i].obj ;                                 // get the line obj for the current selected line
    std::vector<Shape*> &glXYSet = mLineData.getXYSet() ;             // get the line data
    GlacierLine &xyline = dynamic_cast<GlacierLine&>(*glXYSet[selLine]); // get the line matching the selected line from the line data

    item = mListView->firstChild() ;                                  // get the first item in the ListView
    for (int j = 0 ; j < mListView->childCount() ; j++) {             // step through each ListView item
      if (xyline.mGID == j) {                // if the selected line's related glacier ID equals the selected ListView index
        item->setSelected(true) ;                                     // highlight the ListView item
      }
      item = item->nextSibling();                                     // get the next ListView item
    }
  }

  mListView->triggerUpdate () ;

}


// -------------------------------------------------------
// SET SELECTED GID
void GlacierConfigDlg::setSelectedGID() {
  QListViewItem *selectedItem ;    // SELECTED GID
  QListViewItem *item ;            // TEMP LIST ITEM
  int   i = 0 ;            // COUNTER

  // GET CURRENT SELECTED ITEM
  selectedItem = mListView->selectedItem();

  // FIND INDEX OF CURRENTLY SELECTED ITEM
  item = mListView->firstChild();
  while ( selectedItem != item && i < mListView->childCount() ) {
    i++;
    item = item->nextSibling();
  }

  // ASSIGN NEW SELECTED ITEM INDEX
  mSelectedGID = i;
  mLineData.setDefGID( mSelectedGID );



  // added by dls 12/3/2004
  // HIGLIGHT SELECTED GID POINT & RELATED LINE IN VIEWS
  // set selected GID point in selection set so it's highlighted on screen
  SelectionSet &gidselset = mGIDData.getSelSet(); // set of selected glacier ID points from the GID vector data
  if (gidselset.size()) {
    gidselset.clear() ;              // clear the current selection
  }
  SelectionSet::SelObj gidselobj ; // create a new selection object
  gidselobj.obj = mSelectedGID ;   // set the selection object to be current glacier ID
  gidselset.add ( gidselobj ) ;    // add selection to selection set

  // set selected line related to GID point in selection set so it's highlighted on screen
  SelectionSet &lineselset = mLineData.getSelSet(); // set of selected lines from the line vector data
  if (lineselset.size()) {
    lineselset.clear() ;              // clear the current selection
  }
  SelectionSet::SelObj lineselobj ; // create a new selection object
  std::vector<Shape*> &glXYSet = mLineData.getXYSet() ;
  for (i = 0 ; (unsigned int)i < glXYSet.size() ; i++) {
    GlacierLine &xyline = dynamic_cast<GlacierLine&>(*glXYSet[i]);
    if (xyline.mGID == mSelectedGID) {
      lineselobj.obj = i ;            // set the selection object to be current selected glacier ID
      lineselset.add ( lineselobj ) ; // add selection to selection set
    }
  }

  emit repaintViews();

}




