#ifndef GLACIERCONFIGDLG_H
#define GLACIERCONFIGDLG_H

// QT
#include <qwidget.h>
#include <qlayout.h>
#include <qlistview.h>
#include <qpushbutton.h>
#include <qmessagebox.h>

// LOCAL
#include "glimsgiddata.h"
#include "glimslinedata.h"
#include "glacieriddlg.h"
#include "glacieridchooserdlg.h"
#include "selectionset.h"
#include "glimsglobals.h"

/**
 * User interface for allowing users to create edit
 * glacier ids and assign them to lines.
 * A user interface dialog providing a list of the glacier ids
 * that have been created by the user.  The items in the list
 * can be selected and edited.
 *
 * @author Dan Mingus
 * @history 2002-09-25 Dan Mingus
 *                     created
 *
 * @history 2004-12-04 Deborah Lee Soltesz -
 *                     Changed behavior to highlight selected glacier ids
 *                     between the GlacierConfigDlg and Views. When an ID
 *                     is selected in the dialog, it's highlighted on the View
 *                     and vice-versa.
 *
 * @history 2004-12-05 Deborah Lee Soltesz -
 *                     Added a new dialog (GlacierIDChooserDlg) for assigning
 *                     glacier IDs to the lines currently selected in the Views.
 *                     New highlighting behavior prevented the user from being able
 *                     to choose a line and select a glacier ID to "Set Line" for
 *                     under old setup.
 *
 * @whatsnew 2004-12-04 Glacier IDs in the list are highlighted when selected in an image view.
 * @whatsnew 2004-12-05 There's a new dialog box for assigning glacier IDs to the lines selected in the Views.
 *
 * \todo 2004-12-06 refresh the highlighting of selected glacier IDs after a "Set Line" action.
 */
class GlacierConfigDlg : public QWidget {
  Q_OBJECT
protected:
  QListView        *mListView;    //!< list of GIDs
  GLIMSLineData    &mLineData;    //!< line tool that opened this
  GLIMSGIDData     &mGIDData;     //!< GLIMS vector layer

  int               mSelectedGID;  //!< current selected GID

  GlacierIDChooserDlg * mChooserDlg ; //!< ID chooser dialog

/**
 * Handle known key-press events (not working).
 * @internal
 * In order to allow users to delete GIDs using the ListView, probably need to
 * extend ListView so keyboard events can be dealt with... the dialog is not
 * apparently processing keyevents on the ListView...
 */
  void keyPressEvent (QKeyEvent * e) ;

 /**
  * Delete selected glacier ID. Tied to "delete" key press (not working).
  */
  void delSel(QKeyEvent * e) ;

public:
/**
 * This initializes the list of available glacier IDs
 * and builds the interface.
 */
  GlacierConfigDlg( GLIMSGIDData  &giddata,
                    GLIMSLineData &linedata );
  ~GlacierConfigDlg( );


public slots:
/**
 * Displays the new glacier id form.
 */
  void newGlacier();

/**
 * Displays the glacier id form to edit the selected GID.
 */
  void editGlacier();

/**
 * Assign the selected GID to all selected lines.
 */
  void setCurLines();

/**
 * Event handler for GID selection change.
 */
  void setSelectedGID();

/**
 * Fill the list view with available GIDs
 */
  void loadList();

/**
 * set Glacier ID list view item selected based on SelectSet contents
 */
  void updateListSelection() ;

/**
 * Displays the glacier id chooser dialog for setting a line to a glacier id.
 */
  void glacierIDChooser() {
    if (!mChooserDlg) {
      mChooserDlg = new GlacierIDChooserDlg (mGIDData, mLineData) ;
    }
    else {
      mChooserDlg->loadList() ;
    }
    mChooserDlg->show() ;
    mChooserDlg->setFocus();
  }



signals:
/**
 * Unused signal?
 */
  void changeSelLines( int );
/**
 * Unused signal?
 */
  void selGidChanged( int );
/**
 * Signal Views to be refreshed
 */
  void repaintViews( );

};

#endif








