#include "glacieridchooserdlg.h"


GlacierIDChooserDlg::GlacierIDChooserDlg( GLIMSGIDData &giddata,
                                          GLIMSLineData &linedata )
  : QDialog( NULL, NULL, true ),
    mLineData( linedata ), mGIDData( giddata ) {

  setCaption( "Glacier ID Chooser" );

  QPushButton *pb5;   // APPLY SET LINES BUTTON
  QPushButton *pb6;   // HIDE DIALOG
  QVBoxLayout *vbl;   // MAIN LAYOUT
  QHBoxLayout *hbl;   // BUTTON LAYOUT

  mSelectedGID = -1;

  // -------------------------------------------------------
  // CREATE THE LIST AND COLUMN HEADER
  mListView = new QListView( this );
  mListView->addColumn( "GID" );
  mListView->addColumn( "Name" );
  mListView->addColumn( "Local ID" );
  mListView->addColumn( "WGMS ID" );
  mListView->addColumn( "Parent ID" );
  // -------------------------------------------------------

  // REMOVE SORTING ABILITY
  mListView->setSorting( -1 );

  // -------------------------------------------------------
  // CREATE THE BUTTONS
  pb5 = new QPushButton( "Apply", this );
  pb6 = new QPushButton( "Close", this );
  // -------------------------------------------------------

  // -------------------------------------------------------
  // CONNECT THE BUTTONS TO EVENT HANDLERS
  connect( pb5, SIGNAL( clicked() ), this, SLOT( setCurLines() ) );
  connect( pb6, SIGNAL( clicked() ), this, SLOT( hide() ) );
  connect( mListView, SIGNAL( selectionChanged() ), this, SLOT( setSelectedGID() ) );
  // -------------------------------------------------------

  // CREATE THE MAIN LAYOUT
  vbl = new QVBoxLayout( this );

  // ADD THE LIST VIEW
  vbl->addWidget( mListView );

  // CREATE THE BUTTON LAYOUT
  hbl = new QHBoxLayout( vbl );

  // ADD THE BUTTONS
  hbl->addWidget( pb5 );
  hbl->addWidget( pb6 );

  connect( &giddata,
           SIGNAL( datasetChanged() ),
           this,
           SLOT( loadList() ) );

  // FILL THE LIST
  loadList();
}


GlacierIDChooserDlg::~GlacierIDChooserDlg( ) {;}

// -------------------------------------------------------
// SET CUR LINES
void GlacierIDChooserDlg::setCurLines( ) {
  SelectionSet &selset = mLineData.getSelSet();

  // for each selected line, assign the selected GID to it
  for ( int isel = 0 ; isel < selset.size() ; isel++ ) {
    mLineData.setGID( selset[isel].obj, mSelectedGID );
  }
  this->hide() ;
}

// -------------------------------------------------------
// LOAD LIST
void GlacierIDChooserDlg::loadList( ) {
  // CLEAR THE LIST
  mListView->clear();
  std::vector<GlacierIDDef> &gidset = mGIDData.getDefSet();
  unsigned int i = 0 ; // iterator

  // ADD LIST ITEMS FOR EVERY GID
  for (i = mGIDData.getDefSet().size(); i > 0; i--) {
    GlacierIDDef gid = gidset[i - 1];
    QListViewItem *item = new QListViewItem( mListView,
                                             gid.toString().c_str(),
                                             gid.mName.c_str(),
                                             gid.mLocalID.c_str(),
                                             gid.mWGMSID.c_str(),
                                             gid.mParentID.c_str() );
    mListView->insertItem( item );
  }
}


// -------------------------------------------------------
// SET SELECTED GID
void GlacierIDChooserDlg::setSelectedGID() {
  QListViewItem *selectedItem ;    // SELECTED GID
  QListViewItem *item ;            // TEMP LIST ITEM
  int   i = 0 ;            // COUNTER

  // GET CURRENT SELECTED ITEM
  selectedItem = mListView->selectedItem();

  // FIND INDEX OF CURRENTLY SELECTED ITEM
  item = mListView->firstChild();
  while ( selectedItem != item && i < mListView->childCount() ) {
    i++;
    item = item->nextSibling();
  }

  // ASSIGN NEW SELECTED ITEM INDEX
  mSelectedGID = i;
  mLineData.setDefGID( mSelectedGID );

  emit repaintViews();
}




