#ifndef GLACIERIDCHOOSERDLG_H
#define GLACIERIDCHOOSERDLG_H

// QT
#include <qdialog.h>
#include <qlayout.h>
#include <qlistview.h>
#include <qpushbutton.h>
#include <qmessagebox.h>

// LOCAL
#include "glimsgiddata.h"
#include "glimslinedata.h"
#include "selectionset.h"
#include "glimsglobals.h"

/**
 * A user interface dialog providing a list of the glacier ids
 * that have been created by the user.  The items in the list
 * can be selected and edited.
 *
 * @author Deborah Lee Soltesz (original, GlacierIDDlg, by Dan Mingus)
 *
 * @history 2004-12-05 Deborah Lee Soltesz -
 *                     Added this new dialog (GlacierIDChooserDlg) for assigning
 *                     glacier IDs to the lines currently selected in the Views.
 *                     New highlighting behavior prevented the user from being able
 *                     to choose a line and select a glacier ID to "Set Line" for
 *                     under old setup. This dialog was adapted from GlacierIDDlg
 *                     by Dan Mingus.
 *
 * @whatsnew 2004-12-04 This dialog box was added for assigning glacier IDs to the lines selected in the Views.
 *
 */
class GlacierIDChooserDlg : public QDialog {
  Q_OBJECT
  protected:
  QListView        *mListView;    // LIST OF GIDs
  GLIMSLineData    &mLineData;    // LINE TOOL THAT OPENED THIS
  GLIMSGIDData     &mGIDData;     // GLIMS VECTOR LAYER

  int               mSelectedGID;  // CURRENT SELECTED GID

public:
/**
 * This initializes the list of available glacier ids
 * and builds the interface.
 */
  GlacierIDChooserDlg( GLIMSGIDData  &giddata,
                       GLIMSLineData &linedata );
  ~GlacierIDChooserDlg( );

public slots:
/**
 * Assign the selected GID to all selected lines.
 */
  void setCurLines();

/**
 * Fill the list view with available GIDs
 */
  void loadList();

/**
 * Event handler for GID selection change.
 */
  void setSelectedGID();


signals:
/**
 * Signal Views to be refreshed
 */
  void repaintViews( );

};

#endif




