/*
 * IMPLENTATION OF CLASS:
 *      GlacierIDDlg
 *
 * glacieriddlg.cpp
 */

// LOCAL
#include "glacieriddlg.h"


/*
 *     CONSTRUCTOR:
 *          GlacierIDDlg
 *
 *     PARAMS:
 *          lt          PTR to associated LineTool
 *
 *     PURPOSE:
 *          Initializes the form.
 */

GlacierIDDlg::GlacierIDDlg( GLIMSGIDData &gds )
: QDialog( NULL, NULL, true ),
  mGIDData( gds ) {

  // LABEL WINDOW
  setCaption( "Glacier ID Editor" );

  miGID = -1;
  initialize( );
}


/*
 *
 *
 *     CONSTRUCTOR:
 *          GlacierIDDlg
 *
 *     PARAMS:
 *
 *     PURPOSE:
 *          Initializes the form.  This constructor takes an index
 *          of a GID and fills the form with this GID for editing.
 *
 *
 *
 */
GlacierIDDlg::GlacierIDDlg( GLIMSGIDData &gds, int iGID ) :
QDialog( NULL, NULL, true ),
mGIDData( gds ) {
  miGID = iGID;
  initialize();
  loadGID();
}


/*
 *
 *
 *     DESTRUCTOR:
 *          ~GlacierIDDlg
 *
 *     PARAMS:
 *
 *     PURPOSE:
 *          None
 *
 *
 *
 */
GlacierIDDlg::~GlacierIDDlg( ) {

}


/*
 *
 *
 *     FUNCTION:
 *          initialize
 *
 *     PARAMS:
 *
 *
 *     PURPOSE:
 *          This function initializes the form UI.
 *
 *
 *
 */
void GlacierIDDlg::initialize( ) {
  // MAIN LAYOUT
  QVBoxLayout *vboxl = new QVBoxLayout( this );
  vboxl->setAutoAdd( true );

  QVBox           *mainVB;    //!< Main internal widget (QVBox - Qt Vertical Box)
  QLabel          *lbl;       //!< Reusable label
  QVBox           *vbox;      //!< Reusable QV(ertical)Box
  QHBox           *hbox;      //!< Reusable QH(orizontal)Box
  QGrid           *grid;      //!< Reusable grid

  QPushButton     *butAdd;    //!< Add button
  QPushButton     *butClose;  //!< Close button

  // SET MAIN CONTAINER
  mainVB = new QVBox( this );
  mainVB->setMargin( 8 );

  // FIRST SECTION GRID SPACE
  grid = new QGrid( 4, mainVB );

  // ADD LAT/LON FIELDS
  lbl = new QLabel( "Lat: ", grid );
  mtxtLat = new QLineEdit( grid );
  lbl = new QLabel( " Lon: ", grid );
  mtxtLon = new QLineEdit( grid );

  // ADD NAME & WGMS FIELDS
  lbl = new QLabel( "Name: ", grid );
  mtxtName = new QLineEdit( grid );
  lbl = new QLabel( " WGMS ID: ", grid );
  mtxtWGMSID = new QLineEdit( grid );

  // ADD LOCAL & PARENT ID FIELDS
  lbl = new QLabel( "Local ID: ", grid );
  mtxtLocalID = new QLineEdit( grid );
  lbl = new QLabel( " Parent ID: ", grid );
  mtxtParentIMID = new QLineEdit( grid );

  grid = new QGrid( 2, mainVB );

  // ADD PRIMARY CLASSIFICATION & FORM FIELDS
  vbox = new QVBox( grid );
  lbl = new QLabel( "Primary Classification:", vbox );
  mcmbPrimCsfn = new QComboBox( vbox );

  vbox = new QVBox( grid );
  lbl = new QLabel( "Form:", vbox );
  mcmbForm = new QComboBox( vbox );

  // ADD F & L CHARACTERISTICS FIELDS
  vbox = new QVBox( grid );
  lbl = new QLabel( "Frontal Characteristics:", vbox );
  mcmbFrntChar = new QComboBox( vbox );

  vbox = new QVBox( grid );
  lbl = new QLabel( "Longitudinal Characteristics:", vbox );
  mcmbLonChar = new QComboBox( vbox );

  // ADD DMS & TONGUE ACT FIELDS
  vbox = new QVBox( grid );
  lbl = new QLabel( "Dominant Mass Source:", vbox );
  mcmbDomMassSrc = new QComboBox( vbox );

  vbox = new QVBox( grid );
  lbl = new QLabel( "Tongue Activity:", vbox );
  mcmbTongueAct = new QComboBox( vbox );

  // ADD GLACIER WIDTH/LENGTH/AREA FIELDS
  hbox = new QHBox( mainVB );
  grid = new QGrid( 2, hbox );
  lbl = new QLabel( "Width (m): ", grid );
  mtxtWidth = new QLineEdit( grid );
  grid = new QGrid( 2, hbox );
  lbl = new QLabel( "Length (m): ", grid );
  mtxtLength = new QLineEdit( grid );
  grid = new QGrid( 2, hbox );
  lbl = new QLabel( "Area (km^2): ", grid );
  mtxtArea = new QLineEdit( grid );

  // ADD GLACIER ABZONE AREA, SPEED, ELA FIELDS
  hbox = new QHBox( mainVB );
  grid = new QGrid( 2, hbox );
  lbl = new QLabel( "Abzone Area (km^2): ", grid );
  mtxtAbzoneArea = new QLineEdit( grid );
  grid = new QGrid( 2, hbox );
  lbl = new QLabel( "Speed (m/a): ", grid );
  mtxtSpeed = new QLineEdit( grid );
  grid = new QGrid( 2, hbox );
  lbl = new QLabel( "ELA: ", grid );
  mtxtELA = new QLineEdit( grid );

  // ADD ELA DESCRIPTION FIELD
  vbox = new QVBox( mainVB );
  lbl = new QLabel( "ELA Description:", vbox );
  mtxtELADesc = new QLineEdit( vbox );

  // ADD SNOWLINE ELEVATION FIELD
  grid = new QGrid( 2, mainVB );
  lbl = new QLabel( "Snowline Elevation (m): ",  grid );
  mtxtSnowLineElev = new QLineEdit( grid );

  // ADD RECORD STATUS FIELD
  grid = new QGrid( 2, mainVB );
  lbl = new QLabel( "Record Status: ",  grid );
  mtxtRecordStatus = new QLineEdit( grid );

  // ADD BUTTONS
  hbox = new QHBox( mainVB );
  if ( miGID == -1 )
    butAdd = new QPushButton( "Add", hbox );
  else
    butAdd = new QPushButton( "Update", hbox );

  butClose = new QPushButton( "Cancel", hbox );

  // CONNECT THE BUTTONS TO EVENT HANDLERS
  connect( butAdd, SIGNAL( clicked() ), this, SLOT( addClicked() ) );
  connect( butClose, SIGNAL( clicked() ), this, SLOT( cancelClicked() ) );

  // LOAD VALIDS
  loadValids();

}



/*
 *
 *
 *     FUNCTION:
 *          loadGID
 *
 *     PARAMS:
 *
 *
 *     PURPOSE:
 *          Fill the form with selected GID
 *
 *
 *
 */
void GlacierIDDlg::loadGID( ) {
  char         buf[50];    // STRING BUFFER
  GlacierIDDef &gid = mGIDData.getDefSet()[miGID];

  // -----------------------------------------------------------
  // LOAD THE FORM WITH DATA FROM GID
  sprintf( buf, "%3.4f", gid.mLat );
  mtxtLat->setText( buf );

  sprintf( buf, "%3.4f", gid.mLon );
  mtxtLon->setText( buf );

  mtxtName->setText( gid.mName.c_str() );
  mtxtWGMSID->setText( gid.mWGMSID.c_str() );
  mtxtLocalID->setText( gid.mLocalID.c_str() );
  mtxtParentIMID->setText( gid.mParentID.c_str() );



  int setCur = 0 ;
  setCur = GLIMSGlobals::getNdxOfID(GLIMSGlobals::mGlbPrimCsfn, gid.mPrimCsfn) ;
  mcmbPrimCsfn->setCurrentItem( setCur );

  setCur = 0 ;
  setCur = GLIMSGlobals::getNdxOfID(GLIMSGlobals::mGlbForm, gid.mForm) ;
  mcmbForm->setCurrentItem( setCur );

  setCur = 0 ;
  setCur = GLIMSGlobals::getNdxOfID(GLIMSGlobals::mGlbFrntChar, gid.mFChar) ;
  mcmbFrntChar->setCurrentItem( setCur );

  setCur = 0 ;
  setCur = GLIMSGlobals::getNdxOfID(GLIMSGlobals::mGlbLonChar, gid.mLChar) ;
  mcmbLonChar->setCurrentItem( setCur );

  setCur = 0 ;
  setCur = GLIMSGlobals::getNdxOfID(GLIMSGlobals::mGlbDomMassSrc, gid.mDomMassSrc) ;
  mcmbDomMassSrc->setCurrentItem( setCur );

  setCur = 0 ;
  setCur = GLIMSGlobals::getNdxOfID(GLIMSGlobals::mGlbTongueAct, gid.mTongueAct) ;
  mcmbTongueAct->setCurrentItem( setCur );




  sprintf( buf, "%3.2f", gid.mWidth );
  mtxtWidth->setText( buf );

  sprintf( buf, "%3.2f", gid.mLength );
  mtxtLength->setText( buf );

  sprintf( buf, "%3.2f", gid.mArea );
  mtxtArea->setText( buf );

  sprintf( buf, "%3.2f", gid.mAbzoneArea );
  mtxtAbzoneArea->setText( buf );

  sprintf( buf, "%3.2f", gid.mSpeed );
  mtxtSpeed->setText( buf );

  sprintf( buf, "%3.2f", gid.mELA );
  mtxtELA->setText( buf );

  sprintf( buf, "%3.2f", gid.mSnowLineElev );
  mtxtSnowLineElev->setText( buf );

  mtxtELADesc->setText( gid.mELADesc.c_str() );
  mtxtRecordStatus->setText( gid.mRecordStatus.c_str() );
  // -----------------------------------------------------------
}


/*
 *
 *
 *     FUNCTION:
 *          loadValids
 *
 *     PARAMS:
 *
 *
 *     PURPOSE:
 *          This uses the valids available from LineTool for these
 *          combo boxes.
 *
 *
 *
 */
void GlacierIDDlg::loadValids( ) {
  unsigned int i=0;

  // -----------------------------------------------------------
  // LOAD THE VALIDS INTO THE COMBO BOXES
  for ( i = 0; i < GLIMSGlobals::mGlbPrimCsfn.size(); i++ )
    mcmbPrimCsfn->insertItem( GLIMSGlobals::mGlbPrimCsfn[i].value.c_str() );

  for ( i = 0; i < GLIMSGlobals::mGlbForm.size(); i++ )
    mcmbForm->insertItem( GLIMSGlobals::mGlbForm[i].value.c_str() );

  for ( i = 0; i < GLIMSGlobals::mGlbFrntChar.size(); i++ )
    mcmbFrntChar->insertItem( GLIMSGlobals::mGlbFrntChar[i].value.c_str() );

  for ( i = 0; i < GLIMSGlobals::mGlbLonChar.size(); i++ )
    mcmbLonChar->insertItem( GLIMSGlobals::mGlbLonChar[i].value.c_str() );

  for ( i = 0; i < GLIMSGlobals::mGlbDomMassSrc.size(); i++ )
    mcmbDomMassSrc->insertItem( GLIMSGlobals::mGlbDomMassSrc[i].value.c_str() );

  for ( i = 0; i < GLIMSGlobals::mGlbTongueAct.size(); i++ )
    mcmbTongueAct->insertItem( GLIMSGlobals::mGlbTongueAct[i].value.c_str() );
  // -----------------------------------------------------------
}


/*
 *
 *
 *     SLOT:
 *          addClicked
 *
 *     PARAMS:
 *
 *
 *     PURPOSE:
 *          Event handler for the ADD button.  This will capture
 *          the data in the form and fill a new GID with it.  Then
 *          either add or overwrite to the GID set in LineTool.
 *
 *
 *
 */
void GlacierIDDlg::addClicked( ) {
  GlacierIDDef gid;

  // -----------------------------------------------------------
  // CAPTURE THE FORM DATA & FILL gid
  gid.mLat = atof((const char*)mtxtLat->text());
  gid.mLon = atof((const char*)mtxtLon->text());
  gid.mName = (const char*)mtxtName->text();
  gid.mWGMSID = (const char*)mtxtWGMSID->text();
  gid.mLocalID = (const char*)mtxtLocalID->text();
  gid.mParentID = (const char*)mtxtParentIMID->text();

  gid.mPrimCsfn   = GLIMSGlobals::getIntIDOf(GLIMSGlobals::mGlbPrimCsfn,   (mcmbPrimCsfn->currentText()).ascii()) ;
  gid.mForm       = GLIMSGlobals::getIntIDOf(GLIMSGlobals::mGlbForm,       (mcmbForm->currentText()).ascii()) ;
  gid.mFChar      = GLIMSGlobals::getIntIDOf(GLIMSGlobals::mGlbFrntChar,   (mcmbFrntChar->currentText()).ascii()) ;
  gid.mLChar      = GLIMSGlobals::getIntIDOf(GLIMSGlobals::mGlbLonChar,    (mcmbLonChar->currentText()).ascii()) ;
  gid.mDomMassSrc = GLIMSGlobals::getIntIDOf(GLIMSGlobals::mGlbDomMassSrc, (mcmbDomMassSrc->currentText()).ascii()) ;
  gid.mTongueAct  = GLIMSGlobals::getIntIDOf(GLIMSGlobals::mGlbTongueAct,  (mcmbTongueAct->currentText()).ascii()) ;


  gid.mWidth = atof( (const char*)mtxtWidth->text() );
  gid.mLength = atof( (const char*)mtxtLength->text() );
  gid.mArea = atof( (const char*)mtxtArea->text() );
  gid.mAbzoneArea = atof( (const char*)mtxtAbzoneArea->text() );
  gid.mSpeed = atof( (const char*)mtxtSpeed->text() );
  gid.mELA = atof( (const char*)mtxtELA->text() );
  gid.mELADesc = (const char*)mtxtELADesc->text();
  gid.mSnowLineElev = atof( (const char*)mtxtSnowLineElev->text() );
  gid.mRecordStatus = (const char*)mtxtRecordStatus->text();
  // -----------------------------------------------------------

  // -----------------------------------------------------------
  // UPDATE THE LINETOOL GLACIER ID SET
  if ( miGID == -1 )
    mGIDData.getDefSet().push_back( gid );
  else
    mGIDData.getDefSet()[miGID] = gid;
  // -----------------------------------------------------------

  // RELOAD GlacierConfigDlg
  emit gidSetChanged();

  // CLOSE THIS FORM
  close();
}



/*
 *
 *
 *     SLOT:
 *          cancelClicked
 *
 *     PARAMS:
 *
 *
 *     PURPOSE:
 *          Event handler for the CANCEL button.
 *
 *
 *
 */
void GlacierIDDlg::cancelClicked( ) {
  // CLOSE THIS FORM
  close();
}



/*
 *
 *
 *     SLOT:
 *          keyPressEvent
 *
 *     PARAMS:
 *
 *
 *     PURPOSE:
 *          Event handler for the key press event.  This routine
 *          allows the user to capture lat lon from mouse coords
 *          of the views while this window is focused.
 *
 *
 *
 */
void GlacierIDDlg::keyPressEvent( QKeyEvent * ) {
  /*
char    buf[100];      // STRING BUFFER

  // -----------------------------------------------------------
  // PROCESS KEY
switch( ke->key() )
{
  // HANDLE F1 PRESSED
case QKeyEvent::Key_F1:
  sprintf( buf, "%3.4f", mLineTool->mLat );
  mtxtLat->setText( buf );
  sprintf( buf, "%3.4f", mLineTool->mLon );
  mtxtLon->setText( buf );
  break;
}
  // -----------------------------------------------------------
  */
}


