#ifndef __GLACIERLINE_H
#define __GLACIERLINE_H

#include "windows_defs.h"
#include "shape.h"

/**
 * An extension of a Line that provides indecies into
 * the GLIMSDataset equivalents of GlacierIDDef and
 * LineDef.  This also provides easy serialization
 * routines.
 */

class GV_EXPORT GlacierLine : public Line {
public:
  int     mGID; //!< glacier config listbox index associated with this line
  int     mLDF; //!< line definition listbox index associated with this line

/**
 * Construct line not associated with a line definition or glacier ID
 */
  GlacierLine( )
  : mGID( -1 ), mLDF( -1 ) {;}

/**
 * Construct line associated with a line definition or glacier ID
 * @param igid listbox index of associated glacier ID
 * @param ildf listbox index of associated line definition
 */
  GlacierLine( int igid, int ildf )
  : mGID( igid ), mLDF( ildf ) {;}

/**
 * Read line information from project XML file
 * @param elem XML document structure
 * @returns true if a glacier line was found in the file
 */
  virtual bool fromXML( QDomElement &elem ) {
    if ( std::string( "GlacierLine" ).compare( (const char*)elem.tagName() ) )
      return false;

    mGID = readIntTag( elem, "GID" );
    mLDF = readIntTag( elem, "LDF" );
    nodesetFromXML( elem );

    return true;
  }

/**
 * Write line information to project XML file
 * @param doc  XML document
 * @param elem XML document structure
 * @param id   glacier id (?)
 * @returns true no matter what
 */
  virtual bool toXML( QDomDocument &doc,
                      QDomElement &elem,
                      std::string id = "" ) {
    QDomElement base = doc.createElement( "GlacierLine" );
    if ( id != "" )
      base.setAttribute( "id", (char*)id.c_str() );
    elem.appendChild( base );

    writeIntTag( doc, base, "GID", mGID );
    writeIntTag( doc, base, "LDF", mLDF );
    nodesetToXML( doc, base );
    return true;
  }

/**
 * Copy attributes of another line to this line
 * @param gline line to copy attributes from
 */
  void operator=( const GlacierLine &gline ) {
    for ( int inode=0; inode < (int)gline.size(); inode++ ) {
      mNodeSet.push_back( gline[inode] );
    }
    mLDF = gline.mLDF;
    mGID = gline.mGID;
    mAttr = gline.mAttr;
  }
};

#endif



