#ifndef __GLIMSDATASET_H
#define __GLIMSDATASET_H

#include "xmlserializeable.h"
#include "glimslinedata.h"
#include "glimsgiddata.h"
#include "session.h"
#include "imageinf.h"
#include "linedefset.h"

/**
 * A structure holding all data that pertains to a
 * GLIMS ingest.  This provides serialization routines
 * that branch out into the various sub-datasets.
 */
class GLIMSDataset : public XMLSerializeable {
protected:
  GLIMSLineData       mLineData;
  GLIMSGIDData        mGIDData;
  Session             mSession;
  ImageInf            mImgInf;
  Image               *mImg;

public:
  GLIMSDataset( Image *img );
  virtual bool toXML( QDomDocument &doc,
                      QDomElement &elem,
                      std::string id="" );

  virtual bool fromXML( QDomElement &elem );

  GLIMSLineData& getLineData( ) {
    return mLineData;
  }
  GLIMSGIDData&  getGIDData( ) {
    return mGIDData;
  }
  Session&       getSession( ) {
    return mSession;
  }

  /**
   * Return a copy of the Image Information
   */
  ImageInf&      getImageInf( ) {
    return mImgInf;
  }

  // make updates to project as needed based on project version
  void           updateVersion (QDate * currentVersion) ;
};

#endif




