#include "glimsgiddata.h"
#include <iostream>

bool GLIMSGIDData::fromXML( QDomElement &elem ) {
  Node *xynode, *llnode;
  if ( std::string( "GlacierDefinitionSet" ).compare( (const char*)elem.tagName() ) )
    return false;

  QDomNodeList nodelist =
  elem.elementsByTagName( "GlacierIDDef" );
  QDomElement e;
  for ( unsigned int inode=0; inode < nodelist.count(); inode++ ) {
    xynode = new Node;
    llnode = new Node;
    e = nodelist.item( inode ).toElement();
    GlacierIDDef gid;
    gid.fromXML( e );
    mDefSet.push_back( gid );
    llnode->x = gid.mLon;
    llnode->y = gid.mLat;
    *xynode = *llnode;
    mXYSet.push_back( xynode );
    mLLSet.push_back( llnode );
  }

  transformDataset();
  emit datasetChanged();
  return true;
}

bool GLIMSGIDData::toXML( QDomDocument &doc,
                          QDomElement &elem,
                          std::string id ) {
  QDomElement base = doc.createElement( "GlacierDefinitionSet" );
  if ( id != "" )
    base.setAttribute( "id", (char*)id.c_str() );
  elem.appendChild( base );

  for ( unsigned int i=0; i < mDefSet.size(); i++ )
    mDefSet[i].toXML( doc, base );

  return true;
}

void GLIMSGIDData::addNode( Node &node, int ) {
  Node *xynode, *llnode;
  GlacierIDDef gid;

  xynode = new Node;
  llnode = new Node;
  *xynode = node;
  *llnode = node;
  // IF THIS DOESN'T HAPPEN -> LAT/LON VALUES WILL REMAIN XY
  if ( mImg ) {
    mImg->getLL( llnode->x, llnode->y );
  }

  gid.mLat = llnode->y;
  gid.mLon = llnode->x;
  mDefSet.push_back( gid );
  mXYSet.push_back( xynode );
  mLLSet.push_back( llnode );

  clearRedo() ;
  emit datasetChanged();
}

void GLIMSGIDData::delNode( int inode ) {
  if ( inode >= 0 && inode < (int)mXYSet.size() ) {
    delete mXYSet[inode];
    delete mLLSet[inode];

    mXYSet.erase ( mXYSet.begin() + inode );
    mLLSet.erase ( mLLSet.begin() + inode );
    mDefSet.erase( mDefSet.begin() + inode );
  }
  emit datasetChanged();
}

void GLIMSGIDData::moveNode( Node &node, int inode ) {
  if ( inode < 0 || inode >= (int)mXYSet.size() ) return;
  Node *xynode, *llnode;

  xynode = new Node;
  llnode = new Node;
  *xynode = node;
  *llnode = node;

  if ( mImg )
    mImg->getLL( llnode->x, llnode->y );
  GlacierIDDef &def = mDefSet[inode];
  def.mLat = llnode->y;
  def.mLon = llnode->x;
  mXYSet[inode] = xynode;
  mLLSet[inode] = llnode;

  clearRedo() ;
  emit datasetChanged();
}

void GLIMSGIDData::insertNode( Node &, int ) {

}

void GLIMSGIDData::push( ) {
  std::vector<Shape*>     storeLLSet;
  std::vector<Shape*>     storeXYSet;

  for ( int ishp=0; ishp < (int)mLLSet.size(); ishp++ ) {
    Shape *lls = mLLSet[ishp];
    Shape *xys = mXYSet[ishp];
    Node *oldxynode = dynamic_cast<Node*>( xys );
    Node *oldllnode = dynamic_cast<Node*>( lls );
    Node *newllnode = new Node;
    Node *newxynode = new Node;
    *newllnode = *oldllnode;
    *newxynode = *oldxynode;
    storeLLSet.push_back( newllnode );
    storeXYSet.push_back( newxynode );
  }


  mUndoGIDDef.push ( mDefSet ) ;
  mUndoLL.push ( storeLLSet );
  mUndoXY.push ( storeXYSet );
  mUndoSel.push( mSelSet );
}

void GLIMSGIDData::pushRedo( ) {
  std::vector<Shape*>     storeLLSet;
  std::vector<Shape*>     storeXYSet;

  for ( int ishp=0; ishp < (int)mLLSet.size(); ishp++ ) {
    Shape *lls = mLLSet[ishp];
    Shape *xys = mXYSet[ishp];
    Node *oldxynode = dynamic_cast<Node*>( xys );
    Node *oldllnode = dynamic_cast<Node*>( lls );
    Node *newllnode = new Node;
    Node *newxynode = new Node;
    *newllnode = *oldllnode;
    *newxynode = *oldxynode;
    storeLLSet.push_back( newllnode );
    storeXYSet.push_back( newxynode );
  }

  mRedoGIDDef.push ( mDefSet ) ;
  mRedoLL.push( storeLLSet );
  mRedoXY.push( storeXYSet );
  mRedoSel.push( mSelSet );
}

void GLIMSGIDData::pop( ) {

  // do undo
  if ( mUndoLL.size() > 0 ) {

    // push current state on redo
    pushRedo() ;
    // clear current dataset
    clear() ;

    // set current set to stored set
    mLLSet  = mUndoLL.top();
    mXYSet  = mUndoXY.top();
    mSelSet = mUndoSel.top();
    mDefSet = mUndoGIDDef.top() ;

    mUndoLL.pop();
    mUndoXY.pop();
    mUndoSel.pop();
    mUndoGIDDef.pop() ;
  }
  emit datasetChanged();
}

void GLIMSGIDData::redo( ) {

  // do redo
  if ( mRedoLL.size() > 0 ) {

    // push info from redo to undo
    push() ;
    // clear current dataset
    clear () ;

    // set current set to stored set
    mLLSet  = mRedoLL.top();
    mXYSet  = mRedoXY.top();
    mSelSet = mRedoSel.top();
    mDefSet = mRedoGIDDef.top() ;

    mRedoLL.pop();
    mRedoXY.pop();
    mRedoSel.pop();
    mRedoGIDDef.pop() ;
  }
  emit datasetChanged();
}

void GLIMSGIDData::clearRedo() {
  if (mRedoLL.size() > 0) {
    // clear out current set
    for ( int ishp=0; ishp < (int)(mRedoLL.top()).size(); ishp++ ) {

      Shape *lls = (mRedoLL.top())[ishp];
      Shape *xys = (mRedoXY.top())[ishp];
      Node *oldxynode = dynamic_cast<Node*>( xys );
      Node *oldllnode = dynamic_cast<Node*>( lls );
      delete oldxynode;
      delete oldllnode;

    }
//    mRedoXY.pop() ;
//    mRedoLL.pop() ;
//    mRedoSel.pop() ;
//    mRedoGIDDef.pop() ;

    for (unsigned int i = 1 ; i <= mRedoXY.size() ; i++) {
      mRedoXY.pop() ;
      mRedoLL.pop() ;
      mRedoSel.pop() ;
      mRedoGIDDef.pop() ;
    }
  }
}

void GLIMSGIDData::clear() {
  if (mLLSet.size() > 0) {
    // clear out current set
    for ( int ishp=0; ishp < (int)mLLSet.size(); ishp++ ) {

      Shape *lls = mLLSet[ishp];
      Shape *xys = mXYSet[ishp];
      Node *oldxynode = dynamic_cast<Node*>( xys );
      Node *oldllnode = dynamic_cast<Node*>( lls );
      delete oldxynode;
      delete oldllnode;

    }

    mXYSet.clear();
    mLLSet.clear();
    mSelSet.clear();
  }
}


void GLIMSGIDData::addGID( const GlacierIDDef &gid ) {
  Node *n, *ll;
  n = new Node;
  ll = new Node;
  ll->x = gid.mLon;
  ll->y = gid.mLat;
  ll->z = 0.0;
  *n = *ll;
  mImg->getXY( n->x, n->y );
  mXYSet.push_back( n );
  mLLSet.push_back( ll );
  mDefSet.push_back( gid );

  clearRedo() ;
  emit datasetChanged();
}

void GLIMSGIDData::updateVersion (QDate * projectVersion) {
  for ( unsigned int i=0; i < mDefSet.size(); i++ ) {
    mDefSet[i].updateVersion(projectVersion);
  }
  emit datasetChanged();

}


