#ifndef __GLIMSGLOBALS_H
#define __GLIMSGLOBALS_H

#include "attr.h"
#include "linedefset.h"
#include <vector>
#include <string>
#include <qcolor.h>
#include <qfile.h>
#include <qdom.h>

class LineDefSet;

/**
 * This provides various global data that corresponds to the
 * GLIMS specification.  This includes default types/values
 * within the GLIMS database.  Most of the data comes from XML
 * serialized files located in the globals file in the Glimsviewer
 * file structure.  Available display attributes are available here
 * as well.
 */

class GLIMSGlobals {
public:
  static bool                 mGlbsLoaded;

  // LINE ATTRIBUTES
  static std::vector<Attr>    mGlbGlacTypes;
  static std::vector<Attr>    mGlbMDF;
  static std::vector<Attr>    mGlbTypes;
  static std::vector<Attr>    mGlbFtrs;
  static std::vector<Attr>    mGlbLbls;

  // GLACIER ATTRIBUTES
  static std::vector<Attr>    mGlbPrimCsfn;
  static std::vector<Attr>    mGlbForm;
  static std::vector<Attr>    mGlbFrntChar;
  static std::vector<Attr>    mGlbLonChar;
  static std::vector<Attr>    mGlbDomMassSrc;
  static std::vector<Attr>    mGlbTongueAct;

  // GLOBAL ATTRIBUTE FILE NAMES
  static const std::string GLDF_FILE;    // LINE DEFINITIONS
  static const std::string GMDF_FILE;    // SEGMENT MATERIALS
  static const std::string GTYPE_FILE;   // SEGMENT TYPES
  static const std::string GLBL_FILE;    // SEGMENT LABELS
  static const std::string GFTR_FILE;    // SEGMENT FEATURES
  static const std::string GCAT_FILE;    // SEGMENT CATAGORIES
  static const std::string GPCLASS_FILE; // GLACIER PRIMARY CLASSIFICATIONS
  static const std::string GFORM_FILE;   // GLACIER FORMATION
  static const std::string GFCHAR_FILE;  // GLACIER FRONTAL CHARACTERISTICS
  static const std::string GLCHAR_FILE;  // GLACIER LONGITUDINAL CHARACTERISTICS
  static const std::string GDOMMS_FILE;  // GLACIER DOMINANT MASS SOURCE
  static const std::string GTGACT_FILE;  // GLACIER TONGUE ACTIVITY

  static LineDefSet                mGlbLDFSet;

  static std::vector<QColor>       AVAILCOLORS;
  static std::vector<std::string>  AVAILCOLORSTR;

  static void loadGlobals( );
  static void readAttrFile( std::vector<Attr> &, std::string fname );


  // get the ID of an attribute for a given value
  static std::string getIDOf    (const std::vector<Attr> glacierAttribute, const std::string attributeValue) ;
  static int         getIntIDOf (const std::vector<Attr> glacierAttribute, const std::string attributeValue) ;

  // get the index where this value is stored in the vector
  static int getNdxOfValue (const std::vector<Attr> glacierAttribute, const std::string attributeValue) ;
  static int getNdxOfID    (const std::vector<Attr> glacierAttribute, const int attributeID) ;
  static int getNdxOfID    (const std::vector<Attr> glacierAttribute, const std::string attributeID) ;
};

#endif


