#ifndef HISTOGRAMDATASOURCE_H
#define HISTOGRAMDATASOURCE_H

// QT
#include <qobject.h>

// LOCAL
#include "histogramset.h"
#include "histogramdlg.h"

class HistogramDlg;

/**
 * An abstract interface that defines the necessary requirements
 * of a histogram datasource.  This being any object that can produce
 * data that can generate Histogram's.
 *
 * This is an interface of which a histogram data source
 * can derive.  The HistogramDlg constructor takes a set of
 * these and adds the set to it's menubar under datasources.
 * The extending class should override the method getHistName,
 * which returns the name that would be displayed in the menubar,
 * and the method getHistSet, which will return a HistogramSet(RGB).
 *
 * NOTE:   It was intended to have a signal based method for
 *         alerting HistogramDlg when it would need to reload
 *         itself( datasets, display ), i.e. when the dataset
 *         changes in the WholeView after it's resized.  This
 *         wasn't possible due to the nature of the extending
 *         classes being QObjects themselves since this would
 *         have to be a QObject in order to produce signals.
 *         This is why this method has a HistogramDlg* member
 *         variable and the setHistogramDlg method.  When a
 *         HistogramDataSource is passed in the set of the
 *         HistogramDlg, it will be set with a reference to
 *         the HistogramDlg.  That way it can use
 *         HistogramDlg::reloadHistSet().
 *
 *    LAST MODIFIED:
 *      Oct 9, 2002
 *      Dan Mingus
 *
 * \ingroup histogram
 */
class HistogramDataSource {
protected:
  HistogramDlg *mHistDlg;

public:
/**
 * Does nothing but set the HistogramDlg* to NULL.
 */
  HistogramDataSource( ) : mHistDlg( NULL ) {
  }

  virtual char* getHistName( ) const =0;
  virtual HistogramSet getHistSet( ) const =0;

  virtual void setHistogramDlg( HistogramDlg *hdlg ) {
    mHistDlg = hdlg;
  }
};
#endif




