#ifndef HISTOGRAMUSER_H
#define HISTOGRAMUSER_H

#include <qpointarray.h>
#include <qobject.h>
#include <iostream>

/**
 * This provides routines to use HistogramDlg settings
 * from the user.  A look up table generation routine is
 * provided that uses the points linear stretch points
 * from the HistogramDlg.  A scaling routine is provided
 * that scales a char array with the lookup table.
 *
 */
class HistogramUser : public QObject {
  Q_OBJECT
  protected:
  QPointArray   mPnts[3];
  unsigned char mCTable[3][256];

  virtual void calcCTable( ) {
    //\todo initialize variables to get rid of compiler errors
    double min1 = 0, min2 = 0, min3 = 0;
    double max1 = 0, max2 = 0, max3 = 0;
    double scaleX1 = 0, scaleX2 = 0, scaleX3 = 0;
    double m = 0;

    QPointArray arr[3];
    bool isColor = (!mPnts[1].isNull() && !mPnts[2].isNull());

    arr[0] = mPnts[0];
    arr[1] = mPnts[1];
    arr[2] = mPnts[2];

    min1 = arr[0].at(0).x();
    max1 = arr[0].at(arr[0].size()-1).x();
    if ( isColor ) {
      min2 = arr[1].at(0).x();
      max2 = arr[1].at(arr[1].size()-1).x();
      min3 = arr[2].at(0).x();
      max3 = arr[2].at(arr[2].size()-1).x();
    }

    scaleX1 = (max1-min1) / 255.0;
    if ( isColor ) {
      scaleX2 = (max2-min2) / 255.0;
      scaleX3 = (max3-min3) / 255.0;
    }

    mCTable[0][0] = arr[0].at(0).y();
    mCTable[0][255] = arr[0].at(arr[0].size()-1).y();

    if ( isColor ) {
      mCTable[1][0] = arr[1].at(0).y();
      mCTable[1][255] = arr[1].at(arr[1].size()-1).y();

      mCTable[2][0] = arr[2].at(0).y();
      mCTable[2][255] = arr[2].at(arr[2].size()-1).y();
    }

    for ( int iVal=1; iVal < 255; iVal++ ) {
      double val1 = iVal*scaleX1 + min1;
      //\todo initialize variables to get rid of compiler errors
      double val2 = 0 ;
      double val3 = 0 ;

      if ( isColor ) {
        val2 = iVal*scaleX2 + min2;
        val3 = iVal*scaleX3 + min3;
      }

      unsigned int iPnt;

      // FIND POINTS RED
      for ( iPnt=1; iPnt < arr[0].size(); iPnt++ ) {
        if ( arr[0].at(iPnt).x() > val1 )
          break;
      }

      m = (double)( arr[0].at(iPnt).y() - arr[0].at(iPnt-1).y() ) /
          (double)( arr[0].at(iPnt).x() - arr[0].at(iPnt-1).x() );

      mCTable[0][iVal] = (unsigned char)(m*(val1-arr[0].at(iPnt).x()) + arr[0].at(iPnt).y());

      if ( isColor ) {
        // FIND POINTS GREEN
        for ( iPnt=1; iPnt < arr[1].size(); iPnt++ ) {
          if ( arr[1].at(iPnt).x() > val2 )
            break;
        }

        m = (double)( arr[1].at(iPnt).y() - arr[1].at(iPnt-1).y() ) /
            (double)( arr[1].at(iPnt).x() - arr[1].at(iPnt-1).x() );

        mCTable[1][iVal] = (unsigned char)(m*(val2-arr[1].at(iPnt).x()) + arr[1].at(iPnt).y());

        // FIND POINTS RED
        for ( iPnt=1; iPnt < arr[2].size(); iPnt++ ) {
          if ( arr[2].at(iPnt).x() > val3 )
            break;
        }

        m = (double)( arr[2].at(iPnt).y() - arr[2].at(iPnt-1).y() ) /
            (double)( arr[2].at(iPnt).x() - arr[2].at(iPnt-1).x() );

        mCTable[2][iVal] = (unsigned char)(m*(val3-arr[2].at(iPnt).x()) + arr[2].at(iPnt).y());
      } else {
        memcpy( mCTable[1], mCTable[0], 256 );
        memcpy( mCTable[2], mCTable[0], 256 );
      }

    }
  }

public:

  HistogramUser( ) {
    reset();
  }

  void reset( ) {
    // NULLING CTABLE
    for ( int i=0; i < 256; i++ ) {
      mCTable[0][i] = i;
      mCTable[1][i] = i;
      mCTable[2][i] = i;
    }
  }

  virtual ~HistogramUser( ) {
  }

  virtual const QPointArray* getHistPnts( ) {
    return mPnts;
  }

  virtual void setHistPnts( QPointArray pnts[3] ) {
    mPnts[0] = pnts[0].copy();
    mPnts[1] = pnts[1].copy();
    mPnts[2] = pnts[2].copy();
    calcCTable();
    emit ctableChanged();
  }

  void scaleVal( unsigned char *data, int len ) {
    for ( int i=0; i < len; i+=4 ) {
      data[i+2] = mCTable[0][data[i+2]];     // reds
      data[i+1] = mCTable[1][data[i+1]];     // greens
      data[i]   = mCTable[2][data[i]];       // blues
    }
  }

  signals:
  void ctableChanged( );
};

#endif



