#ifndef __IMAGEDLG_H
#define __IMAGEDLG_H

#include <qdialog.h>
#include <qlayout.h>
#include <qvbox.h>
#include <qhbox.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qcheckbox.h>
#include <qgrid.h>
#include <qfiledialog.h>

#include <vector>
#include <string>

/**
 * A user interface dialog that provides the user with
 * choices that define the information needed to open an
 * image with glimsview.  A combo box and associated check
 * box are provided to allow the user to define a multi file
 * image.  For example, LSAT 7 GeoTIFF formats are a group of
 * GeoTIFF files with a standardized naming convention.  An
 * individual GeoTIFF could be opened without defining it as
 * a LSAT image, or all bands that fit the chosen files prefix
 * will be loaded allowing multi band views, etc.
 */

class ImageDlg : public QDialog {
  Q_OBJECT
  protected:
  QLineEdit           *mTxtImg;
  QCheckBox           *mChkMulti;
  QComboBox           *mCmbType;
  std::vector<std::string>         mTypeSet;

  std::string         &mImg;
  int                 &mType;

  void buildDialog( );
  void loadTypes( );

public:
  ImageDlg( std::string &img, int &type );

public slots:
  void setImg( );
  void create( );
  void chkToggle( bool );

};

#endif





