#include "imageinf.h"
#include <qmessagebox.h>
#include <iostream>


bool ImageInf::toXML( QDomDocument &doc,
                      QDomElement &elem,
                      std::string id ) {
  QDomElement base = doc.createElement( "ImageInf" );
  if ( id != "" )
    base.setAttribute( "id", (char*)id.c_str() );

  elem.appendChild( base );

  writeTextTag( doc, base, "ImageID", mImageID );
  writeTextTag( doc, base, "InstID", mInstID );
  writeTextTag( doc, base, "OrigID", mOrigID );
  writeTextTag( doc, base, "LocURL", mLocURL );
  writeTextTag( doc, base, "AquiDateTime", mAquiDateTime );

  writeDoubleTag( doc, base, "CenterLat", mCenterLat );
  writeDoubleTag( doc, base, "CenterLon", mCenterLon );
  writeDoubleTag( doc, base, "CenterLonUnc", mCenterLonUnc );
  writeDoubleTag( doc, base, "CenterLatUnc", mCenterLatUnc );
  writeDoubleTag( doc, base, "ImageAzim", mImageAzim );
  writeDoubleTag( doc, base, "CloudPct", mCloudPct );
  writeDoubleTag( doc, base, "SunAzim", mSunAzim );
  writeDoubleTag( doc, base, "SunElev", mSunElev );
  writeDoubleTag( doc, base, "InstrumentAzimuth", mInstrument_azimuth );
  writeDoubleTag( doc, base, "InstrumentZenith",  mInstrument_zenith );

  writeTextTag( doc, base, "Projection", mProjection );

  mP1.toXML( doc, base, "UL" );
  mP2.toXML( doc, base, "UR" );
  mP3.toXML( doc, base, "LR" );
  mP4.toXML( doc, base, "LL" );

  return true;
}

bool ImageInf::fromXML( QDomElement &elem ) {
  if ( elem.isNull() )
    return false;

  if ( elem.hasAttribute( "id" ) )
    mID = (const char*)elem.attribute( "id" );

  if ( std::string( "ImageInf" ).compare( (const char*)elem.tagName() ) )
    return false;

  mImageID = readTextTag( elem, "ImageID" );
  mInstID  = readTextTag( elem, "InstID" );
  mOrigID  = readTextTag( elem, "OrigID" );
  mLocURL  = readTextTag( elem, "LocURL" );

  mAquiDateTime = readTextTag( elem, "AquiDateTime" );

  mCenterLat    = readDoubleTag( elem, "CenterLat" );
  mCenterLon    = readDoubleTag( elem, "CenterLon" );
  mCenterLonUnc = readDoubleTag( elem, "CenterLonUnc" );
  mCenterLatUnc = readDoubleTag( elem, "CenterLatUnc" );

  mImageAzim = readDoubleTag( elem, "ImageAzim" );
  mCloudPct  = readDoubleTag( elem, "CloudPct" );
  mSunAzim   = readDoubleTag( elem, "SunAzim" );
  mSunElev   = readDoubleTag( elem, "SunElev" );

  mInstrument_azimuth = readDoubleTag( elem, "InstrumentAzimuth" );
  mInstrument_zenith  = readDoubleTag( elem, "InstrumentZenith" );

  mProjection = readTextTag( elem, "Projection" );

  QDomNodeList nodeset = elem.elementsByTagName( "Node" );
  for ( unsigned int i=0; i < nodeset.count(); i++ ) {
    QDomElement e = nodeset.item( i ).toElement();
    std::string id = (const char*)e.attribute( "id" );
    if ( id == "UL" ) {
      mP1.fromXML( e );
    } else if ( id == "UR" ) {
      mP2.fromXML( e );
    } else if ( id == "LR" ) {
      mP3.fromXML( e );
    } else if ( id == "LL" ) {
      mP4.fromXML( e );
    }
  }


  return true;
}



