#ifndef __IMAGEINF_H
#define __IMAGEINF_H

#include "windows_defs.h"

#include <qdom.h>
#include <string>
#include "shape.h"
#include "xmlserializeable.h"

/**
 * A dataset that hold all image information that is
 * containable within the GLIMS database.  This is
 * a class representation of the images shapefile
 * described in the GLIMS data transfer specifications.
 *
 * @history 2006-01-09 dls -
 *    Tweaked as necessary to make sure the image metadata
 *    were being captured and stored
 *
 * @todo 2006-01-23 dls -
 *    Need to confirm all the information required by
 *    the GLIMS database is being captured and stored
 */

class GV_EXPORT ImageInf : public XMLSerializeable {
public:
  std::string         mImageID;
  std::string         mInstID;
  std::string         mOrigID;
  std::string         mLocURL;
  std::string         mAquiDateTime;
  double              mCenterLat;
  double              mCenterLon;
  double              mCenterLatUnc;
  double              mCenterLonUnc;
  double              mImageAzim;
  double              mCloudPct;
  double              mSunAzim;
  double              mSunElev;
  double              mInstrument_azimuth;
  double              mInstrument_zenith;
  std::string         mProjection;

  Node          mP1;
  Node          mP2;
  Node          mP3;
  Node          mP4;

  ImageInf( ) {
    mCenterLat          = 0.0;
    mCenterLon          = 0.0;
    mCenterLatUnc       = 0.0;
    mCenterLonUnc       = 0.0;
    mImageAzim          = 0.0;
    mCloudPct           = 0.0;
    mSunAzim            = 0.0;
    mSunElev            = 0.0;
    mInstrument_azimuth = 0.0;
    mInstrument_zenith  = 0.0;
  }

  virtual ~ImageInf( ) {
  }

  virtual bool toXML( QDomDocument &doc,
                      QDomElement &elem,
                      std::string id = "" );

  virtual bool fromXML( QDomElement &elem );


};


#endif




