#include "imageinfdlg.h"
#include <cstdlib>
#include <qmessagebox.h>

ImageInfDlg::ImageInfDlg( ImageInf &imginf ) :
mImgInf( imginf ) {
  QVBoxLayout     *vb = new QVBoxLayout( this );
  QHBox           *hb; //!< Reusable QHbox
  QHBox           *fnb;
  char             buf[50];


  vb->setMargin( 8 );
  vb->setAutoAdd( true );

// file name box
  fnb = new QHBox( this );
  new QLabel( "Image URL: ", fnb );
  fne = new QLineEdit( fnb );
  QToolTip::add (fne, "Location (URL) of image") ;
  fne->setText( mImgInf.mLocURL.c_str() );

  connect( fne,
           SIGNAL( textChanged( const QString & ) ),
           this,
           SLOT( fnchanged( const QString & ) ) );

// acquisition date box
  hb = new QHBox( this );
  new QLabel( "Acqusition Date/Time: ", hb );
  le = new QLineEdit( hb );
  QToolTip::add (le, "Image Acquisition Date and Time") ;
  le->setText( mImgInf.mAquiDateTime.c_str() );

  checkLe = new QPushButton("Check", hb) ;
  QToolTip::add (checkLe, "Check Image Acquisition Date-Time format") ;

  connect( le,
           SIGNAL( textChanged( const QString & ) ),
           this,
           SLOT( aquchanged( const QString & ) ) );

  connect( checkLe,
           SIGNAL( pressed( ) ),
           this,
           SLOT( aquCompleted( ) ) );

// Image ID
  hb = new QHBox( this );
  new QLabel( "Image ID: ", hb );
  imgID = new QLineEdit( hb );
  QToolTip::add (imgID, "Image ID") ;
  imgID->setText( mImgInf.mImageID.c_str() );

  connect( imgID,
           SIGNAL( textChanged( const QString & ) ),
           this,
           SLOT( imgIdChanged( const QString & ) ) );

// Instrument ID
  hb = new QHBox( this );
  new QLabel( "Instrument ID ", hb );
  instID = new QLineEdit( hb );
  QToolTip::add (instID, "Instrument ID") ;
  instID->setText( mImgInf.mInstID.c_str() );

  connect( instID,
           SIGNAL( textChanged( const QString & ) ),
           this,
           SLOT( instIdChanged( const QString & ) ) );

// Origin ID
  hb = new QHBox( this );
  new QLabel( "Origin ID: ", hb );
  origID = new QLineEdit( hb );
  QToolTip::add (origID, "Origin ID") ;
  origID->setText( mImgInf.mOrigID.c_str() );

  connect( origID,
           SIGNAL( textChanged( const QString & ) ),
           this,
           SLOT( origIdChanged( const QString & ) ) );

// Center Lat
  hb = new QHBox( this );
  new QLabel( "Center Latitude (): ", hb );
  cLat = new QLineEdit( hb );
  QToolTip::add (cLat, "Center Latitude, degrees ()") ;
  sprintf( buf, "%8.4f", mImgInf.mCenterLat );
  cLat->setText( buf );

  connect( cLat,
           SIGNAL( textChanged( const QString & ) ),
           this,
           SLOT( cLatChanged( const QString & ) ) );

// Center Lon
  hb = new QHBox( this );
  new QLabel( "Center Longitude (): ", hb );
  cLon = new QLineEdit( hb );
  QToolTip::add (cLon, "Center Longitude, degrees ()") ;
  sprintf( buf, "%9.4f", mImgInf.mCenterLon );
  cLon->setText( buf );

  connect( cLon,
           SIGNAL( textChanged( const QString & ) ),
           this,
           SLOT( cLonChanged( const QString & ) ) );

// Center Lat Uncertainty
  hb = new QHBox( this );
  new QLabel( "Center Latitude Uncertainty (m): ", hb );
  cLatUnc = new QLineEdit( hb );
  QToolTip::add (cLatUnc, "Center Latitude Uncertainty, degrees ()") ;
  sprintf( buf, "%8.4f", mImgInf.mCenterLatUnc );
  cLatUnc->setText( buf );

  connect( cLatUnc,
           SIGNAL( textChanged( const QString & ) ),
           this,
           SLOT( cLatUncChanged( const QString & ) ) );

// Center Lon Uncertainty
  hb = new QHBox( this );
  new QLabel( "Center Longitude Uncertainty (m): ", hb );
  cLonUnc = new QLineEdit( hb );
  QToolTip::add (cLonUnc, "Center Longitude Uncertainty, degrees ()") ;
  sprintf( buf, "%9.4f", mImgInf.mCenterLonUnc );
  cLonUnc->setText( buf );

  connect( cLonUnc,
           SIGNAL( textChanged( const QString & ) ),
           this,
           SLOT( cLonUncChanged( const QString & ) ) );


// Image Azimuth
  hb = new QHBox( this );
  new QLabel( "Image Azimuth (): ", hb );
  imgAz = new QLineEdit( hb );
  QToolTip::add (imgAz, "Image azimuth, degrees ()") ;
  sprintf( buf, "%9.4f", mImgInf.mImageAzim );
  imgAz->setText( buf );

  connect( imgAz,
           SIGNAL( textChanged( const QString & ) ),
           this,
           SLOT( imgAzChanged( const QString & ) ) );

// Percentage of Cloud Cover
  hb = new QHBox( this );
  new QLabel( "Cloud Cover (%): ", hb );
  cloudPct = new QLineEdit( hb );
  QToolTip::add (cloudPct, "Percentage of cloud cover in the image") ;
  sprintf( buf, "%9.4f", mImgInf.mCloudPct );
  cloudPct->setText( buf );

  connect( cloudPct,
           SIGNAL( textChanged( const QString & ) ),
           this,
           SLOT( cloudPctChanged( const QString & ) ) );

// Sun Azimuth
  hb = new QHBox( this );
  new QLabel( "Sun Azimuth (): ", hb );
  sunAz = new QLineEdit( hb );
  QToolTip::add (sunAz, "Sun Azimuth ()") ;
  sprintf( buf, "%9.4f", mImgInf.mSunAzim );
  sunAz->setText( buf );

  connect( sunAz,
           SIGNAL( textChanged( const QString & ) ),
           this,
           SLOT( sunAzChanged( const QString & ) ) );

// Sun Elevation
  hb = new QHBox( this );
  new QLabel( "Sun Elevation (): ", hb );
  sunElev = new QLineEdit( hb );
  QToolTip::add (sunElev, "Sun elevation, degrees ()") ;
  sprintf( buf, "%9.4f", mImgInf.mSunElev );
  sunElev->setText( buf );

  connect( sunElev,
           SIGNAL( textChanged( const QString & ) ),
           this,
           SLOT( sunElevChanged( const QString & ) ) );

// Instrument Azimuth
  hb = new QHBox( this );
  new QLabel( "Instrument Azimuth (): ", hb );
  instAz = new QLineEdit( hb );
  QToolTip::add (instAz, "Instrument azimuth, degrees ()") ;
  sprintf( buf, "%9.4f", mImgInf.mInstrument_azimuth );
  instAz->setText( buf );

  connect( instAz,
           SIGNAL( textChanged( const QString & ) ),
           this,
           SLOT( instAzChanged( const QString & ) ) );

// Instrument Zenith
  hb = new QHBox( this );
  new QLabel( "Instrument Zenith (): ", hb );
  instZen = new QLineEdit( hb );
  QToolTip::add (instZen, "Instrument zenith, degrees ()") ;
  sprintf( buf, "%9.4f", mImgInf.mInstrument_zenith );
  instZen->setText( buf );

  connect( instZen,
           SIGNAL( textChanged( const QString & ) ),
           this,
           SLOT( instZenChanged( const QString & ) ) );

//
  hb = new QHBox( this );
  new QLabel( "Projection: ", hb );
  proj = new QLineEdit( hb );
  QToolTip::add (proj, "Projection name") ;
  proj->setText( mImgInf.mProjection.c_str() );

  connect( proj,
           SIGNAL( textChanged( const QString & ) ),
           this,
           SLOT( projChanged( const QString & ) ) );


}




// ACQUISITION DATE CHANGED
// set new acquisition date
void ImageInfDlg::aquchanged( const QString &str ) {
  mImgInf.mAquiDateTime = (const char*)str;
}

// FILE NAME CHANGED
// set new "file name" (image location URL)
void ImageInfDlg::fnchanged( const QString &str ) {
  mImgInf.mLocURL = (const char*)str;
}


// IMAGE ID CHANGED
void ImageInfDlg::imgIdChanged(const QString &str ) {
  mImgInf.mImageID = (const char*)str ;
}

// INSTRUMENT ID CHANGED
void ImageInfDlg::instIdChanged(const QString &str ) {
  mImgInf.mInstID = (const char*)str ;
}

// ORIGIN CHANGED
void ImageInfDlg::origIdChanged(const QString &str ) {
  mImgInf.mOrigID = (const char*)str ;
}

// CENTER LAT CHANGED
void ImageInfDlg::cLatChanged(const QString &str ) {
  mImgInf.mCenterLat = str.toDouble() ;
}

// CENTER LON CHANGED
void ImageInfDlg::cLonChanged(const QString &str ) {
  mImgInf.mCenterLon = str.toDouble() ;
}

// CENTER LAT UNCERTAINTY CHANGED
void ImageInfDlg::cLatUncChanged(const QString &str ) {
  mImgInf.mCenterLatUnc = str.toDouble() ;
}

// CENTER LAT UNCERTAINTY CHANGED
void ImageInfDlg::cLonUncChanged(const QString &str ) {
  mImgInf.mCenterLonUnc = str.toDouble() ;
}

// IMAGE AZIMUTH CHANGED
void ImageInfDlg::imgAzChanged(const QString &str ) {
  mImgInf.mImageAzim = str.toDouble() ;
}

// PERCENTAGE CLOUD COVER CHANGED
void ImageInfDlg::cloudPctChanged( const QString &str ) {
  mImgInf.mCloudPct = str.toDouble() ;
}

// SUN AZIMUTH CHANGED
void ImageInfDlg::sunAzChanged(const QString &str ) {
  mImgInf.mSunAzim = str.toDouble() ;
}

// SUN ELEVATION CHANGED
void ImageInfDlg::sunElevChanged(const QString &str ) {
  mImgInf.mSunElev = str.toDouble() ;
}

// CHANGED
void ImageInfDlg::instAzChanged(const QString &str ) {
  mImgInf.mInstrument_azimuth = str.toDouble() ;
}

// CHANGED
void ImageInfDlg::instZenChanged(const QString &str ) {
  mImgInf.mInstrument_zenith = str.toDouble() ;
}

// CHANGED
void ImageInfDlg::projChanged(const QString &str ) {
  mImgInf.mProjection = (const char*)str;
}




// LOAD DATA
void ImageInfDlg::loadData () {

  char         buf[50];    // STRING BUFFER

  le->setText( mImgInf.mAquiDateTime.c_str() );
  fne->setText( mImgInf.mLocURL.c_str() );

  // added by dls 2006-01-09
  imgID ->setText( mImgInf.mImageID.c_str() ) ;
  instID ->setText( mImgInf.mInstID.c_str() ) ;
  origID ->setText( mImgInf.mOrigID.c_str() ) ;

  sprintf( buf, "%8.4f", mImgInf.mCenterLat );
  cLat ->setText( buf ) ;

  sprintf( buf, "%9.4f", mImgInf.mCenterLon );
  cLon ->setText( buf ) ;

  sprintf( buf, "%8.4f", mImgInf.mCenterLatUnc );
  cLatUnc ->setText( buf ) ;

  sprintf( buf, "%9.4f", mImgInf.mCenterLonUnc );
  cLonUnc ->setText( buf ) ;

  sprintf( buf, "%9.4f", mImgInf.mImageAzim );
  imgAz ->setText( buf ) ;

  sprintf( buf, "%8.4f", mImgInf.mCloudPct );
  cloudPct ->setText( buf ) ;

  sprintf( buf, "%9.4f", mImgInf.mSunAzim );
  sunAz ->setText( buf ) ;

  sprintf( buf, "%9.4f", mImgInf.mSunElev );
  sunElev ->setText( buf ) ;

  sprintf( buf, "%9.4f", mImgInf.mInstrument_azimuth );
  instAz ->setText( buf ) ;

  sprintf( buf, "%9.4f", mImgInf.mInstrument_zenith );
  instZen ->setText( buf ) ;

  proj ->setText( mImgInf.mProjection.c_str() ) ;

}

void ImageInfDlg::aquCompleted() {
  QString str = le->text() ;
  if (str.length() == 0) {
    QString errmsg = "Image Acquisition Date-Time is empty" ;
    QMessageBox::information( NULL, "Date-Time Check", errmsg );
  }
  else if ( validIso8601DateTime(str) == false ) {
    QString errmsg = str + " is not correctly formatted. Please provide dates in the following format:\n YYYY-MM-DD\n YYYY-MM-DDThh:mmTZD\n YYYY-MM-DDThh:mm:ssTZD\n YYYY-MM-DDThh:mm:ss.sTZD" ;
    QMessageBox::information( NULL, "Date-Time Check", errmsg );
  }
  else {
    QString errmsg = "Image Acquisition Date-Time appears to be valid" ;
    QMessageBox::information( NULL, "Date-Time Check", errmsg );
  }
}


/*
 * Do the characters in the input string represent
 * a complete and valid date/time string?
 */
bool ImageInfDlg::validIso8601DateTime (QString &str) {
  int i = 0 ;
  bool valid = true ;

  // YEAR
  if (str.length() < 4) {
    valid = false ;
    return valid ;
  }
  else {
    // check year
    for (  ; i <= 3 ; i++ ) {
      if ( !(str[i].digitValue() >= 0 && str[i].digitValue() <= 9) ) {
        valid = false ;
      }
    }
    if (str.length() == 4) {
      return valid ;
    }
  }


  // MONTH
  if (str.length() < 7) {
    valid = false ;
    return valid ;
  }
  else {
    // check dash after year
    if (!(str[4] == '-')) {
      valid = false ;
    }
    // check month
    if ( !(str[5].digitValue() >= 0 && str[5].digitValue() <= 1) ) {
      valid = false ;
    }
    if ( !(str[6].digitValue() >= 0 && str[6].digitValue() <= 9) ) {
      valid = false ;
    }
    if (str.length() == 7) {
      return valid ;
    }
  }

  // DAY
  if (str.length() < 10) {
    valid = false ;
    return valid ;
  }
  else {
    // check dash after month
    if (!(str[7] == '-')) {
      valid = false ;
    }
    // check day
    if ( !(str[8].digitValue() >= 0 && str[8].digitValue() <= 3) ) {
      valid = false ;
    }
    if ( !(str[9].digitValue() >= 0 && str[9].digitValue() <= 9) ) {
      valid = false ;
    }
    if (str.length() == 10) {
      return valid ;
    }
    // check T after date
    if (!(str[10] == 'T')) {
      valid = false ;
    }
  }

  // TIME - HOURS/MINUTES
  if (str.length() < 22) {
    valid = false ;
    return valid ;
  }
  else {
    // check hour
    if ( !(str[11].digitValue() >= 0 && str[11].digitValue() <= 2) ) {
      valid = false ;
    }
    if ( !(str[12].digitValue() >= 0 && str[12].digitValue() <= 9) ) {
      valid = false ;
    }
    if (!(str[13] == ':')) {
      valid = false ;
    }
    // check minute
    if ( !(str[14].digitValue() >= 0 && str[14].digitValue() <= 5) ) {
      valid = false ;
    }
    if ( !(str[15].digitValue() >= 0 && str[16].digitValue() <= 9) ) {
      valid = false ;
    }


    if (str.length() >= 17) {
      i = 16 ;
      // seconds
      if ( str[i] == ':' ) {
        if ( !(str[++i].digitValue() >= 0 && str[i].digitValue() <= 5) ) {
          valid = false ;
          return valid ;
        }
        if ( !(str[++i].digitValue() >= 0 && str[i].digitValue() <= 9) ) {
          valid = false ;
          return valid ;
        }
        // decimal seconds
        if ( str[++i] == '.' ) {
          // skip past decimal places, there must be at least one
          if (str[++i].digitValue() >= 0 && str[i].digitValue() <= 9) {
            while (str[++i].digitValue() >= 0 && str[i].digitValue() <= 9) {;}
          }
          else {
            return valid ;
          }
        }
      }

      // TZD
      if (str[i] == 'Z') {
        return valid ;
      }
      else if (str[i] == '-' || str[i] == '+') {
        if ( !(str[++i].digitValue() >= 0 && str[i].digitValue() <= 2) ) {
          valid = false ;
        }
        if ( !(str[++i].digitValue() >= 0 && str[i].digitValue() <= 9) ) {
          valid = false ;
        }
        if ( !(str[++i] == ':') ) {
          valid = false ;
        }
        if ( !(str[++i].digitValue() >= 0 && str[i].digitValue() <= 5) ) {
          valid = false ;
        }
        if ( !(str[++i].digitValue() >= 0 && str[i].digitValue() <= 9) ) {
          valid = false ;
        }
      }
      else {
        valid = false ;
        return valid ;
      }
    }
  }
  return valid ;
}







