#include "inmemformat.h"
#include <algorithm>

char* InMemFormat::getMemRect( int iband,
                               int x,
                               int y,
                               int w,
                               int h,
                               int nw,
                               int nh,
                               ScaleType st ) const {
  int ow       = mMemBandSet[iband].width;
  int oh       = mMemBandSet[iband].height;
  char *orig   = mMemBandSet[iband].data;

  scaleData( &orig, x, y, w, h, ow, oh, nw ,nh,
             mMemBandSet[iband].dtype, st, false );

  return orig;
}

void InMemFormat::stateToMem( ) {
  std::vector<BandInf> &binfset = mEmulateFormat->getBandInfSet();
  std::vector<int> dellist;

  for ( int imem=0; imem < (int)mMemBandSet.size(); imem++ ) {
    if ( mMemBandSet[imem].iband == mImgState->mRed )
      continue;

    if ( mMemBandSet[imem].iband == mImgState->mGreen )
      continue;

    if ( mMemBandSet[imem].iband == mImgState->mBlue )
      continue;

    dellist.push_back( imem );
  }

  int idel;
  for ( idel=0; idel < (int)dellist.size(); idel++ )
    delete [] mMemBandSet[dellist[idel]].data;

  for ( idel=0; idel < (int)dellist.size(); idel++ )
    mMemBandSet.erase( mMemBandSet.begin() + dellist[idel] );

  if ( !inMemSet( mImgState->mRed ) ) {
    MemBandInf bmem;
    bmem.iband      = mImgState->mRed;
    if ( bmem.iband >= 0 ) {
      bmem.dtype      = binfset[ bmem.iband ].type;
      bmem.width      = binfset[ bmem.iband ].width;
      bmem.height     = binfset[ bmem.iband ].height;
      bmem.sz_dtype   = ImageFormat::dtypeSize( binfset[ bmem.iband ].type );
      bmem.data       = (char*)mEmulateFormat->getBand( bmem.iband, 0, 0, bmem.width,
                                                        bmem.height, bmem.width, bmem.height );
      mMemBandSet.push_back( bmem );
    }
  }

  if ( !inMemSet( mImgState->mGreen ) ) {
    MemBandInf bmem;
    bmem.iband      = mImgState->mGreen;
    if ( bmem.iband >= 0 ) {
      bmem.dtype      = binfset[ bmem.iband ].type;
      bmem.width      = binfset[ bmem.iband ].width;
      bmem.height     = binfset[ bmem.iband ].height;
      bmem.sz_dtype   = ImageFormat::dtypeSize( binfset[ bmem.iband ].type );
      bmem.data       = (char*)mEmulateFormat->getBand( bmem.iband, 0, 0, bmem.width,
                                                        bmem.height, bmem.width, bmem.height );
      mMemBandSet.push_back( bmem );
    }
  }

  if ( !inMemSet( mImgState->mBlue ) ) {
    MemBandInf bmem;
    bmem.iband      = mImgState->mBlue;
    if ( bmem.iband >= 0 ) {
      bmem.dtype      = binfset[ bmem.iband ].type;
      bmem.width      = binfset[ bmem.iband ].width;
      bmem.height     = binfset[ bmem.iband ].height;
      bmem.sz_dtype   = ImageFormat::dtypeSize( binfset[ bmem.iband ].type );
      bmem.data       = (char*)mEmulateFormat->getBand( bmem.iband, 0, 0, bmem.width,
                                                        bmem.height, bmem.width, bmem.height );
      mMemBandSet.push_back( bmem );
    }
  }
}

void* InMemFormat::getBand( int iband,
                            int x1,
                            int y1,
                            int w1,
                            int h1,
                            int w2,
                            int h2,
                            ScaleType st ) const {
  char *data = NULL;
  unsigned int imem;

  for ( imem=0; imem < mMemBandSet.size(); imem++ ) {
    if ( iband == mMemBandSet[imem].iband ) {
      data = getMemRect( imem, x1, y1, w1, h1, w2, h2, st );
      break;
    }
  }

  if ( data == NULL )
    data = (char*)mEmulateFormat->getBand( iband, x1, y1, w1, h1, w2, h2, st );

  return(void*)data;
}





