#ifndef __INMEMFORMAT_H
#define __INMEMFORMAT_H

#include "windows_defs.h"

#include "imageformat.h"
#include "imagestate.h"

#include <iostream>

/**
 * This is a special image format in that it does not
 * implement a file based format.  Instead it holds a
 * file based ImageFormat and masks its I/O routines
 * with its own.  At all times, the selected bands of
 * an image will be held in memory instead of the usual
 * file buffering that is done.  This allows for faster
 * access and display.  The Image class uses this a little
 * different then normal ImageFormat's.
 */

class GV_EXPORT InMemFormat : public ImageFormat {
protected:

  /**
   * Structure to metadata about the bands in an image
   */
  typedef struct {
    int         iband;
    char        *data;
    int         width;
    int         height;
    DataType    dtype;
    int         sz_dtype;
  } MemBandInf;

  ImageFormat     *mEmulateFormat;
  ImageState      *mImgState;

  std::vector<MemBandInf>     mMemBandSet;

  bool inMemSet( int iband ) {
    for ( int i=0; i < (int)mMemBandSet.size(); i++ )
      if ( mMemBandSet[i].iband == iband )
        return true;
    return false;
  }

  char* getMemRect( int iband, int x, int y,
                    int w, int h, int ow, int oh, ScaleType st ) const;


public:
  InMemFormat( ImageFormat *frmt, ImageState *imgstate ) :
  mEmulateFormat( frmt ),
  mImgState( imgstate ) {
    stateToMem();
    mBandSet = frmt->getBandInfSet();
  }

  ~InMemFormat( ) {
    for ( unsigned int imem=0; imem < mMemBandSet.size(); imem++ )
      delete [] mMemBandSet[imem].data;
  }

  void stateToMem( );

  // THIS METHOD IS NOT AVAILABLE
  virtual bool open( std::string ) {
    return false;
  }

  virtual void* getBand( int iband,
                         int x1,
                         int y1,
                         int w1,
                         int h1,
                         int w2,
                         int h2,
                         ScaleType st=SCALE_BI ) const;

  virtual void getLL( int iband, double &i, double &j ) {
    mEmulateFormat->getLL( iband, i, j );
  }

  virtual void getXY( int iband, double &i, double &j ) {
    mEmulateFormat->getXY( iband, i, j );
  }

  ImageFormat* getFormat( ) {
    return mEmulateFormat;
  }
};



#endif




