#include "l7a_fast.h"
#include <iostream>

bool L7A_Fast::open( std::string fname ) {
  const int name_len = 29;
  const std::string lc_end[] = { "hpn.fst", "hrf.fst", "htm.fst"};
  const std::string uc_end[] = { "HPN.FST", "HRF.FST", "HTM.FST"};

  if ( (int)fname.length() < name_len )
    return false;

  std::string dir = fname.substr( 0, fname.length()-name_len );
  fname = fname.substr( fname.length()-name_len, name_len );
  std::string pre = fname.substr( 0, fname.length()-7 );
  std::string testname;
  bool        opened = false;

  for ( int iband=0; iband < 3; iband++ ) {
    testname = dir + pre + lc_end[iband];
    GDALFrmt *gdal = new GDALFrmt;
    if ( gdal->open( testname ) ) {
      mFormatSet.push_back( gdal );
      std::vector<BandInf> &bset = gdal->getBandInfSet();
      for ( int i=0; i < (int)bset.size(); i++ )
        mBandSet.push_back( bset[i] );
      opened = true;
      continue;
    }

    testname = dir + pre + uc_end[iband];
    if ( gdal->open( testname ) ) {
      mFormatSet.push_back( gdal );
      std::vector<BandInf> &bset = gdal->getBandInfSet();
      for ( int i=0; i < (int)bset.size(); i++ )
        mBandSet.push_back( bset[i] );
      opened = true;
      continue;
    }

    delete gdal;
  }
  if ((int)mBandSet.size()) {
    double x = mBandSet[0].width / 2;
    double y = mBandSet[0].height / 2;
    getLL( 0, x, y );
    mImageInf.mCenterLat = y;
    mImageInf.mCenterLon = x;
  }
  mImageInf.mLocURL = fname ;


  return opened;
}

void L7A_Fast::getLL( int iband, double &i, double &j ) {
  if ( iband < 0 || iband > (int)mBandSet.size() )
    return;

  int ifrmt = getFrmt( iband );
  mFormatSet[ifrmt]->getLL( iband, i, j );
}

void L7A_Fast::getXY( int iband, double &i, double &j ) {
  if ( iband < 0 || iband > (int)mBandSet.size() )
    return;

  int ifrmt = getFrmt( iband );
  mFormatSet[ifrmt]->getXY( iband, i, j );
}

void* L7A_Fast::getBand( int iband,
                         int x1,
                         int y1,
                         int w1,
                         int h1,
                         int w2,
                         int h2,
                         ScaleType st ) const {
  if ( iband < 0 || iband > (int)mBandSet.size() )
    return NULL;

  int fiband = iband;
  int ifrmt = getFrmt( fiband );

  return mFormatSet[ifrmt]->getBand( fiband, x1, y1, w1, h1, w2, h2, st );
}

int L7A_Fast::getFrmt( int &iband ) const {
  int ifrmt = 0;
  if ( iband < 0 || iband > (int)mBandSet.size() )
    return -1;


  for ( int i=0; i < (int)mFormatSet.size(); i++ ) {
    int nband = (int)mFormatSet[i]->getNumBands();
    if ( iband >= nband ) {
      ifrmt++;
      iband -= nband;
    } else {
      break;
    }
  }

  return ifrmt;
}


