#include "l7a_geotiff.h"
#include <iostream>

bool L7A_Geotiff::open( std::string fname ) {
  const std::string band_numbers[] = {
    "10",
    "20",
    "30",
    "40",
    "50",
    "61",
    "62",
    "70",
    "80"
  };

  const int nbands = 9;
  const int name_len = 29;

  if ( (int)fname.length() < name_len )
    return false;
  std::string dir = fname.substr( 0, fname.length()-name_len );
  fname = fname.substr( fname.length()-name_len, name_len );
  std::string pre = fname.substr( 0, 2 );
  std::string mid = fname.substr( 3, 20 );
  std::string end = fname.substr( 25, 4 );
  std::string testname;
  bool        opened = false;

  QDialog *pdlg = new QDialog( NULL, NULL, true );
  QVBoxLayout *vbl = new QVBoxLayout( pdlg, 10, 10 );
  vbl->setAutoAdd( true );
  new QLabel( "Loading Files.  Please Wait.", pdlg );
  QProgressBar *pbar = new QProgressBar( pdlg );
  float pcnt = 0.0f;
  pdlg->show();
  qApp->processEvents();

  for ( int iband=0; iband < nbands; iband++ ) {
    pcnt = (1.0f / 9.0f) * (float)iband;
    pbar->setProgress( (int)(pcnt * 100.0f) );
    qApp->processEvents();
    testname = dir + pre + "1" + mid + band_numbers[iband] + end;
    GDALFrmt *gdal = new GDALFrmt;
    if ( gdal->open( testname ) ) {
      mFormatSet.push_back( gdal );
      BandInf binf = gdal->getBandInf( 0 );
      binf.name = std::string( "Band " ) + band_numbers[iband];
      mBandSet.push_back( binf );
      opened = true;
      continue;
    } else {
      delete gdal;
    }

    testname = dir + pre + "2" + mid + band_numbers[iband] + end;
    gdal = new GDALFrmt;
    if ( gdal->open( testname ) ) {
      mFormatSet.push_back( gdal );
      BandInf binf = gdal->getBandInf( 0 );
      binf.name = std::string( "Band " ) + band_numbers[iband];
      mBandSet.push_back( binf );
      opened = true;
    } else {
      delete gdal;
    }

  }

  if ((int)mBandSet.size()) {
    double x = mBandSet[0].width / 2;
    double y = mBandSet[0].height / 2;
    getLL( 0, x, y );
    mImageInf.mCenterLat = y;
    mImageInf.mCenterLon = x;
  }
  mImageInf.mLocURL = fname ;


  delete pdlg;
  return opened;
}

void L7A_Geotiff::getLL( int iband, double &i, double &j ) {
  if ( iband < 0 || iband > (int)mBandSet.size() )
    return;

  mFormatSet[iband]->getLL( iband, i, j );
}

void L7A_Geotiff::getXY( int iband, double &i, double &j ) {
  if ( iband < 0 || iband > (int)mBandSet.size() )
    return;

  mFormatSet[iband]->getXY( iband, i, j );
}

void* L7A_Geotiff::getBand( int iband,
                            int x1,
                            int y1,
                            int w1,
                            int h1,
                            int w2,
                            int h2,
                            ScaleType st ) const {
  if ( iband < 0 || iband > (int)mBandSet.size() )
    return NULL;

  return mFormatSet[iband]->getBand( 0, x1, y1, w1, h1, w2, h2, st );
}


