#include "l7a_nlaps.h"
#include <iostream>

bool L7A_Nlaps::open( std::string fname ) {
  const int name_len = 29;
  const std::string strnum[] = { "1", "2", "3"};

  if ( (int)fname.length() < name_len )
    return false;

  std::string dir = fname.substr( 0, fname.length()-name_len );
  fname = fname.substr( fname.length()-name_len, name_len );
  std::string pre = fname.substr( 0, fname.length()-1 );
  std::string testname;
  bool        opened = false;

  for ( int iband=0; iband < 3; iband++ ) {
    testname = dir + pre + strnum[iband];
    NlapsFrmt *nlaps = new NlapsFrmt;
    if ( nlaps->open( testname ) ) {
      mFormatSet.push_back( nlaps );
      std::vector<BandInf> &bset = nlaps->getBandInfSet();
      for ( int i=0; i < (int)bset.size(); i++ )
        mBandSet.push_back( bset[i] );
      opened = true;
    } else {
      delete nlaps;
    }
  }

  if ((int)mBandSet.size()) {
    double x = mBandSet[0].width / 2;
    double y = mBandSet[0].height / 2;
    getLL( 0, x, y );
    mImageInf.mCenterLat = y;
    mImageInf.mCenterLon = x;
  }
  mImageInf.mLocURL = fname ;

  // delete pdlg;
  return opened;
}

void L7A_Nlaps::getLL( int iband, double &i, double &j ) {
  if ( iband < 0 || iband > (int)mBandSet.size() )
    return;

  int ifrmt = getFrmt( iband );
  mFormatSet[ifrmt]->getLL( iband, i, j );
}

void L7A_Nlaps::getXY( int iband, double &i, double &j ) {
  if ( iband < 0 || iband > (int)mBandSet.size() )
    return;

  int ifrmt = getFrmt( iband );
  mFormatSet[ifrmt]->getXY( iband, i, j );
}

void* L7A_Nlaps::getBand( int iband,
                          int x1,
                          int y1,
                          int w1,
                          int h1,
                          int w2,
                          int h2,
                          ScaleType st ) const {
  if ( iband < 0 || iband > (int)mBandSet.size() )
    return NULL;

  int fiband = iband;
  int ifrmt = getFrmt( fiband );

  return mFormatSet[ifrmt]->getBand( fiband, x1, y1, w1, h1, w2, h2, st );
}

int L7A_Nlaps::getFrmt( int &iband ) const {
  int ifrmt = 0;
  if ( iband < 0 || iband > (int)mBandSet.size() )
    return -1;


  for ( int i=0; i < (int)mFormatSet.size(); i++ ) {
    int nband = (int)mFormatSet[i]->getNumBands();
    if ( iband >= nband ) {
      ifrmt++;
      iband -= nband;
    } else {
      break;
    }
  }

  return ifrmt;
}




