#ifndef __L7A_NLAPS_H
#define __L7A_NLAPS_H

#include <qdialog.h>
#include <qlabel.h>
#include <qprogressbar.h>
#include <qlayout.h>
#include <qapplication.h>

#include <string>
#include <vector>

#include "imageformat.h"
#include "nlapsfrmt.h"

/**
 * This is an ImageFormat implementation of the multi file
 * Landsat 7A Geotiff.  This tests the standard naming convention
 * of this format and loads all available bands within the directory
 * of the file that was chosen for opening.  This uses a GDALFrmt
 * for every loaded band and forwards its information and I/O.
 *
 * @history 2006-01-09 dls -
 *   Altered open function to pull fill in more metadata from image
 *   in ImageInf structure
 */



class L7A_Nlaps : public ImageFormat {
protected:
  std::vector<NlapsFrmt*>       mFormatSet;
  int getFrmt( int &iband ) const;

public:
  virtual bool open( std:: string fname );
  virtual void getLL( int iband, double &i, double &j );
  virtual void getXY( int iband, double &i, double &j );
  virtual void* getBand( int iband,
                         int x1,
                         int y1,
                         int w1,
                         int h1,
                         int w2,
                         int h2,
                         ScaleType st=SCALE_NN ) const;
};


#endif




