#ifndef __LINEDEF_H
#define __LINEDEF_H

#include "xmlserializeable.h"

/**
 * This is a glacier outline line definition that holds all
 * information about the attributes of a glacier outline that
 * are described in the GLIMS specifications.  Each line has
 * a reference to zero or one of these.
 */

class LineDef : public XMLSerializeable {
public:
  int         mGlacType;             // GLACIER TYPE
  int     mType;                 // SEGMENT TYPE
  int     mLabel;                // SEGMENT LABEL
  int       mLMat, mRMat;        // LEFT/RIGHT MATERIAL
  float       mLXUncert, mLYUncert;  // LOCAL X/Y UNCERTAINTY
  float   mGXUncert, mGYUncert;  // GLOBAL X/Y UNCERTAINTY
  int     mLFeature, mRFeature;  // LEFT/RIGHT FEATURE
  int       mColor;                // LINE COLOR
  int         mStyle;                // LINE STYLE
  int         mWidth;                // LINE WIDTH
  char*   mName;                 // LINE NAME

  LineDef( ) {
    mGlacType = 0;
    mType = 0;
    mLabel = 0;
    mLMat = 0;
    mRMat = 0;
    mLXUncert = 0.0f;
    mLYUncert = 0.0f;
    mGXUncert = 0.0f;
    mGYUncert = 0.0f;
    mLFeature = 0;
    mRFeature = 0;
    mColor = 0;
    mStyle = 0;
    mWidth = 0;
    mName = "";
  }

  virtual bool fromXML( QDomElement &elem ) {
    if ( std::string( "LineDefinition" ).compare( (const char*)elem.tagName() ) )
      return false;

    mGlacType = readIntTag( elem, "GlacType" );
    mType =     readIntTag( elem, "Type" );
    mLabel =    readIntTag( elem, "Label" );
    mLMat =     readIntTag( elem, "LMat" );
    mRMat =     readIntTag( elem, "RMat" );
    mLXUncert = readDoubleTag( elem, "LXUncert" );
    mLYUncert = readDoubleTag( elem, "LYUncert" );
    mGXUncert = readDoubleTag( elem, "GXUncert" );
    mGXUncert = readDoubleTag( elem, "GYUncert" );
    mLFeature = readIntTag( elem, "LFeature" );
    mRFeature = readIntTag( elem, "RFeature" );
    mColor =    readIntTag( elem, "Color" );
    mStyle =    readIntTag( elem, "Style" );
    mWidth =    readIntTag( elem, "Width" );
    mName = (char*)readTextTag(elem, "Name" ).c_str();

    return true;
  }

  virtual bool toXML( QDomDocument &doc,
                      QDomElement &elem,
                      std::string id = "" ) {
    QDomElement base = doc.createElement( "LineDefinition" );
    if ( id != "" )
      base.setAttribute( "id", (char*)id.c_str() );
    elem.appendChild( base );

    writeIntTag( doc, base, "GlacType", mGlacType );
    writeIntTag( doc, base, "Type", mType );
    writeIntTag( doc, base, "Label", mLabel );
    writeIntTag( doc, base, "LMat", mLMat );
    writeIntTag( doc, base, "RMat", mRMat );
    writeDoubleTag( doc, base, "LXUncert", mLXUncert );
    writeDoubleTag( doc, base, "LYUncert", mLYUncert );
    writeDoubleTag( doc, base, "GXUncert", mGXUncert );
    writeDoubleTag( doc, base, "GYUncert", mGYUncert );
    writeIntTag( doc, base, "LFeature", mLFeature );
    writeIntTag( doc, base, "RFeature", mRFeature );
    writeIntTag( doc, base, "Color", mColor );
    writeIntTag( doc, base, "Style", mStyle );
    writeIntTag( doc, base, "Width", mWidth );
    writeTextTag( doc, base, "Name", mName ); // dls 5/20/04 - was set to GlacType instead of Name

    return true;
  }

  bool operator==( LineDef &ldf ) {

    if ( ldf.mBaseName == mBaseName &&
         ldf.mGlacType == mGlacType &&
         ldf.mGXUncert == mGXUncert &&
         ldf.mGYUncert == mGYUncert &&
         ldf.mID == mID &&
         ldf.mLabel == mLabel &&
         ldf.mLFeature == mLFeature &&
         ldf.mLMat == mLMat &&
         ldf.mLXUncert == mLXUncert &&
         ldf.mLYUncert == mLYUncert &&
         ldf.mName == mName &&
         ldf.mRFeature == mRFeature &&
         ldf.mRMat == mRMat &&
         ldf.mType == mType )
      return true;
    return false;
  }

  bool operator!=( LineDef &ldf ) {
    return !(*this == ldf);
  }
};

#endif



