#include "linedefdlg.h"

LineDefDlg::LineDefDlg( GLIMSLineData &linedata, int ildf ) :
QDialog( NULL, NULL, true ),
mLineData( linedata ),
miLDF( ildf ) {
  setCaption( "Line Definition Config" ); // LABEL THE WINDOW

  // THE MAIN LAYOUT FOR THE FORM
  QVBoxLayout *vboxl = new QVBoxLayout( this );
  vboxl->setAutoAdd( true );

  // THE MAIN WIDGET FOR THE FORM
  QVBox *mainVB = new QVBox( this );
  mainVB->setMargin( 8 );

  // TYPES OF WIDGETS THAT WILL BE USED IN THE FORM
  QHBox *hbox      = NULL;
  QGrid *grid      = NULL;
  QVBox *vbox      = NULL;
  QLabel *lbl      = NULL;
  QPushButton *but = NULL;

  // ADD TYPE SECTION
  vbox = new QVBox( mainVB );

  grid = new QGrid( 2, vbox );
  lbl = new QLabel( "Glacier Type: ", grid );
  lbl->setMaximumWidth( 120 );
  cmbGlacType  = new QComboBox( grid, NULL );

  grid = new QGrid( 2, vbox );
  lbl = new QLabel( "Segment Type: ", grid );
  lbl->setMaximumWidth( 120 );
  cmbSegType  = new QComboBox( grid, NULL );


  // ADD SEGMENT LABEL SECTION
  grid = new QGrid( 2, vbox );
  lbl = new QLabel( "Segment Label: ", grid );
  lbl->setMaximumWidth( 120 );
  cmbSegLabel = new QComboBox( grid );


  // ADD MATERIALS SECTION
  lbl = new QLabel( "Materials", mainVB );
  grid = new QGrid( 2, mainVB );

  hbox = new QHBox( grid );
  lbl = new QLabel( "Left: ", hbox );
  cmbLMat  = new QComboBox( hbox, NULL );

  hbox = new QHBox( grid );
  lbl = new QLabel( "  Right: ", hbox );
  cmbRMat  = new QComboBox( hbox, NULL );


  // ADD FEATURE SECTION
  lbl = new QLabel( "Features", mainVB );
  grid = new QGrid( 2, mainVB );

  hbox = new QHBox( grid );
  lbl = new QLabel( "Left: ", hbox );
  cmbLFtr  = new QComboBox( hbox, NULL );

  hbox = new QHBox( grid );
  lbl = new QLabel( "  Right: ", hbox );
  cmbRFtr  = new QComboBox( hbox, NULL );


  // ADD LOCAL UNCERT SECTION
  lbl = new QLabel( "Local Uncertainties", mainVB );
  grid = new QGrid( 2, mainVB );

  hbox = new QHBox( grid );
  lbl = new QLabel( "X: ", hbox );
  txtLXUncert = new QLineEdit( hbox, NULL );

  hbox = new QHBox( grid );
  lbl = new QLabel( "  Y: ", hbox );
  txtLYUncert = new QLineEdit( hbox, NULL );


  // ADD GLOBAL UNCERT SECTION
  lbl = new QLabel( "Global Uncertainties", mainVB );
  grid = new QGrid( 2, mainVB );

  hbox = new QHBox( grid );
  lbl = new QLabel( "X: ", hbox );
  txtGXUncert = new QLineEdit( hbox, NULL );

  hbox = new QHBox( grid );
  lbl = new QLabel( "  Y: ", hbox );
  txtGYUncert = new QLineEdit( hbox, NULL );


  // ADD LINE STYLE SECTION
  lbl = new QLabel( "Dislay", mainVB );
  vbox = new QVBox( mainVB );

  grid = new QGrid( 2, vbox );
  lbl = new QLabel( "Color: ", grid );
  lbl->setMaximumWidth( 80 );
  cmbColor = new QComboBox( grid, NULL );

  grid = new QGrid( 2, vbox );
  lbl = new QLabel( "Style: ", grid );
  lbl->setMaximumWidth( 80 );
  cmbStyle = new QComboBox( grid, NULL );

  grid = new QGrid( 2, vbox );
  lbl = new QLabel( "Width: ", grid );
  lbl->setMaximumWidth( 80 );
  cmbWidth = new QComboBox( grid, NULL );


  // ADD BUTTONS FOR ADDING THE LDF OR CANCELING
  hbox = new QHBox( mainVB );

  but = new QPushButton( "Ok", hbox );
  connect( but, SIGNAL( clicked() ), this, SLOT( butOkPress() ) );

  but = new QPushButton( "Cancel", hbox );
  connect( but, SIGNAL( clicked() ), this, SLOT( butCancelPress() ) );

  // FILL THE COMBO BOXES WITH VALIDS
  loadValids();

  // LOAD SELECTED LDF DATA INTO FORM
  if ( miLDF != -1 )
    setSelected();
}

void LineDefDlg::butOkPress( ) {
  LineDef ldf; // NEW LDF

  // GRAB DATA FROM FORM & LOAD INTO ldf
  ldf.mGlacType = cmbGlacType->currentItem();
  ldf.mType     = cmbSegType->currentItem();
  ldf.mLabel    = cmbSegLabel->currentItem();
  ldf.mLMat     = cmbLMat->currentItem();
  ldf.mRMat     = cmbRMat->currentItem();
  ldf.mLXUncert = atof( txtLXUncert->text().latin1() );
  ldf.mLYUncert = atof( txtLYUncert->text().latin1() );
  ldf.mGXUncert = atof( txtGXUncert->text().latin1() );
  ldf.mGYUncert = atof( txtGYUncert->text().latin1() );
  ldf.mLFeature = cmbLFtr->currentItem();
  ldf.mRFeature = cmbRFtr->currentItem();
  ldf.mColor    = cmbColor->currentItem();
  ldf.mWidth    = cmbWidth->currentItem() +1;
  ldf.mStyle    = cmbStyle->currentItem() +1;

  // ADD OR EDIT LDF
  if ( miLDF == -1 )
    mLineData.getLDFSet().add( ldf );
  else
    mLineData.getLDFSet()[miLDF] = ldf;
  close();
}

void LineDefDlg::butCancelPress( ) {
  close();
}



void LineDefDlg::loadValids( ) {
  unsigned int i;

  for ( i = 0; i < GLIMSGlobals::mGlbGlacTypes.size(); i++ )
    cmbGlacType->insertItem( GLIMSGlobals::mGlbGlacTypes[i].value.c_str() );

  for ( i = 0; i < GLIMSGlobals::mGlbTypes.size(); i++ )
    cmbSegType->insertItem( GLIMSGlobals::mGlbTypes[i].value.c_str() );

  for ( i = 0; i < GLIMSGlobals::mGlbLbls.size(); i++ )
    cmbSegLabel->insertItem(  GLIMSGlobals::mGlbLbls[i].value.c_str() );

  for ( i = 0; i < GLIMSGlobals::mGlbMDF.size(); i++ ) {
    cmbLMat->insertItem( GLIMSGlobals::mGlbMDF[i].value.c_str() );
    cmbRMat->insertItem( GLIMSGlobals::mGlbMDF[i].value.c_str() );
  }

  for ( i = 0; i < GLIMSGlobals::mGlbFtrs.size(); i++ ) {
    cmbLFtr->insertItem( GLIMSGlobals::mGlbFtrs[i].value.c_str() );
    cmbRFtr->insertItem( GLIMSGlobals::mGlbFtrs[i].value.c_str() );
  }

  for ( i = 0; i < GLIMSGlobals::AVAILCOLORSTR.size(); i++ )
    cmbColor->insertItem( GLIMSGlobals::AVAILCOLORSTR[i].c_str() );

  cmbStyle->insertItem("Solid Line");
  cmbStyle->insertItem("Dash Line");
  cmbStyle->insertItem("Dot Line");

  cmbWidth->insertItem( "1" );
  cmbWidth->insertItem( "2" );
  cmbWidth->insertItem( "3" );
}



void LineDefDlg::setSelected( ) {
  LineDef   ldf = mLineData.getLDFSet()[miLDF];
  /**
   * \todo Make mProj->getLineDef( mProj->mSelLDF, ldf, attr ) work (note by Dan).
   */
  //mProj->getLineDef( mProj->mSelLDF, ldf, attr );
  char buf[256];

  cmbGlacType->setCurrentItem( ldf.mGlacType );
  cmbSegType->setCurrentItem( ldf.mType );
  cmbSegLabel->setCurrentItem( ldf.mLabel );
  cmbLMat->setCurrentItem( ldf.mLMat );
  cmbRMat->setCurrentItem( ldf.mRMat );
  cmbLFtr->setCurrentItem( ldf.mLFeature );
  cmbRFtr->setCurrentItem( ldf.mRFeature );
  sprintf( buf, "%.6f", ldf.mLXUncert );
  txtLXUncert->setText( buf );
  sprintf( buf, "%.6f", ldf.mLYUncert );
  txtLYUncert->setText( buf );
  sprintf( buf, "%.6f", ldf.mGXUncert );
  txtGXUncert->setText( buf );
  sprintf( buf, "%.6f", ldf.mGYUncert );
  txtGYUncert->setText( buf );

  cmbColor->setCurrentItem( ldf.mColor );
  cmbWidth->setCurrentItem( ldf.mWidth-1 );
  cmbStyle->setCurrentItem( ldf.mStyle-1 );
}





