#include "lineselectordlg.h"
#include "configdlg.h"

#include <stdio.h>

LineSelectorDlg::LineSelectorDlg( GLIMSLineData &linedata ) :
mLineData( linedata ) {
  mSelLDF = -1;

  mListView = new QListView( this );
  mListView->addColumn( "Color" );
  mListView->addColumn( "Glacier Type" );
  mListView->addColumn( "Segment Label" );
  mListView->addColumn( "Left Mat" );
  mListView->addColumn( "Right Mat" );
  mListView->addColumn( "Style" );
  mListView->addColumn( "Width" );
  mListView->addColumn( "Measured" );

  mListView->setSorting( -1 );
  // mListView->setColumnWidthMode( QListView::Maximum );

  QPushButton *pb2 = new QPushButton( "New", this );
  QPushButton *pb3 = new QPushButton( "Edit", this );
  // QPushButton *pb4 = new QPushButton( "Edit Materials", this );
  QPushButton *pb5 = new QPushButton( "Change Line", this );
  QPushButton *pb6 = new QPushButton( "Export", this );
  QPushButton *pb7 = new QPushButton( "Import", this );

  connect( pb2, SIGNAL( clicked() ), this, SLOT( newLineDef() ) );
  connect( pb3, SIGNAL( clicked() ), this, SLOT( editLineDef() ) );
  // connect( pb4, SIGNAL( clicked() ), this, SLOT( editMaterials() ) );
  connect( pb5, SIGNAL( clicked() ), this, SLOT( changeCurLine() ) );
  connect( pb6, SIGNAL( clicked() ), this, SLOT( exportFile() ) );
  connect( pb7, SIGNAL( clicked() ), this, SLOT( importFile() ) );

  connect( mListView, SIGNAL( selectionChanged() ), this, SLOT( makeDefaultLineDef() ) );
  QVBoxLayout *vbl = new QVBoxLayout( this );
  vbl->addWidget( mListView );

  QHBoxLayout *hbl = new QHBoxLayout( vbl );
  hbl->addWidget( pb2 );
  hbl->addWidget( pb3 );
  // hbl->addWidget( pb4 );
  hbl->addWidget( pb5 );
  hbl->addWidget( pb6 );
  hbl->addWidget( pb7 );

  connect( &linedata,
           SIGNAL( datasetChanged( ) ),
           this,
           SLOT( loadList( ) ) );

  loadList();

}

void LineSelectorDlg::loadList( ) {
  QListViewItem *item;

  mListView->clear();

  LineDefSet &ldfset = mLineData.getLDFSet();
  for ( int i=0; i < (int)ldfset.size(); i++ ) {
    char buf[20];
    LineDef &ldf = mLineData.getLDFSet()[i];
    std::string lmat       = GLIMSGlobals::mGlbMDF[ldf.mLMat].id;
    std::string rmat       = GLIMSGlobals::mGlbMDF[ldf.mRMat].id;
    std::string color      = GLIMSGlobals::AVAILCOLORSTR[ldf.mColor];
    std::string glactype   = GLIMSGlobals::mGlbGlacTypes[ldf.mGlacType].id;
    std::string segmentlbl = GLIMSGlobals::mGlbLbls[ldf.mLabel].id;
    std::string style    = "";
    std::string type     = GLIMSGlobals::mGlbTypes[ ldf.mType ].id;

    if ( ldf.mStyle == 1 )
      style = "Solid Line";
    else if ( ldf.mStyle == 2 )
      style = "Dash Line";
    else
      style = "Dot Line";

    sprintf( buf, "%d", ldf.mWidth );
    string width = buf;

    item = new QListViewItem( mListView,
                              color.c_str(),
                              glactype.c_str(),
                              segmentlbl.c_str(),
                              lmat.c_str(),
                              rmat.c_str(),
                              style.c_str(),
                              width.c_str(),
                              type.c_str() );
    mListView->insertItem( item );
  }
}

void LineSelectorDlg::makeDefaultLineDef( ) {
  QListViewItem *selectedItem = mListView->selectedItem();
  QListViewItem *item;

  int i=0;

  item = mListView->firstChild();
  while ( selectedItem != item && i < mListView->childCount() ) {
    i++;
    item = item->nextSibling();
  }

  mSelLDF = mListView->childCount() - i -1;
  mLineData.setDefLDF( mSelLDF );
  // emit selLdfChanged( mSelLDF );
}

void LineSelectorDlg::newLineDef( ) {
  LineDefDlg *ald = new LineDefDlg( mLineData );
  ald->exec();
  loadList();
}

void LineSelectorDlg::editLineDef( ) {
  if ( mSelLDF == -1 )
    return;

  LineDefDlg *ald = new LineDefDlg( mLineData, mSelLDF );
  ald->exec();

  loadList();
  // HMMMM
  // mLineTool->emit repaintAll( 0,0,0,0 );
}

void LineSelectorDlg::changeCurLine( ) {
  SelectionSet &selset = mLineData.getSelSet();
  for ( int isel=0; isel < selset.size(); isel++ ) {
    int iline = selset[isel].obj;
    mLineData.setLDF( iline, mSelLDF );
  }
  emit repaintViews();
}

void LineSelectorDlg::importFile( ) {
  QString qfname = QFileDialog::getOpenFileName();
  if ( qfname.isNull() )
    return;

  mLineData.getLDFSet().fromXMLFile( (const char*)qfname,
                                     "LDF_Export_File" );

  loadList();
}

void LineSelectorDlg::exportFile( ) {
  QString qfname = QFileDialog::getSaveFileName();
  if ( qfname.isNull() )
    return;

  mLineData.getLDFSet().toXMLFile( (const char*)qfname,
                                   "LDF_Export_File" );
}



