#ifndef __MAIN_H
#define __MAIN_H

#include <qapplication.h>
#include <qmainwindow.h>
#include <qworkspace.h>
#include <qmenubar.h>
#include <qaction.h>
#include <qtoolbar.h>
#include <qevent.h>
#include <qstatusbar.h>
#include <qfiledialog.h>
#include <qmessagebox.h>
#include <qlabel.h>
#include <qtooltip.h>
#include <qtextbrowser.h>
#include <qmime.h>

#include <vector>
#include <fstream>
#include <cstdlib>

// #include "view.h"
#include "glimsproject.h"
#include "bandchooserdlg.h"
#include "vectorlayerset.h"
#include "viewset.h"
#include "histogramdlg.h"
#include "xmlserializeable.h"
#include "pluginset.h"
#include "aboutdlg.h"
#include "extensions/SpreadsheetDialog.h"
#include "extensions/ImageCalculatorDialog.h"

/**
 * The MainToolbar is really the applcation level of the
 * program where the program starts and ends.  This contains
 * the menu system and the toolbar that allows creation of a
 * project and forwards toolbar and menu system events to it.
 * Upon creation this attempts to fromXML() the file gvstate.xml
 * which holds position of views and image information.  This
 * allows the program to return to the state it was in when last
 * exited.  Upon destruction or exit, this state is saved to the
 * same file.
 *
 * @history 2005-08-04 deborah lee soltesz
 *          Tweaked close event to accept a don't close (false)
 *          from GLIMSProject::close()
 *
 * @history 2006-01-21 deborah lee soltesz
 *          Added project filename readout to status bar
 *
 * @whatsnew 2006-01-21
 *       Project filename is now displayed in the status bar
 *
 * @todo 2005-08-05 deborah lee soltesz
 *       Set window title to project filename. May involve
 *       making the mainWidget something other than a QToolBar
 */
class MainToolbar : public QMainWindow, XMLSerializeable {
  Q_OBJECT
  protected:
  static const char*     VIEW_FILE;

  GLIMSProject             *mProject;

  bool                     mInit;

  QWorkspace*              mWorkspace;
  QToolBar*                mFileToolbar;
  QToolBar*                mNavToolbar;
  QToolBar*                mEditToolbar;
  QToolBar*                mToolToolbar;
  QActionGroup*            mActGrp;
  QActionGroup*            mViewActGroup;

  QAction*                 mBreakAction;
  QAction*                 mJoinAction;
  QAction*                 mOpenConfigAction;
  QAction*                 mMemBufAction;

  QStatusBar*              mStatBar;

  ImageCalculatorDialog*   imgCalcDlg ; // image polygon calculator tool dialog

  BandChooserDlg*          mBandChooser;
  HistogramDlg*            mHistDlg;
  ViewSet                  mViewSet;

  QLabel*                  mLblLL;
  QLabel*                  mLblXY;
  QLabel*                  mLblProjFile;

  PluginSet*               mPlugSet;

  void buildMenu( );
  void buildToolbar( );
  void buildViews( );
  void buildDefaultViews( );
  void loadProject( GLIMSProject * );
  void setupProj( GLIMSProject * );

  bool fromXML( QDomElement &elem );
  bool toXML( QDomDocument &doc,
              QDomElement &elem,
              std::string id = "" );

  void closeEvent( QCloseEvent * ) {
    if ( mProject ) {
      if (mProject->close()) {
        toXMLFile( "gvstate.xml", "GVState" );
        exit( 0 );
      }
    }
    else exit( 0 );
  }

public:
  MainToolbar( );
  virtual ~MainToolbar( );

public slots:
  void newProject( );
  void openProject( );
  void saveProject( );
  void saveProjectAs( );
  void importGLIMSIngest( );
  void exportGLIMSIngest( );
  void importFromShapefile( );
  void openConfigDlg( );
  void openGtoSDlg();  //added by EL
  void openImgCalcDlg(); //added by EL
  void closeApp( );
  void closeProject( );
  void print( );
  void openBandSel( );
  void openHistDlg( );
  void repaintViews( bool );
  void setMemBuf( );
  void setNN( );
  void setBI( );
  void setBC( );
  void openAbout( );
  void openManual( );

  void setFocusWidget( QWidget * );
  void setPanHand( );
  void setZoomInInc( );
  void setZoomOutInc( );
  void setLineTool( );
  void setSelTool( );
  void setGIDTool( );

  void setCoordLbls( View::MousePos ll, View::MousePos xy );
  void setProjectFileName (QString fn) ;

  void imgCalcEnd() ;

};

#endif




