#include "nlapsfrmt.h"
#include <proj.h>
#include <fstream>
#include <iostream>

const std::string NlapsFrmt::GCTPProjMap[] = {
"latlong",
"utm",
"UNKNOWN STATE PLANE",
"aea",
"lcc",
"merc",
"ups",
"poly",
"eqdc",
"tmerc",
"stere",
"laea",
"aeqd",
"gnom",
"ortho",
"nsper",
"sinu",
"UNKNOWN EQUIRECTANGULAR",
"mill",
"vandg",
"omerc",
"robin",
"omerc",
"robin",
"lsat",
"alsk",
"goode",
"hammer",
"wag4",
"wag7",
"oea",
"UNKNOWN INTEGERIZED SINUSOIDAL GRID"
};

const char* NlapsFrmt::GCTPParmMap[][15] = {
{ "",    "",   "",       "",       "",       "",       "",     "",     "", "", "", "", "", "", ""},
{ "a=",  "b=", "",       "",       "",       "",       "",     "",     "", "", "", "", "", "", ""},
{ "",    "",   "",       "",       "",       "",       "",     "",     "", "", "", "", "", "", ""},
{ "a=",  "b=", "lat_1=", "lat_2=", "lon_0=", "lat_0=", "x_0=", "y_0=", "", "", "", "", "", "", ""},
{ "a=",  "b=", "lat_1=", "lat_2=", "lon_0=", "lat_0=", "x_0=", "y_0=", "", "", "", "", "", "", ""},
{ "a=",  "b=", "",       "",       "lon_0=", "k_0=",   "x_0=", "y_0=", "", "", "", "", "", "", ""},
{ "a=",  "b=", "",       "",       "lon_0=", "k_0=",   "x_0=", "y_0=", "", "", "", "", "", "", ""},
{ "a=",  "b=", "",       "",       "lon_0=", "lat_0=", "x_0=", "y_0=", "", "", "", "", "", "", ""},
{ "a=",  "b=", "lat_1=", "lat_2=", "lon_0=", "lat_0=", "x_0=", "y_0=", "", "", "", "", "", "", ""},
{ "a=",  "",   "k=",     "",       "lon_0=", "lat_0=", "x_0=", "y_0=", "", "", "", "", "", "", ""},
{ "a=1", "b=", "",       "",       "lon_0=", "lat_0=", "x_0=", "y_0=", "", "", "", "", "", "", ""},
{ "a=1", "",   "",       "",       "lon_0=", "lat_0=", "x_0=", "y_0=", "", "", "", "", "", "", ""},
{ "a=1", "",   "",       "",       "lon_0=", "lat_0=", "x_0=", "y_0=", "", "", "", "", "", "", ""},
{ "a=1", "",   "",       "",       "lon_0=", "lat_0=", "x_0=", "y_0=", "", "", "", "", "", "", ""},
{ "a=1", "",   "",       "",       "lon_0=", "lat_0=", "x_0=", "y_0=", "", "", "", "", "", "", ""},
{ "a=1", "",   "",       "",       "lon_0=", "lat_0=", "x_0=", "y_0=", "", "", "", "", "", "", ""},
{ "a=1", "",   "",       "",       "lon_0=", "",       "x_0=", "y_0=", "", "", "", "", "", "", ""},
{ "a=1", "",   "",       "",       "lon_0=", "lat_ts=","x_0=", "y_0=", "", "", "", "", "", "", ""},
{ "a=1", "",   "",       "",       "lon_0=", "lat_0=", "x_0=", "y_0=", "", "", "", "", "", "", ""},
{ "a=1", "",   "",       "",       "lon_0=", "",       "x_0=", "y_0=", "", "", "", "", "", "", ""},
{ "a=",  "b=", "k_0=",   "",       "",       "lat_0=", "x_0=", "y_0=", "", "", "", "", "", "", ""},
{ "a=1", "",   "",       "",       "lon_0=", "",       "x_0=", "y_0=", "", "", "", "", "", "", ""},
{ "a=",  "b=", "",       "",       "",       "",       "",     "",     "", "", "", "", "", "", ""},
{ "a=",  "b=", "",       "",       "",       "",       "x_0=", "y_0=", "", "", "", "", "", "", ""},
{ "a=1", "",   "",       "",       "", "",             "",     "",     "", "", "", "", "", "", ""},
{ "a=1", "",   "",       "",       "lon_0=", "",       "x_0=", "y_0=", "", "", "", "", "", "", ""},
{ "a=1", "",   "",       "",       "",       "",       "",     "",     "", "", "", "", "", "", ""},
{ "a=1", "",   "",       "",       "lon_0=", "",       "x_0=", "y_0=", "", "", "", "", "", "", ""},
{ "a=1", "",   "",       "",       "lon_0=", "",       "x_0=", "y_0=", "", "", "", "", "", "", ""},
{ "a=1", "",   "",       "",       "lon_0=", "",       "x_0=", "y_0=", "", "", "", "", "", "", ""},
{ "a=1", "",   "",       "",       "",       "",       "",     "",     "", "", "", "", "", "", ""}
};

bool NlapsFrmt::open( std::string fname ) {
  mFName = fname;
  if ( !readFile() )
    return false;

  fillHeader();
  fillBandInf();
  fillImageInf();
  return true;
}

void* NlapsFrmt::getBand( int iband,
                          int x1,
                          int y1,
                          int w1,
                          int h1,
                          int w2,
                          int h2,
                          ScaleType st ) const {
  std::string bfname;
  int ifs = -1;
  int pl;
  int scale;

  int w = mBandSet[iband].width;
  // int h = mBandSet[iband].height;
  int dsz = dtypeSize( mBandSet[iband].type );

  unsigned char *data;
  unsigned char *line;

  if ( w2 <= 0 || h2 <= 0 ) return NULL;

  scale = (w1 / w2) + 1;

  ifs = mFName.find_last_of( "\\" );
  if ( ifs == -1 )
    ifs = mFName.find_last_of( "/" );
  if ( ifs == -1 ) return NULL;

  bfname = mFName.substr( 0, ifs + 1 );

  std::ifstream in( ( bfname + mBFNameSet[iband] ).c_str(),
                    std::istream::binary |
                    std::istream::in );
  if ( !in )
    return NULL;


  data = new unsigned char[ ((w1 * h1) / scale) * dsz ];

  pl = ( y1 * w * dsz ) + ( x1 * dsz );
  int sw = w1 / scale;
  int sh = h1 / scale;
  // int off = ( w * dsz ) * scale;
  line = new unsigned char[ w1 * dsz ];

  for ( int i=0; i < sh; i++ ) {

    in.seekg( pl, std::istream::beg );
    in.read( (char*)line, w1 * dsz );

    for ( int x=0; x < sw; x++ )
      data[ (i * sw) + x ] = line[ x * scale ];

    pl += ( w * dsz ) * scale;
  }
  in.close();

  scaleData( (char**)&data, sw, sh, w2, h2, mBandSet[iband].type, st );
  return(void*)data;

}

void NlapsFrmt::getLL( int iband,
                       double &i,
                       double &j ) {
  xyToLL( iband, i, j );
}

void NlapsFrmt::getXY( int iband,
                       double &i,
                       double &j ) {
  llToXY( iband, i, j );
}

bool NlapsFrmt::readFile( ) {
  // int fsz = 0;
  mText = "";
  char ch;
  std::string fl;

  std::ifstream in( mFName.c_str() );
  if ( !in ) return false;

  for ( int i=0; i < 20 && !in.eof(); i++ ) {
    in.get( ch );
    fl += ch;
  }

  if ( (int)fl.find( "NDF_REVISION" ) == -1 )
    return false;

  while ( !in.get( ch ).eof() )
    mText += ch;

  return true;
}

std::vector<std::string>
NlapsFrmt::getValue( std::string id ) {
  std::vector<std::string> ret;
  std::string field, tid, tval;
  int line = 0;
  bool foundOne = false;

  do {
    if ( !getToken( mText, "\n", line++, field ) )
      break;

    field = trimStr( field );
    getToken( field, "=", 0, tid );
    tid = trimStr( tid );

    if ( tid == id ) {
      getToken( field, "=", 1, tval );
      tval = trimStr( tval );
      foundOne = true;
    }
  } while ( !foundOne );

  if ( foundOne ) {
    int i=0;
    std::string val;
    while ( getToken( tval, ",", i++, val ) )
      ret.push_back( val );
  }

  return ret;
}

void NlapsFrmt::fillHeader( ) {
  std::vector<std::string> valset;

  // DATA_FILE_INTERLEAVE
  valset = getValue( "DATA_FILE_INTERLEAVING" );
  if ( valset[0] == "BSQ" )
    mHead.bsq = true;

  valset = getValue( "USGS_PROJECTION_NUMBER" );
  mHead.gctpID = atoi( valset[0].c_str() );

  valset = getValue( "USGS_MAP_ZONE" );
  if ( valset.size() > 0 )
    mHead.utmz = atoi( valset[0].c_str() );

  valset = getValue( "HORIZONTAL_DATUM" );
  mHead.datum = valset[0];

  valset = getValue( "UPPER_LEFT_CORNER" );
  mHead.ul_lon = parseDeg( valset[0] );
  mHead.ul_lat = parseDeg( valset[1] );
}

void NlapsFrmt::fillBandInf( ) {
  BandInf binf;
  std::vector<std::string> val;
  char id[255];

  binf.proj = GCTPProjMap[mHead.gctpID];
  binf.UTMZ = mHead.utmz;
  binf.rot = 0.0;
  binf.ulx = mHead.ul_lon;
  binf.uly = mHead.ul_lat;

  val = getValue( "PIXEL_FORMAT" );
  if ( val[0] == "BYTE" )
    binf.type = UINT8;
  else if ( val[0] == "2BYTEINT" )
    binf.type = INT16;
  else if ( val[0] == "4BYTEINT" )
    binf.type = INT32;
  else if ( val[0] == "REAL" )
    binf.type = INT32;
  else if ( val[0] == "DOUBLE" )
    binf.type = FLOAT32;
  else
    binf.type = NOT_DEFINED;

  val = getValue( "USGS_PROJECTION_PARAMETERS" );
  binf.parms = getProjParms( val, mHead.gctpID );


  val = getValue( "PIXEL_SPACING" );
  binf.res = atoi( val[0].c_str() );

  val = getValue( "LINES_PER_DATA_FILE" );
  binf.height = atoi( val[0].c_str() );

  val = getValue( "PIXELS_PER_LINE" );
  binf.width = atoi( val[0].c_str() );

  binf.ndims = 2;

  int cnt = 1;
  while ( true ) {
    BandInf cbinf = binf;
    sprintf( id, "BAND%d_NAME", cnt );
    val = getValue( id );
    if ( val.size() < 1 ) break; // THIS SHOULD STOP THE LOOP
    cbinf.name = val[0];

    sprintf( id, "BAND%d_FILENAME", cnt );
    val = getValue( id );
    mBFNameSet.push_back( val[0] );

    mBandSet.push_back( cbinf );

    cnt++;
  }

  getULCorners();
}

void NlapsFrmt::fillImageInf( ) {
  double x, y;
  std::vector<std::string> val;

  x = mBandSet[0].width / 2;
  y = mBandSet[0].height / 2;
  xyToLL( 0, x, y );

  mImageInf.mCenterLat = y;
  mImageInf.mCenterLon = x;

  val = getValue( "MAP_PROJECTION_NAME" );
  if ( (int)val.size() > 0 )
    mImageInf.mProjection = val[0];

  val = getValue( "ACQUISITION_DATE/TIME" );
  if ( (int)val.size() > 0 && mImageInf.mAquiDateTime.size() == 0)
    mImageInf.mAquiDateTime = formatDateTime( val[0] );
}

std::string NlapsFrmt::formatDateTime( std::string str ) {
  // 1999-11-04T17:49:07Z

  std::string ret;

  ret = str.substr( 0, 4 ) +
        str.substr( 5, 2 ) +
        str.substr( 8, 2 ) + " " +
        str.substr( 11, 8 ) + "-0000";

  return ret;
}

double NlapsFrmt::parseDeg( std::string sdeg ) {
  double  deg = 0.0;
  double  min = 0.0;
  double  sec = 0.0;
  char    dir;
  double  val;

  deg = atoi( sdeg.substr( 0, 3 ).c_str() );
  min = atoi( sdeg.substr( 3, 2 ).c_str() );
  sec = atof( sdeg.substr( 5, 7 ).c_str() );
  dir = sdeg[12];

  val = deg + (min/60.0) + (sec/60.0/60.0);
  if ( dir == 'S' || dir == 'W' )
    val = 0 - val;

  return val;
}

bool NlapsFrmt::getToken( std::string str,
                          const std::string delim,
                          int n,
                          std::string &tok ) {
  int l_ind=0, r_ind=0;
  int count=-1;
  while ( ( r_ind = str.find(delim, l_ind) ) != -1 && ++count < n )
    l_ind = r_ind + 1;

  if ( count == n || (n == count+1 && r_ind == -1) ) {
    if ( r_ind == -1 )
      r_ind = str.length();
    tok = str.substr(l_ind, r_ind - l_ind);
    if ( tok == "" )
      return false;
    return true;
  }

  tok = "";
  return false;
}

std::string NlapsFrmt::trimStr( std::string str ) {
  int a=0;
  int strsz = str.length();
  while ( a < strsz ) {
    char ch = str[a];
    if ( ch != '\r' && ch != '\n' && ch != ' ' && ch != '\t' && ch != ';' )
      break;
    a++;
  }

  if ( a >= strsz )
    return "";

  int b = strsz;
  while ( b > a ) {
    char ch = str[b-1];
    if ( ch != '\r' && ch != '\n' && ch != ' ' && ch != '\t' && ch != ';' )
      break;
    b--;
  }

  if ( b <= a )
    return "";

  return str.substr( a, b-a );
}

std::vector<std::string>
NlapsFrmt::getProjParms( std::vector<std::string> &raw, int gctpid ) {
  std::vector<std::string> ret;
  char buf[100];

  if ( (int)raw.size() != 15 )
    return ret;

  for ( int i=0; i < 15; i++ ) {
    std::string pjparm = GCTPParmMap[gctpid][i];
    if ( pjparm != "" ) {
      if ( pjparm[pjparm.length()-1] == '=' ) {
        double val = atof( raw[i].c_str() );
        if ( (int)pjparm.find( "lat" ) != -1 || (int)pjparm.find( "lon" ) != -1 ) {
          int    deg =  (int)(val / 1000000);
          double min = (val - (deg*1000000)) / 1000;
          double sec =  val - (deg*1000000) - (min*1000);

          min /= 60;
          sec /= 360.0;

          val = deg + min + sec;
        }

        sprintf( buf, "%s%.5f", pjparm.c_str(), val );
        ret.push_back( std::string( buf ) );
      } else {
        ret.push_back( pjparm );
      }
    }
  }

  return ret;
}






