#ifndef __NLAPSFRMT_H
#define __NLAPSFRMT_H

#include <string>
#include <vector>
#include <fstream>
#include <proj_api.h>

#include "imageformat.h"

/**
 * NLAPS image format.
 */
class NlapsFrmt : public ImageFormat {
public:
  /**
   * Map of projection information
   */
  static const std::string GCTPProjMap[];
  /**
   * Map of image parameter information
   */
  static const char* GCTPParmMap[][15];

/**
 * Structure to hold header information from the image
 */
  typedef struct {
    bool            bsq;
    int             gctpID;
    int             utmz;
    std::string     datum;
    double          ul_lon;
    double          ul_lat;
  } Header;

protected:
  std::string     mText;
  Header          mHead;
  std::vector<std::string> mBFNameSet;

  std::vector<std::string> getValue( std::string id );
  bool readFile( );
  bool getToken( std::string str,
                 const std::string delim,
                 int n,
                 std::string &tok );

  std::string trimStr( std::string str );
  void fillHeader( );
  void fillBandInf( );

  /**
   * Pull out associated image metadata of an image
   * and store in ImageInf.
   *
   * @history 2005-08-20 dls
   *          modified to not overwrite image acquisition date
   *          read in from project file
   */
  void fillImageInf( );
  std::string formatDateTime( std::string );
  double parseDeg( std::string deg );
  std::vector<std::string>
  getProjParms( std::vector<std::string> &, int );
  //projPJ createProj( int iband );

public:

  virtual bool open( std::string fname );

  virtual void* getBand( int iband,
                         int x1,
                         int y1,
                         int w1,
                         int h1,
                         int w2,
                         int h2,
                         ScaleType st = SCALE_BI ) const;

  virtual void getLL( int iband,
                      double &i,
                      double &j );

  virtual void getXY( int iband,
                      double &i,
                      double &j );
};

#endif


