#ifndef ODLTREE_H
#define ODLTREE_H

#include <vector>
#include "odldefs.h"
#include "odlelement.h"

class ODLTree {
protected:
  std::string             mName;
  ODLTYPE                 mType;
  std::vector<ODLTree>    mChld;
  std::vector<ODLElement> mElem;
  bool                    mValid;

public:
  ODLTree( ) :
  mName( "" ),
  mType( ODLGROUP ) {
    mValid = false;
  }

  ODLTree( std::string name, ODLTYPE type ) :
  mName( name ),
  mType( type ) {
    mValid = false;
  }

  std::string getName( ) const {
    return mName;
  }
  void setName( const std::string &name ) {
    mName = name;
  }

  ODLTYPE getType( ) const {
    return mType;
  }
  void setType( const ODLTYPE &type ) {
    mType = type;
  }

  // TREE/CHILDREN ROUTINES
  std::vector<ODLTree> getChildSet( ) const {
    return mChld;
  }
  void addChild( const ODLTree &chld ) {
    mChld.push_back( chld );
  }
  ODLTree * getChild( int iChld ) const {
    if ( iChld > (int)mChld.size() || iChld < 0 )
      return NULL;

    return(ODLTree*)(&mChld[iChld]);
  }

  ODLTree * getLastChild( ) {
    if ( mChld.size() == 0 )
      return NULL;
    return &mChld[mChld.size()-1];
  }

  int getNumChild( ) const {
    return mChld.size();
  }


  // ELEMENT ROUTINES
  void addElem( const ODLElement &elm ) {
    mElem.push_back( elm );
  }
  ODLElement * getElem( int iElem ) const {
    if ( iElem > (int)mElem.size() || iElem < 0 )
      return NULL;

    return(ODLElement*)(&mElem[iElem]);
  }

  int getNumElem( ) const {
    return mElem.size();
  }
};

#endif


