#include "plugin.h"
#include <qmessagebox.h>

bool Plugin::fromXML( QDomElement &elem ) {
  if ( elem.isNull() )
    return false;

  if ( std::string( "Plugin" ).compare( (const char*)elem.tagName() ) )
    return false;

  mFName    = readTextTag( elem, "FName" );
  mPlugName = readTextTag( elem, "PlugName" );
  mDesc     = readTextTag(  elem, "Desc" );

  return true;
}

bool Plugin::toXML( QDomDocument &doc,
                    QDomElement &elem,
                    std::string id ) {
  QDomElement base = doc.createElement( "Plugin" );
  if ( id != "" )
    base.setAttribute( "id", (char*)id.c_str() );
  elem.appendChild( base );

  writeTextTag( doc, base, "FName",    mFName.c_str() );
  writeTextTag( doc, base, "PlugName", mPlugName.c_str() );
  writeTextTag( doc, base, "Desc",     mDesc );

  return true;
}

void Plugin::execPlugin( GLIMSProject *proj ) {
  LIBHANDLE hPlug;
  EXECFUNC  pFunc;

  hPlug = loadPlugin();

  if ( !hPlug )
    return;

  // GET NAME AND CHECK FOR REQUIRED SYMBOLS

#ifdef WIN32
  pFunc = (EXECFUNC) GetProcAddress( (HINSTANCE)hPlug,
                                     "activate" );
#else
  pFunc = (EXECFUNC)dlsym( hPlug, "activate" );

#endif

  if ( pFunc == NULL ) {
    QMessageBox::information( NULL, "Plugin Error", "Failed to execute plugin" );
    return;
  }

  (pFunc)( proj );

  unloadPlugin( hPlug );
}

bool Plugin::loadFromFile( std::string fname ) {
  LIBHANDLE       hPlug;         // PLUGIN HANDLE
  EXECFUNC        pExec;
  GETNAMEFUNC     pGetName;
  GETDESCFUNC     pGetDesc;
  char            sep = QDir::separator();
  char            fnamebuf[256];

  if ( fname == "" )
    return false;

  mFName = fname;
  hPlug = loadPlugin();
  if ( !hPlug )
    return false;

  // GET NAME AND CHECK FOR REQUIRED SYMBOLS

#ifdef WIN32
  pGetName = (GETNAMEFUNC)GetProcAddress( hPlug, "getName" );
  pGetDesc = (GETDESCFUNC)GetProcAddress( hPlug, "getDesc" );
  pExec    = (EXECFUNC)GetProcAddress( hPlug, "activate" );

#else
  pGetName = (GETNAMEFUNC)dlsym( hPlug, "getName" );
  pGetDesc = (GETDESCFUNC)dlsym( hPlug, "getDesc" );
  pExec    = (EXECFUNC)dlsym( hPlug, "activate" );

#endif


  if ( pGetName == NULL )
    return false;
  mPlugName = (pGetName)();

  if ( pGetDesc == NULL )
    return false;
  mDesc = (pGetDesc)();

  if ( pExec == NULL )
    return false;

  unloadPlugin( hPlug );

  // COPY THE FILE
  int pos = mFName.find_last_of( "\\" );
  if ( pos == -1 )
    pos = mFName.find_last_of( "/" );
  pos++;

  mFName = mFName.substr( pos );
  sprintf( fnamebuf, ".%cplugins%c%s", sep, sep, mFName.c_str() );
  mFName = fnamebuf;

  std::ifstream infile( fname.c_str(), std::ios::binary );
  if ( !infile )
    return false;

  std::ofstream outfile( mFName.c_str(), std::ios::binary );
  if ( !outfile ) {
    infile.close();
    return false;
  }

  outfile << infile.rdbuf();
  outfile.close();
  infile.close();

  return true;
}

LIBHANDLE Plugin::loadPlugin( ) {
  LIBHANDLE hPlug;

#ifdef WIN32
  hPlug = LoadLibraryA( mFName.c_str() );
  if ( !hPlug )
    return NULL;

#else
  hPlug = dlopen( mFName.c_str(), RTLD_NOW );
  if ( !hPlug ) {
    std::cout << dlerror() << std::endl;
    return NULL;
  }

#endif

  return hPlug;
}

void Plugin::unloadPlugin( LIBHANDLE hPlug ) {
#ifdef WIN32
  FreeLibrary( hPlug );
#else
  if ( hPlug ) dlclose( hPlug );
#endif
}


