#ifndef __PLUGIN_H
#define __PLUGIN_H

#include "xmlserializeable.h"
#include "glimsproject.h"

#ifdef WIN32
    #include <windows.h>
    #define LIBHANDLE HINSTANCE
#else
  #include <dlfcn.h>
    #define LIBHANDLE void*
#endif

typedef void  (*EXECFUNC)( GLIMSProject * );
typedef const char* (*GETNAMEFUNC)( );
typedef const char* (*GETDESCFUNC)( );

/**
 * An implementation of a plugin interface allowing application
 * extensibility.  This is designed to let a programer develop
 * a C++ plugin that extends this class and overrides its members
 * to perform actions on the currently loaded project.  A project
 * is passed to the plugin through its constructor.  The plugin
 * should define the name of itself which will be available in the
 * menu system under Tools->Plugins for execution.  The plugin should
 * also define its description which is available in the plugin
 * configurator.  A description of how to implement a plugin is
 * available within the source tree of this distribution.
 *
 * NOTE:  This is the only code within glimsviewer other than windows_defs.h
 * that contains #defines that alter compiler output.  This is implemented
 * for Unix variants that provide the dl dynamic module loading
 * functions and Windows.  This does NOT include Mac OSX as they have
 * not conformed to Unix in this respect.  For now, this code will fail
 * to compile on any compiler unable to handle it.
 */
class Plugin : public XMLSerializeable {
protected:
  std::string     mFName;
  std::string     mPlugName;
  std::string     mDesc;

  LIBHANDLE loadPlugin( );
  void unloadPlugin( LIBHANDLE hPlug );

public:
  Plugin( ) :
  mFName( "" ),
  mPlugName( "" ),
  mDesc( "" ) {
  }

  virtual bool fromXML( QDomElement &elem );
  virtual bool toXML( QDomDocument &doc,
                      QDomElement &elem,
                      std::string id="" );

  void execPlugin( GLIMSProject *proj );

  bool loadFromFile( std::string fname );

  std::string getName( ) {
    return mPlugName;
  }
  std::string getFile( ) {
    return mFName;
  }
  std::string getDesc( ) {
    return mDesc;
  }
};

#endif


