#ifndef __PLUGINSET_H
#define __PLUGINSET_H

#include <qwidget.h>
#include <qaction.h>
#include <qlistview.h>
#include <qlayout.h>
#include <qhbox.h>
#include <qgroupbox.h>
#include <qpushbutton.h>
#include <qmultilineedit.h>
#include <qpopupmenu.h>


#include <vector>
#include <string>

#include "plugin.h"
#include "xmlserializeable.h"
#include "viewpopupaction.h"
#include "glimsproject.h"

/**
 * This holds all plugins available to the application.  These
 * include all plugins that are defined in the ./plugins/plugcfg.xml
 * file.  If the plugin is not loadable, it will fail to open.  This
 * is also the user interface dialog that allows the user to import
 * and view the descriptions of plugins.
 */
class PluginSet :
public QWidget,
public XMLSerializeable {
  Q_OBJECT
  protected:
  const std::string       PLUGCFG_FNAME;
  std::vector<Plugin>     mPlugSet;
  QActionGroup            *mMenuGroup;
  QListView               *mPlugList;
  QMultiLineEdit          *mDescBox;
  QPopupMenu              *mMenuParent;
  GLIMSProject            *mProj;

  void buildWidget( );
  void loadMenu( );
  void loadList( );

public:
  PluginSet( QWidget *parent, QPopupMenu *menuparent );

  virtual bool fromXML( QDomElement &elem );
  virtual bool toXML( QDomDocument &doc,
                      QDomElement &elem,
                      std::string id="" );
  void setProj( GLIMSProject *proj ) {
    mProj = proj;
  }

public slots:
  void importPlugin( );
  void rescanDir( );
  void showPluginCfg( );
  void pluginChosen( ViewPopupAction * );
  void selectedPluginChanged( QListViewItem * );
};

#endif




