#include "projectdlg.h"
#include "image.h"

ProjectDlg::ProjectDlg( std::string &name,
                        std::string &dir,
                        std::string &img,
                        int &type ) :
QDialog( NULL, NULL, true ),
mName( name ),
mDir( dir ),
mImg( img ),
mType( type ) {
  setCaption( "Create GLIMS Project" );

  for ( int imffn=0; imffn < Image::NUM_MFF; imffn++ )
    mTypeSet.push_back( Image::MFF_NAMES[imffn] );

  buildDialog();
}


void ProjectDlg::buildDialog( ) {
  QHBox   *hbox = NULL;
  QLabel  *lbl = NULL;
  QGrid   *grid = NULL;
  QPushButton *but = NULL;

  QVBoxLayout *vbox = new QVBoxLayout( this );
  vbox->setAutoAdd( true );
  vbox->setMargin( 10 );

  grid = new QGrid( 2, this );
  grid->setSpacing( 7 );
  grid->setMargin( 5 );

  lbl = new QLabel( "Name: ", grid );
  mTxtName = new QLineEdit( grid );

  // grid = new QGrid( 2, this );
  new QLabel( "Save Directory: ", grid );
  hbox = new QHBox( grid );
  hbox->setSpacing( 5 );
  mTxtDir = new QLineEdit( hbox );
  mTxtDir->setMinimumWidth( 225 );
  but = new QPushButton( "Browse", hbox );
  connect( but, SIGNAL( clicked() ), this, SLOT( setDir() ) );

  // grid = new QGrid( 2, this );
  new QLabel( "Image: ", grid );
  hbox = new QHBox( grid );
  hbox->setSpacing( 5 );
  mTxtImg = new QLineEdit( hbox );
  mTxtImg->setMinimumWidth( 225 );
  but = new QPushButton( "Browse", hbox );
  connect( but, SIGNAL( clicked() ), this, SLOT( setImg() ) );

  mChkMulti = new QCheckBox( "Multi File", grid );
  connect( mChkMulti, SIGNAL( toggled( bool ) ), this, SLOT( chkToggle( bool ) ) );
  mCmbType = new QComboBox( grid );
  mCmbType->setEnabled( false );
  mCmbType->setMaximumWidth( 200 );
  loadTypes();

  // grid = new QGrid( 2, this );
  hbox = new QHBox( this );
  hbox->setMargin( 5 );
  but = new QPushButton( "Create", hbox );
  but->setMaximumWidth( 85 );
  connect( but, SIGNAL( clicked() ), this, SLOT( create() ) );
  but = new QPushButton( "Cancel", hbox );
  but->setMaximumWidth( 85 );
  connect( but, SIGNAL( clicked() ), this, SLOT( reject() ) );
}

void ProjectDlg::setDir( ) {
  QString dir = QFileDialog::getExistingDirectory( QString::null, this );
  if ( dir.isNull() )
    return;
  mTxtDir->setText( (const char*)dir );
}

void ProjectDlg::setImg( ) {
  QString img = QFileDialog::getOpenFileName( QString::null, QString::null, this );
  if ( img.isNull() )
    return;
  mTxtImg->setText( (const char*)img );
}

void ProjectDlg::create( ) {
  mName = (const char*)mTxtName->text();
  mDir = (const char*)mTxtDir->text();
  mImg = (const char*)mTxtImg->text();
  if ( mChkMulti->isChecked() )
    mType = mCmbType->currentItem();
  else
    mType = -1;

  accept();
}

void ProjectDlg::chkToggle( bool toggle ) {
  mCmbType->setEnabled( toggle );
}

void ProjectDlg::loadTypes( ) {
  for ( int i=0; i < (int)mTypeSet.size(); i++ )
    mCmbType->insertItem( mTypeSet[i].c_str() );
}


