#ifndef __SESSION_H
#define __SESSION_H

#include <string>
#include "shape.h"
#include "xmlserializeable.h"

/**
 * An class representation of the session.shp file as
 * described in the GLIMS specifications.  This is user
 * and dataset information that is session wide.
 *
 */
class Session : public XMLSerializeable {
public:
  int             mRC_ID;            //!< Regional Center ID
  std::string     mDataSrc;          //!< Data Source
  std::string     mProcDesc;         //!< Processing description
  std::string     manlst_surn;       //!< Analyst's surname
  std::string     manlst_givn;       //!< Analyst's given name
  std::string     m3d_desc;          //!< 3D description
  std::string     manaly_time;       //!< Analysis time
  Node            mCenterPoint;      //!< Center point of image


/**
 * Read Session information from the project XML file.
 *
 * @history 2005-08-04 deborah lee soltesz
 *          fixed issue with data not being read from XML
 *          file correctly (several elements read into mDataSrc
 *          instead of the appropriate member vars)
 */
  virtual bool fromXML( QDomElement &elem ) {
    mRC_ID       = readIntTag ( elem, "RC_ID" );
    mDataSrc     = readTextTag( elem, "DataSrc" );
    mProcDesc    = readTextTag( elem, "ProcDesc" );
    manlst_surn  = readTextTag( elem, "anlst_surn" );
    manlst_givn  = readTextTag( elem, "anlst_givn" );
    m3d_desc     = readTextTag( elem, "ThreeD_desc" );
    manaly_time  = readTextTag( elem, "analy_time" );
    mCenterPoint.fromXML( elem );
    return true;
  }

  /**
   * Write Session information to the XML file
   */
  virtual bool toXML( QDomDocument &doc,
                      QDomElement &elem,
                      std::string id = "" ) {
    QDomElement base = createBaseTag( doc, elem, id, "Session" );

    writeIntTag ( doc, base, "RC_ID", mRC_ID );
    writeTextTag( doc, base, "DataSrc", mDataSrc );
    writeTextTag( doc, base, "ProcDesc", mProcDesc );
    writeTextTag( doc, base, "anlst_surn", manlst_surn );
    writeTextTag( doc, base, "anlst_givn", manlst_givn );
    writeTextTag( doc, base, "ThreeD_desc", m3d_desc );
    writeTextTag( doc, base, "analy_time", manaly_time );
    mCenterPoint.toXML( doc, base );
    return true;
  }
};

#endif


