#ifndef SESSIONDLG_H
#define SESSIONDLG_H

#include <qwidget.h>
#include <qlineedit.h>
#include <qmultilineedit.h>
#include <qevent.h>
#include <qvbox.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qgrid.h>

#include <string>
#include <vector>

#include "glimsdataset.h"

using namespace std;

class LineTool;

/**
 * A user interface dialog that allows a user to
 * modify the Session information that is stored
 * by GLIMSDataset.
 *
 * @history 2005-08-04 deborah lee soltesz
 *          added RC ID and Analysis Time to dialog
 *
 * @history 2005-08-20 deborah lee soltesz
 *          Added "ID" to RC ID label, limited RC ID to 3 chars,
 *          and changed description fields to be multi-line,
 *          wrapping, scrolling boxes with plenty of room to
 *          add content
 *
 * @whatsnew 2005-08-20
 *          Session information is correctly saved. Description
 *          fields now multi-line boxes, making it easier to
 *          write long descriptions. Description fields also now
 *          unlimited length (used to be restricted to 200 characters).
 *
 * @history 2005-08-22 deborah lee soltesz
 *          Changed QMultiLineEdit to connect signal
 *          textChanged (w/o parameter) because there
 *          is no param passed along with that signal
 *          (fails in Linux, but not Windows?). Fixed
 *          filling RC ID field in GUI.
 */
class SessionDlg : public QWidget {
  Q_OBJECT
  protected:
  QLineEdit       *mtxtDataSource;
  QLineEdit       *mtxtFName;
  QLineEdit       *mtxtLName;

  //QLineEdit       *mtxtProcDesc;
  //QLineEdit       *mtxt3dDesc;

  QMultiLineEdit  *mtxtProcDesc;
  QMultiLineEdit  *mtxt3dDesc;

  QLineEdit       *mtxtRCID ;
  QLineEdit       *mtxtAnalysisTime ;

  Session         &mSession;

public:
  SessionDlg( Session & );
  ~SessionDlg( );

public slots:
 /**
  * Fill dialog box with info from Session
  *
  * @history 2005-08-22 dls
  *          added filling in RC ID
  * @history 2006-01-21 dls
  *          added filling in analysis time
  */
  void loadData( );

  //! data source changed
  void dschanged( const QString & );
  //! first name changed
  void fnchanged( const QString & );
  //! last name changed
  void lnchanged( const QString & );
  /**
   * Processing description changed
   */
  void pdchanged( const QString & );
  /**
   * 3D description changed
   */
  void d3changed( const QString & );

  /**
   * Processing description changed
   *
   * @author 2005-08-22 Deborah Lee Soltesz -
   * added to handle textChanged() signal, which
   * does not pass a parameter
   *
   */
  void pdchanged( );
  /**
   * 3D description changed
   *
   * @author 2005-08-22 Deborah Lee Soltesz -
   * added to handle textChanged() signal, which
   * does not pass a parameter
   *
   */
  void d3changed( );

/**
 * Change Analysis Time
 *
 * @history 2005-08-04 deborah lee soltesz
 *          added Analysis Time to dialog
 * @todo    2005-08-04 deborah lee soltesz
 *          Analysis Time not quite legal -
 *          needs to be read-only because it's
 *          the timestamp of the last export
 */
  void atchanged( const QString & );

/**
 * Change RC ID
 *
 * @history 2005-08-04 deborah lee soltesz
 *          added RC ID to dialog
 * @todo    2005-08-04 deborah lee soltesz
 *          RC ID not quite right -
 *          needs to be a drop-down box read
 *          from a "valids" file
 */
  void rcchanged( const QString & );

};


#endif



