#ifndef __SIMPLELINEDATA_H
#define __SIMPLELINEDATA_H

#include "vectordata.h"

/**
 * An extension of VectorData for the sole purpose of
 * masking out Point functions.  This remains abstract
 * and allows its child implementations to only implement
 * those functions that pertain to line data.
 */
class SimpleLineData : public VectorData {
public:
  SimpleLineData( ) : VectorData( LINE, NULL ) {
  }
  virtual ~SimpleLineData( ) {
  }

  virtual void delNode( int ) {
  }
  virtual void moveNode( Node &, int =-1 ) {
  }
  virtual void insertNode( Node &, int ) {
  }
  virtual void addNode( Node &, int =-1 ) {
  }

  virtual bool fromXML( QDomElement &elem ) =0;
  virtual bool toXML( QDomDocument &doc,
                      QDomElement &elem,
                      std::string id ) =0;

  virtual void addLine( Node &node ) =0;
  virtual void delLine( int iline ) =0;
  virtual void breakLine( int iline, int inode ) =0;
  virtual void joinLine( int il1, int il2, int in1, int in2 ) =0;
  virtual void addLineNode( Node &node, int iline, int inode = -1 ) =0;
  virtual void insertLineNode( Node &node, int iline, int inode = -1 ) =0;
  virtual void delLineNode( int iline, int inode ) =0;
  virtual void moveLineNode( int iline, int inode, Node &node ) =0;
  virtual void push( ) =0;
  virtual void pushSelection( ) =0;
  virtual void pop( ) =0;
  virtual void redo( ) =0;
};

#endif



