#ifndef __VECTORDATA_H
#define __VECTORDATA_H

#include <qobject.h>
#include <qprogressbar.h>
#include <qdialog.h>
#include <qlayout.h>

#include <vector>
#include <stack>
#include <iostream>
#include "shape.h"
#include "selectionset.h"
#include "image.h"

/**
 * An interface with all methods for point and line
 * data.  All vector data is derived from this allowing
 * easy handling of vector objects.
 */
class VectorData : public QObject, public XMLSerializeable {
  Q_OBJECT
public:
  typedef enum {
    POINT,
    LINE,
    POLY
  } ShapeType;

protected:
  std::vector<Shape*>  mLLSet;
  std::vector<Shape*>  mXYSet;
  SelectionSet         mSelSet;

  std::stack<std::vector<Shape*> >     mUndoLL;  // storage for previous states
                                                 // of the lat-lon shape data
                                                 // for undo facility
  std::stack<std::vector<Shape*> >     mUndoXY;  // storage for previous states
                                                 // of the X-Y shape data
                                                 // for undo facility
  std::stack<SelectionSet>             mUndoSel; // storage for previous states
                                                 // of selected vector data
                                                 // for undo facility

  std::stack<std::vector<Shape*> >     mRedoLL;  // storage for previous states
                                                 // of the lat-lon shape data
                                                 // for undo facility
  std::stack<std::vector<Shape*> >     mRedoXY;  // storage for previous states
                                                 // of the X-Y shape data
                                                 // for undo facility
  std::stack<SelectionSet>             mRedoSel; // storage for "undone" states
                                                 // of selected vector data
                                                 // for redo facility

  ShapeType            mType;  // type of shape
  Image                *mImg;

public:
  VectorData( ShapeType type, Image *img=NULL );
  virtual ~VectorData( ) {
  }

  virtual bool toXML( QDomDocument &doc,
                      QDomElement &elem,
                      std::string id ) =0;

  virtual bool fromXML( QDomElement &elem ) =0;

  virtual void addNode( Node &node, int inode =-1 ) =0;
  virtual void insertNode( Node &node, int inode ) =0;
  virtual void delNode( int inode ) =0;
  virtual void moveNode( Node &node, int inode = -1 ) =0;
  virtual void addLine( Node &node ) =0;
  virtual void delLine( int iline ) =0;
  virtual void breakLine( int iline, int inode ) =0;
  virtual void joinLine( int il1, int il2, int in1, int in2 ) =0;
  virtual void addLineNode( Node &node, int iline, int inode = -1 ) =0;
  virtual void insertLineNode( Node &node, int iline, int inode = -1 ) =0;
  virtual void delLineNode( int iline, int inode ) =0;
  virtual void moveLineNode( int iline, int inode, Node &node ) =0;
  virtual void push( ) =0;
  virtual void pop( ) =0;
  virtual void redo( ) =0;
  virtual void pushSelection( ) =0;


  void setImage( Image *img ) {
    mImg = img;
  }
  ShapeType getType( ) {
    return mType;
  }
  std::vector<Shape*>& getXYSet( ) {
    return mXYSet;
  }
  std::vector<Shape*>& getLLSet( ) {
    return mLLSet;
  }
  unsigned int size( ) {
    return mLLSet.size();
  }
  SelectionSet& getSelSet() {
    return mSelSet;
  }

public slots:
  void transformDataset( );

  signals:
  void datasetChanged( );

};

#endif



