#include "vectorlayer.h"

VectorLayer::VectorLayer( VectorData *data ) :
mData( data ),
mSelSet( data->getSelSet() ) {
  mVisible = true;
  mEditable = true;
}

void VectorLayer::draw( QPainter &p, Rect &imgclip, double scale ) {
  std::vector<Shape*> xyset = mData->getXYSet();

  DspAttr selAttr;
  selAttr.color = QColor( 255,255,0 );
  selAttr.style = Qt::SolidLine;
  selAttr.width = 1;

  if ( mData->getType() == VectorData::POINT ) {
    int nobj = (int)xyset.size();
    for ( int inode=0; inode < nobj; inode++ ) {
      Shape &s = *xyset[inode];
      Node node = dynamic_cast<Node&>( s );
      node.x = (node.x - imgclip.x) * scale;
      node.y = (node.y - imgclip.y) * scale;

      if ( mSelSet.isSelObj( inode ) ) {
        p.setPen( QPen( selAttr.color,
                        selAttr.width,
                        selAttr.style ) );
        p.setBrush( selAttr.color );
      } else {
        p.setPen( QPen( mLyrAttr.color, mLyrAttr.width, mLyrAttr.style ) );
        p.setBrush( mLyrAttr.color );
      }
      /*
            int halfw = mLyrAttr.width / 2;
            p.drawEllipse( (int)node.x - halfw,
                           (int)node.y - halfw,
                           (int)mLyrAttr.width,
                           (int)mLyrAttr.width );
      */
      p.drawEllipse( (int)node.x-2, (int)node.y-2, 5, 5 );
    }
  } else {
    for ( int iobj=0; iobj < (int)xyset.size(); iobj++ ) {
      Shape &s    = *xyset[iobj];
      Line &l     = dynamic_cast<Line&>( s );
      int nnodes  = (int)l.size();

      DspAttr &attr = l.getAttr();

      if ( mSelSet.isSelObj( iobj ) ) {
        Node n = l[0];
        n.x = (n.x - imgclip.x) * scale;
        n.y = (n.y - imgclip.y) * scale;

        if ( mSelSet.isSelNode( iobj, 0 ) ) {
          p.setPen( QPen( selAttr.color, selAttr.width, selAttr.style ) );
          p.setBrush( selAttr.color );
        } else {
          p.setPen( QPen( attr.color, attr.width, attr.style ) );
          p.setBrush( Qt::NoBrush );
        }
        p.drawEllipse( (int)n.x-2, (int)n.y-2, 5, 5 );

        for ( int inode=1; inode < nnodes; inode++ ) {
          Node pn = l[inode-1];
          Node n  = l[inode];
          n.x = (n.x - imgclip.x) * scale;
          n.y = (n.y - imgclip.y) * scale;
          pn.x = (pn.x - imgclip.x) * scale;
          pn.y = (pn.y - imgclip.y) * scale;

          if ( mSelSet.isSelNode( iobj, inode ) ) {
            p.setPen( QPen( selAttr.color, selAttr.width, selAttr.style ) );
            p.setBrush( selAttr.color );
          } else {
            p.setPen( QPen( attr.color, attr.width, attr.style ) );
            p.setBrush( Qt::NoBrush );
          }
          p.drawEllipse( (int)n.x-2, (int)n.y-2, 5, 5 );

          p.setPen( QPen( selAttr.color, selAttr.width, selAttr.style ) );
          p.drawLine( (int)pn.x, (int)pn.y, (int)n.x, (int)n.y );
        }
      } else {
        p.setPen( QPen( attr.color, attr.width, attr.style ) );
        for ( int inode=1; inode < nnodes; inode++ ) {
          Node pn = l[inode-1];
          Node n  = l[inode];
          n.x = (n.x - imgclip.x) * scale;
          n.y = (n.y - imgclip.y) * scale;
          pn.x = (pn.x - imgclip.x) * scale;
          pn.y = (pn.y - imgclip.y) * scale;
          p.drawLine( (int)pn.x, (int)pn.y, (int)n.x, (int)n.y );
        }
      }
    }
  }
}


