#ifndef __VIEWSTATE_H
#define __VIEWSTATE_H

#include "zoomattr.h"
#include "shape.h"
#include "xmlserializeable.h"

/**
 * A set of variables associated with views that
 * pertain to the each views current state.  These
 * are modified continuously as the user makes change
 * to the views condition.
 */
class ViewState : public XMLSerializeable {
public:
  bool          mEditable;
  ZoomAttr      mZoom;
  std::string   mName;
  std::string   mChildName;
  std::string   mParentName;
  Rect          mGeom;

public:
  ViewState( ) {
    mZoom.factor  = 1.0;
    mZoom.type    = ZoomAttr::WHOLE;
    mZoom.min   = -99999999;
    mZoom.max   = 99999999;
    mEditable   = true;
  }


  virtual bool toXML(QDomDocument &doc,
                     QDomElement &elem,
                     std::string id = "" ) {
    QDomElement base =
    createBaseTag( doc, elem, id, "ViewState" );

    writeIntTag( doc, base, "Editable", mEditable );
    writeTextTag( doc, base, "Name", mName );
    writeTextTag( doc, base, "ChildName", mChildName );
    writeTextTag( doc, base, "ParentName", mParentName );
    mZoom.toXML( doc, base );
    mGeom.toXML( doc, base );
    return true;
  }

  virtual bool fromXML( QDomElement &elem ) {
    QDomNodeList nodelist;
    QDomElement  e;
    if ( std::string( "ViewState" ).compare( (const char*)elem.tagName() ) )
      return false;

    mEditable =     readIntTag( elem, "Editable" );
    mName =         readTextTag( elem, "Name" );
    mChildName =    readTextTag( elem, "ChildName" );
    mParentName =   readTextTag( elem, "ParentName" );

    nodelist = elem.elementsByTagName( "ZoomAttr" );
    if ( nodelist.count() >= 1 ) {
      e = nodelist.item(0).toElement();
      mZoom.fromXML( e );
    }

    nodelist = elem.elementsByTagName( "Rect" );
    if ( nodelist.count() >= 1 ) {
      e = nodelist.item(0).toElement();
      mGeom.fromXML( e );
    }

    return true;
  }

};

#endif





