#ifndef __ZOOMATTR_H
#define __ZOOMATTR_H

#include "xmlserializeable.h"

/**
 * A set of variables associated with the current
 * state of a view's zoom.  A view may be FIXED so that
 * no change of the zoom is possible by the user.  Or, it
 * may be WHOLE so that at any given moment the entire image
 * is displayed.  SCALE and FREE do the same thing right now
 * where the zoom is always a divisible number such as .25 or 2.
 * This allows less distortion in the image.  FREE should be any
 * zoom any time with the aid of a zoom select box.
 */
class ZoomAttr : public XMLSerializeable {
public:
  typedef enum {
    FIXED,
    SCALE,
    FREE,
    WHOLE
  } ZoomType;

  double      factor;
  ZoomType    type;
  double      min;
  double      max;

  virtual bool toXML( QDomDocument &doc,
                      QDomElement &elem,
                      std::string id = "" ) {
    QDomElement base;
    base = createBaseTag( doc, elem, id, "ZoomAttr" );

    writeDoubleTag( doc, base, "Factor", factor );
    writeIntTag( doc, base, "Type", type );
    writeDoubleTag( doc, base, "Min", min );
    writeDoubleTag( doc, base, "Max", max );
    return true;
  }

  virtual bool fromXML( QDomElement &elem ) {
    if ( std::string( "ZoomAttr" ).compare( (const char*)elem.tagName() ) )
      return false;

    factor = readDoubleTag( elem, "Factor" );
    type   = (ZoomType)readIntTag( elem, "Type" );
    min    = readDoubleTag( elem, "Min" );
    max    = readDoubleTag( elem, "Max" );
    return true;
  }
};

#endif




